/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

package ojb.broker;
import java.lang.reflect.Array;

/**
 * this Iterator is used to iterate over Arrays.
 * Used in PersistenceBroker::storeCollections() and deleteCollections()
 * if arrays are used instead of Collections
 * @author: Thomas Mahler
 */
public class ArrayIterator implements java.util.Iterator
{

	/**
	 * the underlying array
	 */
	private Object myArray;

	/**
	 * current position of the Iterator
	 */
	private int position;

	/**
	 * total length of myArray
	 */
	private int length;

	/**
	 * create an Iterator for the Array array.
	 * @throws UnsupportedOperationException if array is not an Array
	 * @param array java.lang.Object
	 */
	public ArrayIterator(Object array)
	{
		if (!array.getClass().isArray())
		{
			throw new UnsupportedOperationException("ArrayIterator must be initialized with an Array to iterate over");
		}
		myArray = array;
		position = 0;
		length = Array.getLength(myArray);
	}

	/**
	 * Returns <tt>true</tt> if the iteration has more elements. (In other
	 * words, returns <tt>true</tt> if <tt>next</tt> would return an element
	 * rather than throwing an exception.)
	 * 
	 * @return <tt>true</tt> if the iterator has more elements.
	 */
	public boolean hasNext()
	{
		return (position < length);
	}

	/**
	 * Returns the next element in the interation.
	 * 
	 * @returns the next element in the interation.
	 * @exception NoSuchElementException iteration has no more elements.
	 */
	public synchronized Object next()
	{
		Object result = Array.get(myArray, position);
		// System.out.println("ArrayIterator.next: " + result);
		position++;
		return result;
	}

	/**
	 * 
	 * Removes from the underlying collection the last element returned by the
	 * iterator (optional operation).  This method can be called only once per
	 * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
	 * the underlying collection is modified while the iteration is in
	 * progress in any way other than by calling this method.
	 * 
	 * @exception UnsupportedOperationException if the <tt>remove</tt>
	 * operation is not supported by this Iterator.
	 * 
	 * @exception IllegalStateException if the <tt>next</tt> method has not
	 * yet been called, or the <tt>remove</tt> method has already
	 * been called after the last call to the <tt>next</tt>
	 * method.
	 */
	public void remove()
	{
		throw new UnsupportedOperationException("I'm sorry Dave...");
	}

}