package ojb.broker;

import java.util.Properties;
import java.io.*;
import java.util.StringTokenizer;

/** 
 * Configuration Base Class that
 * keeps a Properties based configuration persistent in a file.
 * This class provides only basic infrastructure for loading etc.
 */
public class BaseConfiguration
{
	/**the name of the properties file*/
	protected String filename;
	/**the properties object holding the configuration data*/
	protected Properties properties;

    /**
      * The constructor loads the configuration from file
      */
    public BaseConfiguration()
    {
    	load();
    }

/** gibt den unter einem Schlssel abgelegten Wert aus der Konfiguration zurck
  * falls kein Wert gefunden wird, wird ein default Wert angenommen und abgelegt
  * @param key der Schlssel
  * @param defaultValue der Default Wert
  */
protected String getProperty(String key, String defaultValue)
{
	properties.put(key, properties.getProperty(key, defaultValue));
	return properties.getProperty(key);
}
/**ldt die Konfiguration aus der Properties Datei*/
protected void load()
{
	properties = new Properties();
	try
	{
		FileInputStream strIn = new FileInputStream(getFilename());
		properties.load(strIn);
		strIn.close();
	}
	catch (Exception ignore)
	{
	}
}
/** zerlegt einen String anhand eines Trennzeichens in ein Array von Teilstrings*/
protected String[] parseSeparatedString(String str, String separator)
{
	StringTokenizer st = new StringTokenizer(str, separator);
	String[] ret = new String[st.countTokens()];
	for (int i = 0; i < ret.length; i++)
	{
		ret[i] = new String(st.nextToken());
	}
	return ret;
}
/** speichert die Konfiguration*/
protected void save(String header)
{
    try {
	  FileOutputStream strOut = new FileOutputStream(getFilename());
	  properties.store(strOut, header);
	  strOut.close();
	}
	catch (Exception ignore) {}
  }

    private String getFilename()
    {
        if (filename == null)
            filename = this.getClass().getName() + ".properties";
        return filename;
    }
}
