/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker;
import java.util.Iterator;

/**
 * 
 * OJB can handle java.util.Collection as well as user defined collection classes as collection attributes
 * 
 * in persistent classes. In order to collaborate with the OJB mechanisms these collection must provide a minimum
 * 
 * protocol as defined by this interface ManageableCollection.
 * 
 * The methods have a prefix "ojb" that indicates that these methods are "technical" methods, required
 * 
 * by OJB and not to be used in business code.
 * 
 * 
 * 
 * @author: Thomas Mahler
 * 
 */
public interface ManageableCollection extends java.io.Serializable
{

	/**
	 * add a single Object to the Collection. This method is used during reading Collection elements
	 * 
	 * from the database. Thus it is is save to cast anObject to the underlying element type of the
	 * 
	 * collection.
	 * 
	 */
	void ojbAdd(Object anObject);

	/**
	 * adds a Collection to this collection. Used in reading Extents from the Database.
	 * 
	 * Thus it is save to cast otherCollection to this.getClass().
	 * 
	 */
	void ojbAddAll(ManageableCollection otherCollection);

	/**
	 * returns an Iterator over all elements in the collection. Used during store and delete Operations.
	 * 
	 * If the implementor does not return an itertaor over ALL elements, OJB cannot store and delete all elements properly.
	 * 
	 */
	Iterator ojbIterator();
}