/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.net.MalformedURLException;
import javax.xml.parsers.ParserConfigurationException;
import ojb.broker.metadata.DescriptorRepository;
import ojb.broker.server.*;


/**
 * 
 * Factory class that produces PersistenceBroker instances
 * 
 */
public class PersistenceBrokerFactory
{

    private static DescriptorRepository repository = null;

    private static PersistenceBroker lastServed;

    private static PersistenceBrokerFactoryConfiguration configuration = new PersistenceBrokerFactoryConfiguration();

	/**
	 * Method declaration
	 * 
	 * 
	 * @param repositoryFile
	 * 
	 * @return
	 * 
	 * @throws IOException
	 * @throws MalformedURLException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * 
	 * @see
	 */
	public static PersistenceBroker createPersistenceBroker(String repositoryFile) 
			throws MalformedURLException, ParserConfigurationException, SAXException, IOException
	{
	    if (repository == null)
	        repository = DescriptorRepository.getInstance(repositoryFile);

	    if (configuration.isServerUsed())
	    {
	        ServerPool pool = new ServerPool(configuration.getServers());

	        lastServed = new PersistenceBrokerClient(pool);
		    return lastServed;
        }
        else
        {
            lastServed = new PersistenceBrokerImpl(repository);
            return lastServed;
        }
	}


	/**
	 * only for internal use, e.g in VirtualProxy
	 *
	 *
	 */
	public static PersistenceBroker createPersistenceBroker()
	{
	    if (lastServed != null) return lastServed;
	    else
	    {
            try
            {
                return createPersistenceBroker(configuration.getRepositoryFilename());
            }
            catch(Exception e)
            {
                System.out.println(e.getMessage());
                e.printStackTrace();
                System.exit(1);
                return null;
            }
        }

	}


}