/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker;
import java.util.Hashtable;
import ojb.broker.metadata.*;
import ojb.broker.query.*;
import java.sql.Statement;
import java.sql.ResultSet;
import ojb.broker.accesslayer.StatementManager;

/**
 * 
 * A simple SequenceManager. This class is responsible for creating new unique ID's for primary columns
 * 
 * containing integer values.
 * 
 * The SequenceManager is aware of extends, that is: if you ask for an uid for an Interface with several implementor classes,
 * 
 * or a baseclass with several subclasses the returned uid will unique accross all tables representing
 * 
 * objects of the extent in question.
 * 
 * @author: thma
 * 
 */
public class SequenceManager
{

	/**
	 * reference to the PersistenceBroker
	 */
	private PersistenceBrokerImpl broker;

	/**
	 * singleton instance of the SequenceManager
	 */
	private static SequenceManager _instance;

	/**
	 * 
	 * Public constructor
	 * 
	 */
	public SequenceManager(PersistenceBrokerImpl broker)
	{
		this.broker = broker;
	}



	/**
	 * lookup current maximum value for fieldName in table cld.getTableName()
	 */
	private int getMaxIdForClass(ClassDescriptor cld, String fieldName)
	{
		try
		{
			String table = cld.getTableName();
			String schema = cld.getScheme();
			if (schema != null)
			{
				table = schema + "." + table;
			}
			String column = cld.getFieldDescriptorByName(fieldName).getColumnName();
			String sql = "SELECT MAX(" + column + ") FROM " + table;
			Statement stmt = StatementManager.getGenericStatement(cld, broker);
			ResultSet rs = stmt.executeQuery(sql);
			rs.next();
			int result = rs.getInt(1);
			rs.close();
			stmt.close();
			return result;
		}
		catch (java.sql.SQLException ex)
		{
			return 0;
		}
	}

	/**
	 */
	private synchronized int getNextId(Class clazz, String fieldName)
	{
		try
		{
			// check if there is an entry for this class
			SequenceEntry tmp = new SequenceEntry();
			tmp.setClassname(clazz.getName());
			tmp.setFieldname(fieldName);
			SequenceEntry entry = (SequenceEntry) broker.getObjectByQuery(new QueryByExample(tmp));
			// if no entry found, build a new one first:
			if (entry == null)
			{
				int lastId = getMaxForExtent(clazz, fieldName);
				entry = new SequenceEntry(clazz.getName(), fieldName, lastId);
			}
			// compute next uid, update the entry, store it and return
			int nextId = entry.getCurrent() + 1;
			entry.setCurrent(nextId);
			broker.store(entry);
			// System.out.println("UID (" + clazz + ", " + fieldName + ") : " + nextId);
			return nextId;
		}
		catch (PersistenceBrokerException ex)
		{
			throw new RuntimeException(ex.getMessage());
		}
	}

	/**
	 * returns a unique int for class clazz and field fieldName.
	 * 
	 * the returned uid is unique accross all tables in the extent of clazz.
	 * 
	 */
	public int getUniqueId(Class clazz, String fieldName)
	{
		return getNextId(clazz, fieldName);
	}

	/**
	 * lookup all tables in extent clazz to find the current maximum value for fieldName
	 */
	private int getMaxForExtent(Class clazz, 
								String fieldName) throws ClassNotPersistenceCapableException
	{
		int max = 0;
		ClassDescriptor cld = DescriptorRepository.getInstance().getDescriptorFor(clazz);
		// if class is no Interface we have to search its directly mapped table
		if (!cld.isInterface())
		{
			int tmp = getMaxIdForClass(cld, fieldName);
			if (tmp > max)
			{
				max = tmp;
			}
		}
		// if class is an extent we have to search through its subclasses
		if (cld.isExtent())
		{
			java.util.Vector extentClasses = cld.getExtentClasses();
			for (int i = 0; i < extentClasses.size(); i++)
			{
				Class ec = (Class) extentClasses.get(i);
				cld = DescriptorRepository.getInstance().getDescriptorFor(ec);
				int tmp = getMaxIdForClass(cld, fieldName);
				if (tmp > max)
				{
					max = tmp;
				}
			}
		}
		return max;
	}

}