/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker;
import ojb.broker.PersistenceBroker;
import ojb.broker.PersistenceBrokerFactory;
import ojb.broker.Identity;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/**
 * 
 * Proxy base class. can be used to implement lazy materialization techniques.
 * 
 * @see: ojb.examples.proxy.App as a sample application.
 * 
 * @author: Thomas Mahler
 * 
 */
public class VirtualProxy implements java.io.Serializable
{

	/**
	 * reference to the used PersistenceBroker
	 */
	private PersistenceBroker broker = null;

	/**
	 * the real subject which this is hidden by the proxy
	 */
	private Object realSubject = null;

	/**
	 * represents the identity of the real subject. when the real subject is not yet materialized,
	 * 
	 * it can be loaded from the underlying db by this identity object.
	 * 
	 */
	private Identity id = null;

	/**
	 * 
	 * Public constructor
	 * 
	 */
	public VirtualProxy()
	{
		super();
	}

	/**
	 * 
	 * create a VirtualProxy from an identity
	 * 
	 * @param id ojb.broker.Identity
	 * 
	 */
	public VirtualProxy(Identity oid, PersistenceBroker broker)
	{
		this.id = oid;
        try
        {
            this.broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml"); //broker;
        }
        catch(IOException e)
        {
        }
        catch(ParserConfigurationException e)
        {
        }
        catch(SAXException e)
        {
        }
    }

	/**
	 * returns true if the real subject already been loaded
	 */
	public boolean alreadyMaterialized()
	{
		return realSubject != null;
	}

	/**
	 * 
	 * static factory method creates VirtualProxy instances of a given subclass and an real subjects identity.
	 * 
	 * @return ojb.app.VirtualProxy
	 * 
	 * @param proxyClass java.lang.Class
	 * 
	 * @param realSubjectsIdentity ojb.broker.Identity
	 * 
	 */
	public static VirtualProxy createProxy(Class proxyClass, Identity realSubjectsIdentity)
			throws InstantiationException, IllegalAccessException
	{
		VirtualProxy proxy = (VirtualProxy) proxyClass.newInstance();
		proxy.setId(realSubjectsIdentity);
		proxy.broker = PersistenceBrokerFactory.createPersistenceBroker();
		return proxy;
	}



	/**
	 * 
	 * returns the proxies real subject. The subject will be materialized if necessary.
	 * 
	 * Creation date: (08.11.2000 21:42:40)
	 * 
	 */
	public Object getRealSubject() throws PersistenceBrokerException
	{
		if (realSubject == null)
		{
			materializeSubject();
		}
		return realSubject;
	}

	/**
	 * 
	 * retrieve the real subject from the underlying RDBMS
	 * 
	 */
	private void materializeSubject() throws PersistenceBrokerException
	{
		realSubject = broker.getObjectByIdentity(id);
	}

	/**
	 * 
	 * sets the Identity of the proxy. Is used by the createProxy() factory method.
	 * 
	 * @param newId ojb.broker.Identity
	 * 
	 */
	protected void setId(Identity newId)
	{
		id = newId;
	}

	/**
	 * 
	 * returns the proxies real subject. The subject will be materialized if necessary.
	 * 
	 * Creation date: (08.11.2000 21:42:40)
	 * 
	 */
	protected void setRealSubject(Object theRealSubject)
	{
		realSubject = theRealSubject;
	}

	/**
	 * 
	 * return a String representation.
	 * 
	 * @return java.lang.String
	 * 
	 */
	public String toString()
	{
		if (realSubject == null)
		{
			return "" + id;
		}
		else
		{
			return realSubject.toString();
		}
	}

}