/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.accesslayer;
import java.util.Hashtable;
import java.util.Iterator;
import ojb.broker.metadata.*;
import ojb.broker.PersistenceBrokerImpl;
import java.util.*;
import java.sql.*;

/**
 * manages JDBC Connection ressources.
 * @author: Thomas Mahler
 */
public class ConnectionManager
{

    private PersistenceBrokerImpl broker = null;
	/**
	 * constructor is private: use the static methods only
	 */
	public ConnectionManager(PersistenceBrokerImpl broker)
	{
	    this.broker = broker;
		connectionTable = new Hashtable();
	}

	/**
	 * internal table of StatementForClass objects
	 */
	private Hashtable connectionTable;

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Enumeration enumeration()
	{
		return connectionTable.elements();
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param cld
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Connection getConnectionForClassDescriptor(ClassDescriptor cld)
	{
		// multiple classes can share the same JBDC Connection Descriptor,
		// so we must lookup the Connection with the jcd as key
		JdbcConnectionDescriptor jcd = cld.getConnectionDescriptor();
		Connection conn = (Connection) connectionTable.get(jcd);
		if (conn == null)
		{
			conn = getNewConnection(jcd);
			connectionTable.put(jcd, conn);
		}
		return conn;
	}


	/**
	 * returns a newly allocated Connection. Information to construct the Connection
	 * is taken from the JdbcConnectionDecriptor jcd.
	 */
	private Connection getNewConnection(JdbcConnectionDescriptor jcd)
	{
		String driver = jcd.getDriver();
		String url = jcd.getProtocol() + ":" + jcd.getSubProtocol() + ":" + jcd.getDbAlias();
		String user = jcd.getUserName();
		String pass = jcd.getPassWord();
		if (user == null)
		{
			user = "";
		}
		if (pass == null)
		{
			pass = "";
		}
		Connection con = null;
		try
		{
			Class.forName(driver);    // loads the driver
			con = java.sql.DriverManager.getConnection(url, user, pass);
			con.setAutoCommit(!broker.isInTransaction());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		return con;
	}

}