/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.accesslayer;
import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import java.sql.ResultSet;
import ojb.broker.metadata.ClassDescriptor;
import java.sql.Connection;
import ojb.broker.PersistenceBroker;
import ojb.broker.PersistenceBrokerImpl;
import ojb.broker.PersistenceBrokerFactory;
import ojb.broker.Identity;
import java.util.Vector;
import java.util.NoSuchElementException;
import java.util.Enumeration;
import ojb.broker.metadata.FieldDescriptor;
import ojb.broker.query.*;
import java.util.Iterator;
import java.io.Serializable;

/**
 * 
 * can be used to create enumerations of PrimaryKey objects.
 * 
 * This is especially interesting for EJB finder methods
 * 
 * in BMP entity beans which must return such enumerations.
 * 
 * 
 * 
 * NOTE: this code uses features that only JDBC 2.0 compliant Drivers
 * 
 * support. It will NOT work with JDBC 1.0 Drivers (e.g. SUN's JdbcOdbcDriver)
 * 
 * If you are forced to use such a driver, you can use code from
 * 
 * the 0.1.30 release.
 * 
 * 
 * 
 * @author: Thomas Mahler
 * 
 */
public class PkEnumeration implements Enumeration, Serializable
{

	/**
	 * The underlying jdbc resultset produced by select statement
	 */
	protected ResultSet resultSet;

	/**
	 * descriptor for the class of which items are to be found
	 */
	protected ClassDescriptor classDescriptor;

	/**
	 * the Constructor that is needed to build the PrimaryKey Objects
	 */
	protected Constructor constructor;

	/**
	 * 
	 * PkEnumeration constructor.
	 * 
	 * @param qry the SELECT statement gerating the underlying resultset
	 * 
	 * @param mif classDescriptor of the target entity class (say Article)
	 * 
	 * @param primaryKeyClass the entity classes PrimaryKey class (say ArticleKey).
	 * 
	 * this key-class MUST have a constructor with one argument of type ojb.broker.Identity !
	 * 
	 */
	public PkEnumeration(Query query, ClassDescriptor mif, Class primaryKeyClass, PersistenceBrokerImpl broker)
	{
		ResultSet rs = new JdbcAccess(broker).executeQuery(query, mif);
		this.resultSet = rs;
		this.classDescriptor = mif;
		// get a contructor object that can be used to build instances of class primaryKeyClass
		try
		{
			Class[] argArray = 
			{
				ojb.broker.Identity.class
			};
			this.constructor = primaryKeyClass.getConstructor(argArray);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			System.out.println(primaryKeyClass.getName() 
							   + " must implement a Constructor with one argument of type ojb.broker.Identity");
		}
	}

	/**
	 * returns an Identity object representing the current resultset row
	 */
	private Identity getIdentityFromResultSet()
	{
		try
		{
			// 1. get an empty instance of the target class
			Class c = classDescriptor.getClassOfObject();
			Object obj = c.newInstance();
			// 2. fill only primary key values from Resultset
			Object colValue;
			FieldDescriptor fld;
			FieldDescriptor[] pkfields = classDescriptor.getPkFields();
			for (int i = 0; i < pkfields.length; i++)
			{
				fld = pkfields[i];
				colValue = resultSet.getObject(fld.getColumnName());
				fld.getPersistentField().set(obj, colValue);
			}
			// 3. return the representing identity object
			return new Identity(c, classDescriptor.getKeyValues(obj));
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}
		return null;
	}

	/**
	 * 
	 * Tests if this enumeration contains more elements.
	 * 
	 * 
	 * 
	 * @return  <code>true</code> if and only if this enumeration object
	 * 
	 * contains at least one more element to provide;
	 * 
	 * <code>false</code> otherwise.
	 * 
	 */
	public boolean hasMoreElements()
	{
		boolean end = false;
		try
		{
			end = !resultSet.isLast();
			// if ResultSet is exhausted do final cleanup to free JDBC resources
			if (!end)
			{
			    resultSet.getStatement().close();
				resultSet.close();
				resultSet = null;
			}
			return end;
		}
		catch (Exception ex)
		{
			return false;
		}
	}

	/**
	 * 
	 * Returns the next element of this enumeration if this enumeration
	 * 
	 * object has at least one more element to provide.
	 * 
	 * 
	 * 
	 * @return     the next element of this enumeration.
	 * 
	 * @exception  NoSuchElementException  if no more elements exist.
	 * 
	 */
	public Object nextElement()
	{
		try
		{
			resultSet.next();
			Identity oid = getIdentityFromResultSet();
			Identity[] args = 
			{
				oid
			};
			return this.constructor.newInstance(args);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
			throw new NoSuchElementException();
		}
	}

}