/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.accesslayer;
import java.util.Iterator;
import java.sql.ResultSet;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.Field;
import java.sql.Connection;
import ojb.broker.metadata.*;
import ojb.broker.*;
import ojb.broker.query.*;

/**
 * 
 * ProxyRsIterator can be used to iterate over a jdbc ResultSet to retrieve
 * 
 * Proxy-objects rather than real-object.
 * 
 * This Iterator is used to build lazy Collections by the PersistenceBroker.
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class ProxyRsIterator extends RsIterator
{

	/**
	 * the proxy-class of which shall be used for instanciation
	 */
	Class itemProxyClass = null;

	/**
	 * 
	 * ProxyRsIterator constructor.
	 * 
	 * @param qry the SELECT producing the underlying resultset
	 * 
	 * @param mif ojb.broker.metadata.ClassDescriptor
	 * 
	 * @param proxyClass Class
	 * 
	 */
	public ProxyRsIterator(Query query, ojb.broker.metadata.ClassDescriptor mif, Class proxyClass, PersistenceBrokerImpl broker)
	{
		super(query, mif, broker);
		itemProxyClass = proxyClass;
	}

	/**
	 * Reads primary key information from current RS row and generates a
	 * 
	 * corresponding Identity, and returns a proxy from the Identity.
	 * 
	 */
	private Object getProxyFromResultSet()
	{
		try
		{
			// 1. get Identity of current row:
			Identity oid = getIdentityFromResultSet();
			// 2. return a Proxy instance:
			return VirtualProxy.createProxy(itemProxyClass, oid);
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}
		return null;
	}

	/**
	 * 
	 * moves to the next row of the underlying ResultSet and
	 * 
	 * returns a corresponding proxy.
	 * 
	 */
public synchronized Object next() throws NoSuchElementException {
	try
	{
	    if (! hasCalledCheck) hasNext();
	    hasCalledCheck = false;
	    if (hasNext)
	    {
		Object obj = getProxyFromResultSet();
		if (_debug)
			System.out.println("next() -> " + obj);
		return obj;
        }
        else throw new NoSuchElementException();
	}
	catch (Exception ex)
	{
		if (_debug)
			System.out.println("next() -> ERROR");
		ex.printStackTrace();
		throw new NoSuchElementException();
	}
}

}