/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.accesslayer;
import java.util.Vector;
import java.util.Iterator;
import java.lang.reflect.Field;
import ojb.broker.metadata.*;
import java.util.Enumeration;
import ojb.broker.query.*;
import java.util.List;
import ojb.broker.ObjectModification;

/**
 * 
 * This Class is responsible for building sql statements
 * 
 * Objects fields and their repective values are accessed by Java reflection
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class SqlGenerator
{

	/**
	 * Constructor is private, use getInstance() to retrieve the singleton instance
	 * 
	 * of this class.
	 * 
	 */
	private SqlGenerator() {}

	/**
	 * singleton instance
	 */
	private static SqlGenerator _instance;

	/**
	 * this flag en/disables tracing
	 */
	private static boolean _debug = false;

	/**
	 * returns the singleton instance
	 */
	public static SqlGenerator getInstance()
	{
		if (_instance == null)
		{
			_instance = new SqlGenerator();
		}
		return _instance;
	}

	/**
	 * en/disables tracing
	 */
	public static void setDebug(boolean b)
	{
		_debug = b;
	}

	/**
	 * returns a comma separated list of column names as a single String
	 * 
	 * @param mif provides information on table and columns
	 * 
	 */
	private String getListOfColumns(ClassDescriptor mif)
	{
		String list = null;
		FieldDescriptor[] fields = mif.getFieldDescriptions();
		for (int i = 0; i < fields.length; i++)
		{
			FieldDescriptor fmd = fields[i];
			String cn = fmd.getColumnName();
			if (list == null)
			{
				list = quote(cn);
			}
			else
			{
				list += (", " + quote(cn));
			}
		}
		return list;
	}

	/**
	 * returns a String that embraces the String col with double quotes ("), currently disabled.
	 */
	private String quote(String col)
	{
		// return "\"" + col + "\"";
		return col;
	}

	/**
	 * applies single quotes to DATE, TIME, CHAR, or STRING columns
	 */
	private String quoteIfNeeded(String str, String dbtype)
	{
		String[] quotables = 
		{
			"DATE", "TIME", "CHAR", "STRING"
		};
		boolean test = false;
		for (int i = 0; i < quotables.length; i++)
		{
			if (dbtype.startsWith(quotables[i]))
			{
				test = true;
				break;
			}
		}
		if (test)
		{
			str = escapeSingleQuotes(str);
			return "'" + str + "'";
		}
		else
		{
			return str;
		}
	}

	/**
	 * replace single quotes (') by an escape sequence (\')
	 * 
	 */
	private String escapeSingleQuotes(String str)
	{
		String result = str;
		if (str.indexOf("'") != -1)
		{
			StringBuffer buf = new StringBuffer();
			char c;
			for (int i = 0; i < str.length(); i++)
			{
				c = str.charAt(i);
				if (c == '\'')
				{
					buf.append("\\'");
				}
				else
				{
					buf.append(c);
				}
			}
			result = buf.toString();
		}
		return result;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * @param cld
	 * 
	 * @return
	 * 
	 * @see
	 */
	protected String asSQLClause(SelectionCriteria c, ClassDescriptor cld)
	{
	String colName;
	if (cld != null)
	{
			colName = cld.getFieldDescriptorByName(c.getAttribute()).getColumnName();
	}
	else
	{
		colName = c.getAttribute();
	}
		return colName + c.getClause() + " ? ";
	}

	/**
	 * @param criteria Selection criteria
	 * 
	 * 26/06/99 Change statement to a StringBuffer for efficiency
	 */
	public String asSQLStatement(Criteria crit, ClassDescriptor cld)
	{
		Enumeration e = crit.getElements();
		StringBuffer statement = new StringBuffer();
		while (e.hasMoreElements())
		{
			Object o = e.nextElement();
			if (o instanceof Criteria)
			{
				String addAtStart;
				String addAtEnd;
				Criteria pc = (Criteria) o;
				// need to add parenthesises?
				if (pc.isEmbraced())
				{
					addAtStart = " (";
					addAtEnd = ") ";
				}
				else
				{
					addAtStart = "";
					addAtEnd = "";
				}



                switch (pc.getType())
                {
                    case (Criteria.OR):
                    {
                        statement.append( " OR " + addAtStart);
                        statement.append(asSQLStatement(pc, cld));
                        statement.append(addAtEnd);
                        break;
                    }
                    case (Criteria.AND):
                    {
                        statement.insert(0,"( ");
                        statement.append( ") ");
                        statement.append(" AND "+ addAtStart);
                        statement.append(asSQLStatement(pc, cld));
                        statement.append(addAtEnd);
                        break;
                    }
                }
   			}
			else
			{
			    SelectionCriteria c = (SelectionCriteria) o;
                if (statement.length() == 0)
                {
                    statement.append(asSQLClause(c, cld));
                }
                else
                {
                    statement.insert(0,"(");
                    statement.append(") ");
                    statement.append(" AND ");
                    statement.append(asSQLClause(c, cld));
                }
			}
		}    // while
		if (statement.length() == 0)
		{
			return null;
		}
		return statement.toString();
	}

	/**
	 * returns a comma separated list of ? parameters as a single String.
	 * 
	 * Used to build prepared statements.
	 * 
	 * @param mif provides information on table and columns
	 * 
	 */
	private String getListOfValues(ClassDescriptor mif)
	{
		String list = "";
		FieldDescriptor[] fields = mif.getFieldDescriptions();
		for (int i = 0; i < fields.length; i++)
		{
			FieldDescriptor fmd = fields[i];
			Field f = fmd.getPersistentField();
			if (list != "")
			{
				list += ", ";
			}
			list += "?";
		}
		return list;
	}

	/**
	 * 
	 * generate a prepared DELETE-Statement for the Class
	 * 
	 * described by mif.
	 * 
	 * @param mif the ClassDescriptor
	 * 
	 */
	public String getPreparedDeleteStatement(ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String stmt = "DELETE ";
		stmt += " FROM " + tbl;
		stmt += getWhereClause(mif);
		return stmt;
	}

	/**
	 * 
	 * generate a prepared INSERT-Statement for the Class
	 * 
	 * described by mif.
	 * 
	 * @param mif the ClassDescriptor
	 * 
	 */
	public String getPreparedInsertStatement(ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String qry = "INSERT INTO " + tbl;
		qry += " (" + getListOfColumns(mif) + ")";
		qry += " VALUES (" + getListOfValues(mif) + ")";
		return qry;
	}

	/**
	 * 
	 * generate a prepared SELECT-Statement for the Class
	 * 
	 * described by mif
	 * 
	 * @param mif the ClassDescriptor
	 * 
	 */
	public String getPreparedSelectByPkStatement(ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String stmt = "SELECT ";
		stmt += getListOfColumns(mif);
		stmt += " FROM " + tbl;
		stmt += getWhereClause(mif);
		return stmt;
	}

	/**
	 * 
	 * generate a select-Statement according to the constraints given by the values
	 * 
	 * in vecValues that correspond to the fields in vecFields
	 * 
	 */
	public String getPreparedSelectStatement(Query query, ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String stmt = "SELECT ";
		stmt += getListOfColumns(mif);
		stmt += " FROM " + tbl;
		stmt += getWhereClause(query, mif);
		String orderby = query.getCriteria().getOrderby();
		if (orderby != null)
		{
			stmt += " ORDER BY " + orderby;
		}
		return stmt;
	}

	/**
	 * 
	 * generate a prepared UPDATE-Statement for the Class
	 * 
	 * described by mif
	 * 
	 * @param mif the ClassDescriptor
	 * 
	 */
	public String getPreparedUpdateStatement(ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String stmt = "UPDATE " + tbl;
		stmt += getSetClause(mif);
		stmt += getWhereClause(mif);
		return stmt;
	}

	/**
	 * 
	 * generate a select-Statement according to the constraints given by the values
	 * 
	 * in vecValues that correspond to the fields in vecFields
	 * 
	 */
	public String getSelectStatementDep(Query query, ClassDescriptor mif)
	{
		String scm = mif.getScheme();
		String tbl = mif.getTableName();
		if (scm != null)
		{
			tbl = scm + "." + tbl;
		}
		String stmt = "SELECT ";
		stmt += getListOfColumns(mif);
		stmt += " FROM " + tbl;
		stmt += getWhereClause(query, mif);
		if (query.getCriteria() != null)
		{
			String orderby = query.getCriteria().getOrderby();
			if (orderby != null)
			{
				stmt += " ORDER BY " + orderby;
			}
		}
		return stmt;
	}

	/**
	 * generates a SET-phrase for a prepared update statement
	 */
	private String getSetClause(ClassDescriptor mif)
	{
		String clause = "";
		FieldDescriptor[] fields = mif.getFieldDescriptions();
		for (int i = 0; i < fields.length; i++)
		{
			FieldDescriptor fmd = fields[i];
			if (!fmd.isPrimaryKey())
			{
				Field f = fmd.getPersistentField();
				String cn = quote(fmd.getColumnName());
				if (clause.equals(""))
				{
					clause = " SET ";
				}
				else
				{
					clause += ", ";
				}
				clause += (cn + " = ?");
			}
		}
		return clause;
	}

	/**
	 * 
	 * Generate a where clause for a prepared Statement.
	 * 
	 * Only primary key fields are used in this where clause
	 * 
	 */
	private String getWhereClause(ClassDescriptor mif)
	{
		String clause = "";
		FieldDescriptor[] pkFields = mif.getPkFields();
		return getWhereClause(pkFields, null, mif);
	}

	/**
	 * 
	 * Generate a sql where-clause matching the contraints defined by the criteria of the query
	 * 
	 * @param Query the query
	 * 
	 * @param mif the Classdescriptor providing mapping information
	 * 
	 */
	private String getWhereClause(Query query, ClassDescriptor mif)
	{
		if (query.getCriteria() != null)
		{
			String clause = this.asSQLStatement(query.getCriteria(), mif);
			if (clause != null)
			{
				return " WHERE " + clause;
			}
			else return "";
		}
		else
		{
			return "";
		}
	}

	/**
	 * 
	 * Generate a sql where-clause matching the contraints defined by the vector of
	 * 
	 * Values given for the vector of fields
	 * 
	 * @param vecFields vector containing all columns used in WHERE clause
	 * 
	 * @param vecValues vector containing the Values for the above columns, if vecValues == null, a prepared Statement clause is generated
	 * 
	 * @param mif the Classdescriptor providing mapping information
	 * 
	 */
	private String getWhereClause(FieldDescriptor[] fields, Object[] values, ClassDescriptor mif)
	{
		String clause = "";
		for (int i = 0; i < fields.length; i++)
		{
			FieldDescriptor fmd = fields[i];
			Field f = fmd.getPersistentField();
			String cn = fmd.getColumnName();
			String ct = fmd.getColumnType();
			if (clause.equals(""))
			{
				clause = " WHERE ";
			}
			else
			{
				clause = clause + " AND ";
			}
			Object val = null;
			// value != null ==> generate a dynamic SQL statement
			if (values != null)
			{
				try
				{
					val = quoteIfNeeded(values[i].toString(), ct);
				}
				catch (Exception ex)
				{
					System.out.println("Vector of FieldDescriptors " + fields 
									   + " and Vector of values " + values + " differ in size");
					ex.printStackTrace();
				}
			}
			// vecValues is null ==> generate a prepared stmt
			else
			{
				val = "?";
			}
			clause = clause + "(" + quote(cn) + " = " + val + ")";
		}
		return clause;
	}

}