/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.accesslayer;
import java.util.Hashtable;
import ojb.broker.metadata.ClassDescriptor;
import java.sql.PreparedStatement;
import java.sql.Statement;
import ojb.broker.Identity;
import java.util.Iterator;
import ojb.broker.query.*;
import java.sql.SQLException;
import java.util.Enumeration;
import ojb.broker.PersistenceBrokerImpl;

/**
 * 
 * manages JDBC Connection and Statement ressources.
 * 
 * @author: Thomas Mahler
 * 
 */
public class StatementManager
{

	/**
	 * the singleton instance
	 */
	private static StatementManager _instance = new StatementManager();

	/**
	 * internal table of StatementForClass objects
	 */
	private Hashtable statementTable = new Hashtable();

	/**
	 * constructor is private: use the static methods only
	 */
	private StatementManager() {}

	/**
	 * binds the Identities Primary key values to the statement
	 */
	public static void bindDelete(PreparedStatement stmt, Identity oid) throws java.sql.SQLException
	{
		for (int i = 0; i < oid.getPrimaryKeyValues().length; i++)
		{
			stmt.setObject(i + 1, oid.getPrimaryKeyValues()[i]);
		}
	}

   /**
	 * bind a Query based Select Statement
	 */
	public static int bindStatement(PreparedStatement stmt, Criteria crit, ClassDescriptor cld, 
								int param) throws SQLException
	{
		Enumeration e = crit.getElements();
		while (e.hasMoreElements())
		{
			Object o = e.nextElement();
			if (o instanceof Criteria)
			{
				Criteria pc = (Criteria) o;
				param = bindStatement(stmt, pc, cld, param);
			}
			else
			{
				SelectionCriteria c = (SelectionCriteria) o;
		Object value = c.getO();
				
		if (value != null)
		{
					stmt.setObject(param, value);
		}
		else
		{
				 	stmt.setNull(param, java.sql.Types.NULL);
		}
				param++;
			}
		}
		return param;
	}

    
	/**
	 * binds the values of the object obj to the statements parameters
	 */
	public static void bindInsert(PreparedStatement stmt, ClassDescriptor cld, 
								  Object obj) throws java.sql.SQLException
	{
		Object[] values = cld.getAllValues(obj);
		for (int i = 0; i < values.length; i++)
		{
			stmt.setObject(i + 1, values[i]);
		}
	}

	/**
	 * binds the Identities Primary key values to the statement
	 */
	public static void bindSelect(PreparedStatement stmt, Identity oid) throws java.sql.SQLException
	{
        int i = 0;
        try
        {
            for (i = 0; i < oid.getPrimaryKeyValues().length; i++)
            {
                stmt.setObject(i + 1, oid.getPrimaryKeyValues()[i]);
            }
        }
        catch(SQLException e)
        {
            System.out.println("bindSelect failed for: " + oid.toString() + ", PK: " + i + ", value: " + oid.getPrimaryKeyValues()[i] );
            throw e;
        }
    }

	/**
	 * binds the values of the object obj to the statements parameters
	 */
	public static void bindUpdate(PreparedStatement stmt, ClassDescriptor cld, 
								  Object obj) throws java.sql.SQLException
	{
		int index = 1;
		// parameters for SET-clause
		Object[] values = cld.getNonKeyValues(obj);
		for (int i = 0; i < values.length; i++)
		{
			stmt.setObject(index, values[i]);
			index++;
		}
		values = cld.getKeyValues(obj);
		for (int i = 0; i < values.length; i++)
		{
			stmt.setObject(index, values[i]);
			index++;
		}
	}

	/**
	 * return a prepared DELETE Statement fitting for the given ClassDescriptor
	 */
	public static PreparedStatement getDeleteStatement(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getDeleteStmt();
	}

	/**
	 * return a generic Statement for the given ClassDescriptor
	 */
	public static Statement getGenericStatement(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getGenericStmt();
	}

	/**
	 * return a prepared Insert Statement fitting for the given ClassDescriptor
	 */
	public static PreparedStatement getInsertStatement(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getInsertStmt();
	}

	/**
	 * return a generic Statement for the given ClassDescriptor
	 */
	public static PreparedStatement getPreparedStatement(ClassDescriptor cds, String sql, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getPreparedStmt(sql);
	}

	/**
	 * return a prepared Select Statement for the given ClassDescriptor
	 */
	public static PreparedStatement getSelectByPKStatement(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getSelectByPKkStmt();
	}

	/**
	 * return a StatementsForClass object for the given ClassDescriptor
	 */
	private static StatementsForClass getStatementsForClass(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		StatementsForClass sfc = (StatementsForClass) _instance.statementTable.get(cds);
		if (sfc == null)
		{
			sfc = new StatementsForClass(cds, broker);
			_instance.statementTable.put(cds, sfc);
		}
		return sfc;
	}

	/**
	 * return a prepared Update Statement fitting to the given ClassDescriptor
	 */
	public static PreparedStatement getUpdateStatement(ClassDescriptor cds, PersistenceBrokerImpl broker)
	{
		return getStatementsForClass(cds, broker).getUpdateStmt();
	}

}