/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.metadata;
import java.lang.reflect.Field;
import java.util.Comparator;
import java.util.Vector;

/**
 * 
 * is the base class for all other attribute descriptors. It holds basic the mapping information for a specific
 * 
 * attribute.
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class AttributeDescriptorBase extends Object
{
	private Field m_PersistentField;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * @param fieldname
	 * 
	 * @see
	 */
	public void setPersistentField(Class c, String fieldname)
	{
		m_PersistentField = getField(c, fieldname);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * @param fieldname
	 * 
	 * @return
	 * 
	 * @see
	 */
	protected Field getField(Class c, String fieldname)
	{
		Field f = null;
		try
		{
			f = getFieldRecursive(c, fieldname);    // c.getDeclaredField(fieldname);
			f.setAccessible(true);                  // allow access to private members
		}
		catch (Exception e)
		{
			System.out.println("FieldDescriptor: Can't find member " + fieldname + " in " 
							   + c.getName());
			e.printStackTrace();
		}
		return f;
	}

	/**
	 * try to find a field in class c, recurse through class hierarchy if necessary
	 */
	protected Field getFieldRecursive(Class c, String fieldname) throws NoSuchFieldException
	{
		try
		{
			Field f = c.getDeclaredField(fieldname);
			return f;
		}
		catch (NoSuchFieldException e)
		{
			// if field  could not be found in the inheritance hierarchy, signal error
			if (c == Object.class)
			{
				throw e;
			}
			// if field could not be found in class c try in superclass
			else
			{
				return getFieldRecursive(c.getSuperclass(), fieldname);
			}
		}
	}

	private static boolean _debug = false;

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public AttributeDescriptorBase()
	{
		m_PersistentField = null;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Field getPersistentField()
	{
		return m_PersistentField;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public static void setDebug(boolean b)
	{
		_debug = b;
	}

}