/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.metadata;
import java.util.Properties;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Collections;
import java.lang.reflect.Field;
import java.lang.reflect.Constructor;
import ojb.broker.ConversionStrategy;
import ojb.broker.accesslayer.ConversionStrategyImpl;

/**
 * 
 * A ClassDescriptor contains all information for mapping objects of a
 * 
 * given class to database tables.
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class ClassDescriptor
{

	/**
	 * en/disables tracing
	 */
	private static boolean _debug = false;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public static void setDebug(boolean b)
	{
		_debug = b;
	}

	/**
	 * transaction isolation level specified for this class, used in the ODMG server
	 */
	private int m_IsolationLevel = IsolationLevels.RW_READ_UNCOMMITTED;

	/**
	 * the described class
	 */
	private Class m_Class = null;

	/**
	 * the table name used to store the scalar attributes of this class
	 */
	private String m_TableName = null;

	/**
	 * the FieldDescriptors for the primitive attributes
	 */
	private FieldDescriptor[] m_FieldDescriptions = null;

	/**
	 * the descriptors for collection attributes
	 */
	private Vector m_CollectionDescriptors = new Vector();

	/**
	 * the descriptor for 1-1 reference attributes
	 */
	private Vector m_ObjectReferenceDescriptors = new Vector();

	/**
	 * the JDBCConnectionDescriptor describing the RDBMS connection to be used for this class
	 */
	private JdbcConnectionDescriptor m_ConnDescription = null;
   
	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public ClassDescriptor() {}

	public void setConversionStrategy(ConversionStrategy newStrategy)
	{
		this.conversionStrategy = newStrategy;
	}
    
	public ConversionStrategy getConversionStrategy()
	{
		// if no strategy has been declared, use the default strategy
		if (this.conversionStrategy == null)
		{
			this.conversionStrategy = new ConversionStrategyImpl();
		}
		return this.conversionStrategy;
	}
    
	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String getScheme()
	{
		if (m_ConnDescription != null)
		{
			return m_ConnDescription.getScheme();
			// if class has no special JDBC Connection use the DefaultConnection
		}
		else
		{
			return DescriptorRepository.getInstance().getDefaultJdbcConnection().getScheme();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String getTableName()
	{
		return m_TableName;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param str
	 * 
	 * @see
	 */
	public void setTableName(String str)
	{
		m_TableName = str;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Class getClassOfObject()
	{
		return m_Class;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * 
	 * @see
	 */
	public void setClassOfObject(Class c)
	{
		m_Class = c;
	}

	/**
	 * adds a FieldDescriptor to this ClassDescriptor.
	 * 
	 * 
	 * @param fld
	 */
	public void addFieldDescriptor(FieldDescriptor fld)
	{
		if (m_FieldDescriptions == null)
		{
			m_FieldDescriptions = new FieldDescriptor[1];
			m_FieldDescriptions[0] = fld;
		}
		else
		{
			int size = m_FieldDescriptions.length;
			FieldDescriptor[] tmpArray = new FieldDescriptor[size + 1];
			System.arraycopy(m_FieldDescriptions, 0, tmpArray, 0, size);
			m_FieldDescriptions = tmpArray;
			m_FieldDescriptions[size] = fld;
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Vector getCollectionDescriptors()
	{
		return m_CollectionDescriptors;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param cod
	 * 
	 * @see
	 */
	public void addCollectionDescriptor(CollectionDescriptor cod)
	{
		m_CollectionDescriptors.add(cod);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Vector getObjectReferenceDescriptors()
	{
		return m_ObjectReferenceDescriptors;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param ord
	 * 
	 * @see
	 */
	public void addObjectReferenceDescriptor(ObjectReferenceDescriptor ord)
	{
		m_ObjectReferenceDescriptors.add(ord);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public JdbcConnectionDescriptor getConnectionDescriptor()
	{
		if (m_ConnDescription == null)
		{
			// if class has no special JDBC Connection use the DefaultConnection
			m_ConnDescription = DescriptorRepository.getInstance().getDefaultJdbcConnection();
		}
		return m_ConnDescription;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param jcd
	 * 
	 * @see
	 */
	public void setConnectionDescriptor(JdbcConnectionDescriptor jcd)
	{
		m_ConnDescription = jcd;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param index
	 * 
	 * @return
	 * 
	 * @see
	 */
	public FieldDescriptor getFieldDescriptorByIndex(int index)
	{
		return m_FieldDescriptions[index - 1];
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * 
	 * @return
	 * 
	 * @see
	 */
	public ObjectReferenceDescriptor getORDescriptorByClass(Class c)
	{
		Iterator i = m_ObjectReferenceDescriptors.iterator();
		while (i.hasNext())
		{
			ObjectReferenceDescriptor ord = (ObjectReferenceDescriptor) i.next();
			if (ord.getItemClass() == c)
			{
				return ord;
			}
		}
		return null;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String toString()
	{
		return "table:      \t" + m_TableName + "\n" + "FieldDescriptions:\t" 
			   + m_FieldDescriptions.toString();
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param name
	 * 
	 * @return
	 * 
	 * @see
	 */
	public FieldDescriptor getFieldDescriptorByName(String name)
	{
		for (int i = 0; i < m_FieldDescriptions.length; i++)
		{
			FieldDescriptor fmd = m_FieldDescriptions[i];
			if (fmd.getPersistentField().getName().equals(name))
			{
				return fmd;
			}
		}
		return null;
	}

	/**
	 * returns the transaction isolation level to be used for this class. Used only in the ODMG server
	 */
	public int getIsolationLevel()
	{
		return m_IsolationLevel;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param isoLevel
	 * 
	 * @see
	 */
	public void setIsolationLevel(int isoLevel)
	{
		m_IsolationLevel = isoLevel;
	}

	/**
	 * tells whether we tried to lookup multiargs Constructor already
	 */
	private boolean alreadyLookedup = false;

	/**
	 * the types of the arguments for the attributes in m_FieldDescriptions
	 */
	private Class[] argsForConstructor = null;

	/**
	 * the list of classes in the extent of this class. can be empty
	 */
	private Vector extentClasses = new Vector();

	/**
	 * does the described class represent an interface?
	 */
	private boolean isInterface = false;

	/**
	 * the constructor defined by m_Class to initialze all scalar attributes described in m_FieldDescriptions
	 */
	private Constructor multiArgumentConstructor = null;

	/**
	 * optional orderby clause for initial sorting of collection elements
	 */
	private String orderby = null;

	/**
	 * the proxy class for the described class, may be null
	 */
	private Class proxyClass = null;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param newExtendClass
	 * 
	 * @see
	 */
	public void addExtentClass(Class newExtendClass)
	{
		extentClasses.add(newExtendClass);
	}

	/**
	 * returns a Constructor that takes all persistent attributes
	 * 
	 * of the class as arguments.
	 * 
	 * Returns null, if such a constructor does not exist.
	 * 
	 * PersistenceBroker clients are not urged to implement such constructors
	 * 
	 * for persistence capable classes. But it's strongly recommended as
	 * 
	 * it minimizes the overhead for reflective operations.
	 * 
	 */
	public Constructor getConstructor()
	{
		if (multiArgumentConstructor == null &&!alreadyLookedup)
		{
			if (argsForConstructor == null)
			{
				argsForConstructor = new Class[getFieldDescriptions().length];
				for (int i = 0; i < getFieldDescriptions().length; i++)
				{
					argsForConstructor[i] = 
						getFieldDescriptions()[i].getPersistentField().getType();
				}
			}
			try
			{
				multiArgumentConstructor = m_Class.getConstructor(argsForConstructor);
			}
			catch (Exception ignored)
			{
				String args = "";
				for (int i = 0; i < argsForConstructor.length; i++)
				{
					args += ((args.length() == 0) ? "(" : ", ");
					args += argsForConstructor[i].getName();
				}
				args += ")";
				System.out.println("OJB WARNING: Please define a public constructor for " 
								   + this.getClassOfObject() + "\nwith the following signature: " 
								   + args 
								   + ".\nIt must initialize the classes persistent attributes. This is recommended to increase performance but it's not mandatory!");
			}
			alreadyLookedup = true;
		}
		return multiArgumentConstructor;
	}

	/**
	 * 
	 * return all classes in this extent.
	 * 
	 * Creation date: (02.02.2001 17:49:11)
	 * 
	 * @return java.util.Vector
	 * 
	 */
	public java.util.Vector getExtentClasses()
	{
		return extentClasses;
	}


	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (06.02.2001 22:50:16)
	 * 
	 * @return java.lang.String
	 * 
	 */
	public java.lang.String getOrderby()
	{
		return orderby;
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (26.01.2001 09:20:09)
	 * 
	 * @return java.lang.Class
	 * 
	 */
	public java.lang.Class getProxyClass()
	{
		return proxyClass;
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (02.02.2001 17:49:11)
	 * 
	 * @return boolean
	 * 
	 */
	public boolean isExtent()
	{
		return (extentClasses.size() > 0);
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (02.02.2001 17:49:11)
	 * 
	 * @return boolean
	 * 
	 */
	public boolean isInterface()
	{
		return isInterface;
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (02.02.2001 17:49:11)
	 * 
	 * @param newIsInterface boolean
	 * 
	 */
	public void setIsInterface(boolean newIsInterface)
	{
		isInterface = newIsInterface;
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (06.02.2001 22:50:16)
	 * 
	 * @param newOrderby java.lang.String
	 * 
	 */
	public void setOrderby(java.lang.String newOrderby)
	{
		orderby = newOrderby;
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (26.01.2001 09:20:09)
	 * 
	 * @param newProxyClass java.lang.Class
	 * 
	 */
	public void setProxyClass(java.lang.Class newProxyClass)
	{
		proxyClass = newProxyClass;
	}

	/**
	 * the non-primary key FieldDescriptors
	 */
	private FieldDescriptor[] nonPrimaryKeyFieldDescriptors = null;

	/**
	 * the primary key FieldDescriptors
	 */
	private FieldDescriptor[] primaryKeyFieldDescriptors = null;

	/** the conversion used for java /sql type and value mappings for this class*/
	private ConversionStrategy conversionStrategy = null;
    
	/**
	 * 
	 * returns a vector containing values for all the Objects attributes
	 * 
	 */
	public Object[] getAllValues(Object obj)
	{
		FieldDescriptor[] allFields = getFieldDescriptions();
		Object[] result = new Object[allFields.length];
		for (int i = 0; i < allFields.length; i++)
		{
			FieldDescriptor fmd = allFields[i];
			Field f = fmd.getPersistentField();
			Object cv = null;
			try
			{
				// retrieve the Objects value for the specified field
				cv = f.get(obj);    
				// apply type and value mapping
				cv = this.getConversionStrategy().javaToSql(cv, fmd);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			result[i] = cv;
		}
		return result;
	}

	/**
	 * returns array of all FieldDescriptors.
	 * 
	 * 
	 * @return FieldDescriptor[]
	 * 
	 */
	public FieldDescriptor[] getFieldDescriptions()
	{
		return m_FieldDescriptions;
	}

	/**
	 * 
	 * returns an Array with an Objects PK VALUES
	 */
	public Object[] getKeyValues(Object o)
	{
		FieldDescriptor[] pkFields = getPkFields();
		Object[] result = new Object[pkFields.length];
		for (int i = 0; i < result.length; i++)
		{
			FieldDescriptor fmd = pkFields[i];
			Field f = fmd.getPersistentField();
			Object cv = null;
			try
			{
				cv = f.get(o);    // retrieve the Objects value for the specified pk-field
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			result[i] = cv;
		}
		return result;
	}

	/**
	 * 
	 * returns an Array with an Objects NON-PK VALUES
	 */
	public Object[] getNonKeyValues(Object o)
	{
		FieldDescriptor[] nonPkFields = getNonPkFields();
		Object[] result = new Object[nonPkFields.length];
		for (int i = 0; i < result.length; i++)
		{
			FieldDescriptor fmd = nonPkFields[i];
			Field f = fmd.getPersistentField();
			Object cv = null;
			try
			{
				// retrieve the Objects value for the specified field
				cv = f.get(o); 
				// apply type and value conversion
				cv = this.getConversionStrategy().javaToSql(cv,fmd);
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
			result[i] = cv;
		}
		return result;
	}

	/**
	 * 
	 * return an array of PK FieldDescription sorted ascending
	 * 
	 * according to the field-descriptions getOrder() property
	 * 
	 */
	public FieldDescriptor[] getNonPkFields()
	{
		if (nonPrimaryKeyFieldDescriptors == null)
		{
			// 1. collect all Primary Key fields from Field list
			Vector vec = new Vector();
			for (int i = 0; i < m_FieldDescriptions.length; i++)
			{
				FieldDescriptor fmd = m_FieldDescriptions[i];
				if (!fmd.isPrimaryKey())
				{
					vec.add(fmd);
				}
			}
			// 2. Sort fields according to their getOrder() Property
			Collections.sort(vec, FieldDescriptor.getComparator());
			nonPrimaryKeyFieldDescriptors = 
				(FieldDescriptor[]) vec.toArray(new FieldDescriptor[vec.size()]);
		}
		return nonPrimaryKeyFieldDescriptors;
	}

	/**
	 * 
	 * return an array of PK FieldDescription sorted ascending
	 * 
	 * according to the field-descriptions getOrder() property
	 * 
	 */
	public FieldDescriptor[] getPkFields()
	{
		if (primaryKeyFieldDescriptors == null)
		{
			// 1. collect all Primary Key fields from Field list
			Vector vec = new Vector();
			for (int i = 0; i < m_FieldDescriptions.length; i++)
			{
				FieldDescriptor fmd = m_FieldDescriptions[i];
				if (fmd.isPrimaryKey())
				{
					vec.add(fmd);
				}
			}
			// 2. Sort fields according to their getOrder() Property
			Collections.sort(vec, FieldDescriptor.getComparator());
			primaryKeyFieldDescriptors = 
				(FieldDescriptor[]) vec.toArray(new FieldDescriptor[vec.size()]);
		}
		return primaryKeyFieldDescriptors;
	}

}