/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.metadata;
import java.util.Hashtable;
import java.util.Iterator;
import java.io.File;
import org.xml.sax.Parser;
import org.xml.sax.DocumentHandler;
import org.xml.sax.helpers.ParserFactory;
import java.io.IOException;
import org.xml.sax.SAXException;
import java.net.MalformedURLException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;
import ojb.broker.ClassNotPersistenceCapableException;

/**
 * 
 * The repository containing all mapping information of the PersistenceBroker.
 * 
 * The data is stored in a xml file "repository.xml" in the directory of the main class.
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class DescriptorRepository
{
	// it's a Singleton
	private static DescriptorRepository _instance;

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	private DescriptorRepository()
	{
		try
		{
			buildRepository(_repositoryfile);
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public static DescriptorRepository getInstance()
	{
		if (_instance == null)
		{
			_instance = new DescriptorRepository();
		}
		return _instance;
	}

	/**
	 * path to the xml repository file containing the mapping information
	 */
	private static String _repositoryfile = "repository.xml";

	/**
	 * debugging flag, false by default = no debugging
	 */
	private static boolean _debug = false;

	/**
	 * set the debugging flag to true to enable debugging
	 */
	public static void setDebug(boolean b)
	{
		_debug = b;
	}

	/**
	 * set the path to the xml repository
	 */
	public static void setRepositoryPath(String pathToRepository)
	{
		_repositoryfile = pathToRepository;
	}

	/**
	 * 
	 * This table holds all known Mapping descriptions.
	 * 
	 * Key values are the respective Class objects
	 * 
	 */
	private Hashtable m_DescriptorTable = new Hashtable();

	/**
	 * 
	 * The Descriptor of a default JDBC connection, to be used when no
	 * 
	 * individual descriptor is given for a certain class
	 * 
	 */
	private JdbcConnectionDescriptor m_DefaultConnectionDescriptor;

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String toString()
	{
		return "DescriptorRepository\n" + m_DefaultConnectionDescriptor + "\n" + m_DescriptorTable 
			   + "\n" + "";
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param jcd
	 * 
	 * @see
	 */
	public void setDefaultJdbcConnection(JdbcConnectionDescriptor jcd)
	{
		m_DefaultConnectionDescriptor = jcd;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public JdbcConnectionDescriptor getDefaultJdbcConnection()
	{
		return m_DefaultConnectionDescriptor;
	}

	/**
	 * Add a ClassDescriptor to the internal Hashtable
	 */
	public void put(Class c, ClassDescriptor cld)
	{
		m_DescriptorTable.put(c, cld);
	}

	/**
	 * lookup a ClassDescriptor in the internal Hashtable
	 */
	public ClassDescriptor getDescriptorFor(Class c) throws ClassNotPersistenceCapableException
	{
		ClassDescriptor result = (ClassDescriptor) m_DescriptorTable.get(c);
		if (result == null)
		{
			throw new ClassNotPersistenceCapableException(c.getName() 
					+ " not found in OJB Repository");
		}
		else
		{
			return result;
		}
	}


    public Class getExtentClass(Class clazz)
    {
        //try to find an extent that contains clazz
        Iterator iter =  m_DescriptorTable.values().iterator();
        while (iter.hasNext())
        {
            ClassDescriptor cld = (ClassDescriptor) iter.next();
            if (cld.getExtentClasses().contains(clazz))
            {
                return cld.getClassOfObject();
            }
        }
        // if such an extent could not be found just return clazz itself.
        return clazz;
    }


	/**
	 * Constructor declaration
	 * @param repositoryFileName
	 * 
	 * @see
	 */
	private DescriptorRepository(String repositoryFileName) 
			throws MalformedURLException, ParserConfigurationException, SAXException, IOException
	{
		buildRepository(repositoryFileName);
	}

	/**
	 * Build the Repository by parsing the XML file repository.xml
	 */
	private void buildRepository(String repositoryFileName) 
			throws MalformedURLException, ParserConfigurationException, SAXException, IOException
	{
		File f = new File(repositoryFileName);
		String xmlfile = f.toURL().toString();
		System.out.println("OJB Descriptor Repository: " + xmlfile);
		// get a xml-parser instance:
		// Parser parser = new com.sun.xml.parser.ValidatingParser(true);
		SAXParserFactory saxFact = SAXParserFactory.newInstance();
		saxFact.setValidating(true);
		Parser parser = saxFact.newSAXParser().getParser();
		// create handler for building the repository structure:
		DocumentHandler handler = new RepositoryXmlHandler(this);
		// tell parser to use our handler:
		parser.setDocumentHandler(handler);
		parser.parse(xmlfile);
		if (_debug)
		{
			System.out.println("...Finished parsing");
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param repositoryFileName
	 * 
	 * @return
	 * 
	 * @throws IOException
	 * @throws MalformedURLException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * 
	 * @see
	 */
	public static DescriptorRepository getInstance(String repositoryFileName) 
			throws MalformedURLException, ParserConfigurationException, SAXException, IOException
	{
		if (_instance == null)
		{
			_instance = new DescriptorRepository(repositoryFileName);
		}
		return _instance;
	}

}