/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.metadata;
import java.util.Vector;
import java.util.Iterator;
import java.lang.reflect.Field;
import ojb.broker.ClassNotPersistenceCapableException;

/**
 * 
 * describes a Field containing a reference to another class. Provides handling for foreign keys etc.
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class ObjectReferenceDescriptor extends AttributeDescriptorBase
{
	private Class m_ClassOfItems;
	private Vector m_ForeignKeyFields;
	private boolean m_CascadeRetrieve = true;
	private boolean m_CascadeStore = false;
	private boolean m_CascadeDelete = false;

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public ObjectReferenceDescriptor() {}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Class getItemClass()
	{
		return m_ClassOfItems;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param c
	 * 
	 * @see
	 */
	public void setItemClass(Class c)
	{
		m_ClassOfItems = c;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Vector getForeignKeyFields()
	{
		return m_ForeignKeyFields;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param vec
	 * 
	 * @see
	 */
	public void setForeignKeyFields(Vector vec)
	{
		m_ForeignKeyFields = vec;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean getCascadeRetrieve()
	{
		return m_CascadeRetrieve;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public void setCascadeRetrieve(boolean b)
	{
		m_CascadeRetrieve = b;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean getCascadeStore()
	{
		return m_CascadeStore;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public void setCascadeStore(boolean b)
	{
		m_CascadeStore = b;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean getCascadeDelete()
	{
		return m_CascadeDelete;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public void setCascadeDelete(boolean b)
	{
		m_CascadeDelete = b;
	}

	private Class m_ProxyOfItems = null;

	/**
	 * Insert the method's description here.
	 * Creation date: (10.11.2000 22:44:15)
	 * @return java.lang.Class
	 */
	public java.lang.Class getItemProxyClass() throws ClassNotPersistenceCapableException
	{
		if (m_ProxyOfItems == null)
		{
			m_ProxyOfItems = 
				DescriptorRepository.getInstance().getDescriptorFor(m_ClassOfItems).getProxyClass();
		}
		return m_ProxyOfItems;
	}

	private FieldDescriptor[] foreignKeyFieldDescriptors = null;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param mif
	 * 
	 * @return
	 * 
	 */
	public FieldDescriptor[] getForeignKeyFieldDescriptors(ClassDescriptor mif)
	{
		if (foreignKeyFieldDescriptors == null)
		{
			// 1. collect vector of indices of Fk-Fields
			Vector v = getForeignKeyFields();
			// 2. get FieldDescriptor for each index from Class-descriptor
			Vector ret = new Vector();
			Iterator iter = v.iterator();
			while (iter.hasNext())
			{
				Integer index = (Integer) iter.next();
				ret.add(mif.getFieldDescriptorByIndex(index.intValue()));
			}
			foreignKeyFieldDescriptors = 
				(FieldDescriptor[]) ret.toArray(new FieldDescriptor[ret.size()]);
		}
		return foreignKeyFieldDescriptors;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param obj
	 * @param mif
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Object[] getForeignKeyValues(Object obj, ClassDescriptor mif)
	{
		FieldDescriptor[] fks = getForeignKeyFieldDescriptors(mif);
		Object[] result = new Object[fks.length];
		for (int i = 0; i < result.length; i++)
		{
			FieldDescriptor fmd = fks[i];
			Field f = fmd.getPersistentField();
			try
			{
				Object val = f.get(obj);
				result[i] = val;
			}
			catch (Exception ex)
			{
				System.out.println("Could not get Value for field " + f + " in object " + obj);
				ex.printStackTrace();
			}
		}
		return result;
	}

}