/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.broker.metadata;
import org.xml.sax.HandlerBase;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.AttributeList;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Hashtable;
import ojb.broker.ConversionStrategy;

/**
 * 
 * The handler catches Parsing events raised by the xml-parser
 * 
 * and builds up the DescriptorRepository that is used within the
 * 
 * JDBC Persistence Broker System.
 * 
 * HandlerBase implements the DocumentHandler interface.
 * 
 * 
 * 
 * @author <a href="mailto:thomas.mahler@itellium.com">Thomas Mahler<a>
 * 
 */
public class RepositoryXmlHandler extends HandlerBase
{
	private static boolean _debug = false;
	private int defIsoLevel = IsolationLevels.RW_READ_UNCOMMITTED;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param b
	 * 
	 * @see
	 */
	public static void setDebug(boolean b)
	{
		_debug = b;
	}

	private DescriptorRepository m_repository;
	private JdbcConnectionDescriptor m_CurrentJCD;
	private ClassDescriptor m_CurrentCLD;
	private FieldDescriptor m_CurrentFLD;
	private ObjectReferenceDescriptor m_CurrentORD;
	private CollectionDescriptor m_CurrentCOD;
	private String m_CurrentString;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param literal
	 * 
	 * @return
	 * 
	 * @see
	 */
	private static int getLiteralId(String literal)
	{
		return ((Integer) LiteralsMap.get(literal)).intValue();
	}

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @param dr
	 * 
	 * @see
	 */
	public RepositoryXmlHandler(DescriptorRepository dr)
	{
		m_repository = dr;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void startDocument()
	{
		if (_debug)
		{
			System.out.println("startDoc");
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void endDocument()
	{
		if (_debug)
		{
			System.out.println(m_repository);
		}
		if (_debug)
		{
			System.out.println("endDoc");
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param name
	 * @param atts
	 * 
	 * @see
	 */
	public void startElement(String name, AttributeList atts)
	{
	m_CurrentString = null;
		try
		{
			switch (getLiteralId(name))
			{
				case 0:
				{
					if (m_repository == null)
					{
						m_repository = DescriptorRepository.getInstance();
					}
					String defIso = atts.getValue("isolation");
					if (defIso != null)
					{
						defIsoLevel = getIsoLevel(defIso);
					}
					if (_debug)
					{
						System.out.println(" > MappingRepository");
					}
					break;
				}
				case 1:
				{
					if (_debug)
					{
						System.out.println("   > JdbcConnectionDescription");
					}
					m_CurrentJCD = new JdbcConnectionDescriptor();
					if (m_CurrentCLD == null)
					{
						m_repository.setDefaultJdbcConnection(m_CurrentJCD);
					}
					else
					{
						m_CurrentCLD.setConnectionDescriptor(m_CurrentJCD);
					}
					break;
				}
				case 12:
				{
					if (_debug)
					{
						System.out.println("  > ClassDescriptor");
					}
					String isoLevel = atts.getValue("isolation");
					m_CurrentCLD = new ClassDescriptor();
					if (isoLevel != null)
					{
						m_CurrentCLD.setIsolationLevel(getIsoLevel(isoLevel));
					}
					else
					{
						m_CurrentCLD.setIsolationLevel(defIsoLevel);
					}
					break;
				}
				case 15:
				{
					if (_debug)
					{
						System.out.println("    > FieldDescriptor");
					}
					int id = Integer.parseInt(atts.getValue("id"));
					m_CurrentFLD = new FieldDescriptor(id);
					m_CurrentCLD.addFieldDescriptor(m_CurrentFLD);
					break;
				}
				case 20:
				{
					if (_debug)
					{
						System.out.println("    > ReferenceDescriptor");
					}
					m_CurrentORD = new ObjectReferenceDescriptor();
					m_CurrentCLD.addObjectReferenceDescriptor(m_CurrentORD);
					break;
				}
				case 27:
				{
					if (_debug)
					{
						System.out.println("    > CollectionDescriptor");
					}
					m_CurrentCOD = new CollectionDescriptor();
					m_CurrentCLD.addCollectionDescriptor(m_CurrentCOD);
					break;
				}
				default:
				{
					// most elements are only treated on closing
				}
			}
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param name
	 * 
	 * @see
	 */
	public void endElement(String name)
	{
		try
		{
			switch (getLiteralId(name))
			{
				case 0:
				{
					if (_debug)
					{
						System.out.println(" < MappingRepository");
					}
					break;
				}
				case 1:
				{
					if (_debug)
					{
						System.out.println("   < JdbcConnectionDescription");
					}
					break;
				}
				// SubElements of JdbcConnectionDescription start here
				case 2:
				{
					if (_debug)
					{
						System.out.println("     dbms.name: " + m_CurrentString);
					}
					m_CurrentJCD.setDbms(m_CurrentString);
					break;
				}
				case 5:
				{
					if (_debug)
					{
						System.out.println("     schema.name: " + m_CurrentString);
					}
					if (m_CurrentString != "")
					{
						m_CurrentJCD.setScheme(m_CurrentString);
					}
					break;
				}
				case 6:
				{
					if (_debug)
					{
						System.out.println("     driver.name: " + m_CurrentString);
					}
					m_CurrentJCD.setDriver(m_CurrentString);
					break;
				}
				case 7:
				{
					if (_debug)
					{
						System.out.println("     url.protocol: " + m_CurrentString);
					}
					m_CurrentJCD.setProtocol(m_CurrentString);
					break;
				}
				case 8:
				{
					if (_debug)
					{
						System.out.println("     url.subprotocol: " + m_CurrentString);
					}
					m_CurrentJCD.setSubProtocol(m_CurrentString);
					break;
				}
				case 9:
				{
					if (_debug)
					{
						System.out.println("     url.dbalias: " + m_CurrentString);
					}
					m_CurrentJCD.setDbAlias(m_CurrentString);
					break;
				}
				case 10:
				{
					if (_debug)
					{
						System.out.println("     user.name: " + m_CurrentString);
					}
					m_CurrentJCD.setUserName(m_CurrentString);
					break;
				}
				case 11:
				{
					if (_debug)
					{
						System.out.println("     user.passwd: " + m_CurrentString);
					}
					m_CurrentJCD.setPassWord(m_CurrentString);
					break;
				}
				// SubElements of JdbcConnectionDescription end here
				case 12:
				{
					if (_debug)
					{
						System.out.println("  < ClassDescriptor");
					}
					m_CurrentCLD = null;
					break;
				}
				// SubElements of ClassDescription start here
				case 13:
				{
					if (_debug)
					{
						System.out.println("     class.name: " + m_CurrentString);
					}
					try
					{
						Class c = Class.forName(m_CurrentString);
						m_CurrentCLD.setClassOfObject(c);
						m_repository.put(c, m_CurrentCLD);
					}
					catch (Exception ex)
					{
						ex.printStackTrace();
					}
					break;
				}
				case 35:
				{
					if (_debug)
					{
						System.out.println("     class.proxy: " + m_CurrentString);
					}
					Class pc = Class.forName(m_CurrentString);
					m_CurrentCLD.setProxyClass(pc);
					break;
				}
				case 33:
				{
					if (_debug)
					{
						System.out.println("     class.extend: " + m_CurrentString);
					}
					Class ec = Class.forName(m_CurrentString);
					m_CurrentCLD.addExtentClass(ec);
					break;
				}
				case 34:
				{
					// ExtendDescriptor is only needed for Interfaces
					if (_debug)
					{
						System.out.println("     ExtendDescriptor: " + m_CurrentString);
					}
					m_CurrentCLD.setIsInterface(true);
					break;
				}
				case 14:
				{
					if (_debug)
					{
						System.out.println("     table.name: " + m_CurrentString);
					}
					m_CurrentCLD.setTableName(m_CurrentString);
					break;
				}
				case 36:
				{
					if (_debug)
					{
						System.out.println("     orderby: " + m_CurrentString);
					}
					m_CurrentCLD.setOrderby(m_CurrentString);
					break;
				}
				case 30:
				{
					if (_debug)
					{
						System.out.println("     conversionStrategy: " + m_CurrentString);
					}
					m_CurrentCLD.setConversionStrategy((ConversionStrategy) Class.forName(m_CurrentString).newInstance());
					break;
				}


				// SubElements of ClassDescription end here
				case 15:
				{
					if (_debug)
					{
						System.out.println("    < FieldDescriptor");
					}
					m_CurrentFLD = null;
					break;
				}
				// SubElements of FieldDescriptor start here
				case 16:
				{
					if (_debug)
					{
						System.out.println("     field.name: " + m_CurrentString);
					}
					Class c = m_CurrentCLD.getClassOfObject();
					m_CurrentFLD.setPersistentField(c, m_CurrentString);
					break;
				}
				case 17:
				{
					if (_debug)
					{
						System.out.println("     column.name: " + m_CurrentString);
					}
					m_CurrentFLD.setColumnName(m_CurrentString);
					break;
				}
				case 18:
				{
					if (_debug)
					{
						System.out.println("     jdbc_type: " + m_CurrentString);
					}
					m_CurrentFLD.setColumnType(m_CurrentString);
					break;
				}
				case 19:
				{
					if (_debug)
					{
						System.out.println("     PrimaryKey: " + m_CurrentString);
					}
					boolean b = (new Boolean(m_CurrentString)).booleanValue();
					m_CurrentFLD.setPrimaryKey(b);
					break;
				}
				// SubElements of FieldDescriptor end here
				case 20:
				{
					if (_debug)
					{
						System.out.println("    < ReferenceDescriptor");
					}
					m_CurrentORD = null;
					break;
				}
				// SubElements of ReferenceDescriptor start here
				case 21:
				{
					if (_debug)
					{
						System.out.println("     rdfield.name: " + m_CurrentString);
					}
					m_CurrentORD.setPersistentField(m_CurrentCLD.getClassOfObject(), 
													m_CurrentString);
					break;
				}
				case 22:
				{
					if (_debug)
					{
						System.out.println("     referenced.class: " + m_CurrentString);
					}
					m_CurrentORD.setItemClass(Class.forName(m_CurrentString));
					break;
				}
				case 23:
				{
					if (_debug)
					{
						System.out.println("     descriptor_ids: " + m_CurrentString);
					}
					Vector v = new Vector();
					StringTokenizer st = new StringTokenizer(m_CurrentString, " ");
					while (st.hasMoreTokens())
					{
						Integer FieldIndex = new Integer(st.nextToken());
						v.add(FieldIndex);
					}
					if (m_CurrentCOD != null)
					{
						m_CurrentCOD.setForeignKeyFields(v);
					}
					else
					{
						m_CurrentORD.setForeignKeyFields(v);
					}
					break;
				}
				case 24:
				{
					if (_debug)
					{
						System.out.println("     auto.retrieve: " + m_CurrentString);
					}
					boolean b = (new Boolean(m_CurrentString)).booleanValue();
					if (m_CurrentCOD != null)
					{
						m_CurrentCOD.setCascadeRetrieve(b);
					}
					else
					{
						m_CurrentORD.setCascadeRetrieve(b);
					}
					break;
				}
				case 25:
				{
					if (_debug)
					{
						System.out.println("     auto.update: " + m_CurrentString);
					}
					boolean b = (new Boolean(m_CurrentString)).booleanValue();
					if (m_CurrentCOD != null)
					{
						m_CurrentCOD.setCascadeStore(b);
					}
					else
					{
						m_CurrentORD.setCascadeStore(b);
					}
					break;
				}
				case 26:
				{
					if (_debug)
					{
						System.out.println("     auto.delete: " + m_CurrentString);
					}
					boolean b = (new Boolean(m_CurrentString)).booleanValue();
					if (m_CurrentCOD != null)
					{
						m_CurrentCOD.setCascadeDelete(b);
					}
					else
					{
						m_CurrentORD.setCascadeDelete(b);
					}
					break;
				}
				// SubElements of ReferenceDescriptor end here
				case 27:
				{
					if (_debug)
					{
						System.out.println("    < CollectionDescriptor");
					}
					m_CurrentCOD = null;
					break;
				}
				// SubElements of CollectionDescriptor start here
				case 28:
				{
					if (_debug)
					{
						System.out.println("     cdfield.name: " + m_CurrentString);
					}
					m_CurrentCOD.setPersistentField(m_CurrentCLD.getClassOfObject(), 
													m_CurrentString);
					break;
				}
				case 29:
				{
					if (_debug)
					{
						System.out.println("     items.class: " + m_CurrentString);
					}
					m_CurrentCOD.setItemClass(Class.forName(m_CurrentString));
					break;
				}
				case 37:
				{
					if (_debug)
					{
						System.out.println("     collection.class: " + m_CurrentString);
					}
					m_CurrentCOD.setCollectionClass(Class.forName(m_CurrentString));
					break;
				}
				// SubElements of CollectionDescriptor end here
				// handle failure:
				default:
				{
					System.out.println("Ignoring unknown Element " + name);
				}
			}
		}
		catch (Exception ex)
		{
			System.out.println(ex.getMessage());
			ex.printStackTrace();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param ch
	 * @param start
	 * @param length
	 * 
	 * @see
	 */
		 public void characters(char ch[], int start, int length)
	{
		if (m_CurrentString == null)
			m_CurrentString = new String(ch, start, length);
		else
			m_CurrentString += new String(ch, start, length);
	}

	/**
	 * Error handling
	 */
	public void error(SAXParseException e) throws SAXException
	{
		System.out.println(e.getMessage());
		e.printStackTrace();
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param e
	 * 
	 * @throws SAXException
	 * 
	 * @see
	 */
	public void fatalError(SAXParseException e) throws SAXException
	{
		System.out.println(e.getMessage());
		e.printStackTrace();
		throw e;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param e
	 * 
	 * @throws SAXException
	 * 
	 * @see
	 */
	public void warning(SAXParseException e) throws SAXException
	{
		System.out.println(e.getMessage());
		e.printStackTrace();
	}

	/**
	 * maps element literals to ints (needed for case-expression)
	 */
	private static final Hashtable LiteralsMap = buildLiteralsMap();

	/**
	 * builds up the hastable mapping literals to Integers.
	 * 
	 * I prefer to use case-statements rather than else-if spaghetti-code
	 * 
	 * in the Handler-methods.
	 * 
	 * With this table I hold unique ids for all literals known to
	 * 
	 * the handler.
	 * 
	 */
	private static final Hashtable buildLiteralsMap()
	{
		// avoid rehash by using a fitting size for the table
		int LiteralCount = 40;
		Hashtable table = new Hashtable((int) (1 + LiteralCount / 0.75));
		table.put("MappingRepository", new Integer(0));
		table.put("JdbcConnectionDescriptor", new Integer(1));
		table.put("dbms.name", new Integer(2));
		table.put("dbms.hostname", new Integer(3));
		table.put("dbms.port", new Integer(4));
		table.put("schema.name", new Integer(5));
		table.put("driver.name", new Integer(6));
		table.put("url.protocol", new Integer(7));
		table.put("url.subprotocol", new Integer(8));
		table.put("url.dbalias", new Integer(9));
		table.put("user.name", new Integer(10));
		table.put("user.passwd", new Integer(11));
		table.put("ClassDescriptor", new Integer(12));
		table.put("class.name", new Integer(13));
		table.put("class.proxy", new Integer(35));
		table.put("class.extent", new Integer(33));
		table.put("table.name", new Integer(14));
		table.put("orderby", new Integer(36));
	table.put("conversionStrategy", new Integer(30));
		table.put("FieldDescriptor", new Integer(15));
		table.put("field.name", new Integer(16));
		table.put("column.name", new Integer(17));
		table.put("jdbc_type", new Integer(18));
		table.put("PrimaryKey", new Integer(19));
		table.put("ReferenceDescriptor", new Integer(20));
		table.put("rdfield.name", new Integer(21));
		table.put("referenced.class", new Integer(22));
		table.put("descriptor_ids", new Integer(23));
		table.put("auto.retrieve", new Integer(24));
		table.put("auto.update", new Integer(25));
		table.put("auto.delete", new Integer(26));
		table.put("CollectionDescriptor", new Integer(27));
		table.put("cdfield.name", new Integer(28));
		table.put("items.class", new Integer(29));
		table.put("collection.class", new Integer(37));
		table.put("ExtentDescriptor", new Integer(34));

		table.put("31", new Integer(31));
		table.put("32", new Integer(32));
		table.put("38", new Integer(38));
		table.put("39", new Integer(39));
		return table;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param isoLevel
	 * 
	 * @return
	 * 
	 * @see
	 */
	private int getIsoLevel(String isoLevel)
	{
		if (isoLevel.equals("read-uncommitted"))
		{
			return IsolationLevels.RW_READ_UNCOMMITTED;
		}
		if (isoLevel.equals("read-committed"))
		{
			return IsolationLevels.RW_READ_COMMITTED;
		}
		if (isoLevel.equals("repeatable-read"))
		{
			return IsolationLevels.RW_REPEATABLE_READ;
		}
		if (isoLevel.equals("serializable"))
		{
			return IsolationLevels.RW_SERIALIZABLE;
		}
		return defIsoLevel;
	}

}