/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * $Id: Criteria.java,v 1.8 2001/06/01 19:22:16 thma Exp $
 * COBRA - Java Object Persistence Layer
 * Copyright (C) 1997, 1998    DB Harvey-George
 * eMail: cobra@lowrent.org
 * 
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package ojb.broker.query;
import java.util.Vector;
import java.util.Enumeration;
import ojb.broker.accesslayer.SqlGenerator;

/**
 * Persistent Criteria can be used to retrieve sets of objects based on their attributes
 * Normally each attribute is ANDed together, an OR can be performed by creating a new
 * PersistentCriteria and adding it.
 * <P>
 * Criteria are used, rather than a simple string, because they can be precompiled for
 * efficiency.
 */
public class Criteria implements java.io.Serializable
{
	public final static int OR = 0;
	public final static int AND = 1;
	public final static int NONE = 9;
	private Vector criteria;
	private int type;
	private boolean embraced;
	private String orderby = null;

	/**
	 * Constructor declaration
	 * 
	 * 
	 * @see
	 */
	public Criteria()
	{
		criteria = new Vector();
		type = NONE;
		embraced = false;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public Enumeration getElements()
	{
		return criteria.elements();
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	protected Vector getCriteria()
	{
		return criteria;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public int getType()
	{
		return type;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param type
	 * 
	 * @see
	 */
	public void setType(int type)
	{
		this.type = type;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean isEmbraced()
	{
		return embraced;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param embraced
	 * 
	 * @see
	 */
	public void setEmbraced(boolean embraced)
	{
		this.embraced = embraced;
	}

	/**
	 * Adds and equals (=) criteria,
	 * customer_id = 10034
	 * <P>
	 * Where Value is a string simple pattern matching can be used, the
	 * character * is a wildcard, e.g.
	 * customer_name = Ander*
	 * <P>
	 * Would match the customers: Anderson and Anderton.  The character ? matches a single
	 * occurence of a character.
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addEqualTo(String attribute, Object value)
	{
		if (value instanceof String)
		{
			// substitute our special character for SQL versions
			String pattern = (String) value;
			pattern = pattern.replace('*', '%');
			pattern = pattern.replace('?', '_');
			if (value != pattern)
			{
				LikeCriteria c = new LikeCriteria(attribute, pattern);
				criteria.addElement(c);
			}
			else
			{
				EqualToCriteria c = new EqualToCriteria(attribute, value);
				criteria.addElement(c);
			}
		}
		else
		{
			EqualToCriteria c = new EqualToCriteria(attribute, value);
			criteria.addElement(c);
		}
	}

	/**
	 * Adds GreaterOrEqual Than (>=) criteria,
	 * customer_id >= 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addGreaterOrEqualThan(String attribute, Object value)
	{
		GreaterOrEqualCriteria c = new GreaterOrEqualCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * Adds LessOrEqual Than (<=) criteria,
	 * customer_id <= 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addLessOrEqualThan(String attribute, Object value)
	{
		LessOrEqualCriteria c = new LessOrEqualCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * Adds Like (LIKE) criteria,
	 * customer_id LIKE 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addLike(String attribute, Object value)
	{
		LikeCriteria c = new LikeCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * Adds NotEqualTo (<>) criteria,
	 * customer_id <> 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addNotEqualTo(String attribute, Object value)
	{
		NotEqualToCriteria c = new NotEqualToCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * Adds Greater Than (>) criteria,
	 * customer_id > 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addGreaterThan(String attribute, Object value)
	{
		GreaterThanCriteria c = new GreaterThanCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * Adds Less Than (<) criteria,
	 * customer_id < 10034
	 * 
	 * @param  attribute   The field name to be used
	 * @param  value       An object representing the value of the field
	 */
	public void addLessThan(String attribute, Object value)
	{
		LessThanCriteria c = new LessThanCriteria(attribute, value);
		criteria.addElement(c);
	}

	/**
	 * ORs two sets of criteria together:
	 * active = true AND balance < 0 OR active = true AND overdraft = 0
	 * 
	 * @param  persistent criteria
	 * @throws PersisentException when there are no previous criteria to OR with
	 */
	public void orderBy(String column)
	{
		orderby = column;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String getOrderby()
	{
		return orderby;
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public String toString()
	{
		return SqlGenerator.getInstance().asSQLStatement(this, null);
	}

	/**
	 * ORs two sets of criteria together:
	 * active = true AND balance < 0 OR active = true AND overdraft = 0
	 * 
	 * @param  persistent criteria
	 * @throws PersisentException when there are no previous criteria to OR with
	 */
	// public void addOrCriteria(Criteria pc) {
	public void addOrCriteria(Criteria pc) throws CriteriaException
	{
		if (!criteria.isEmpty())
		{
			pc.setEmbraced(false);
			pc.setType(OR);
			criteria.addElement(pc);
		}
		else
		{
			setEmbraced(false);
			setType(NONE);    // root object
			criteria = pc.getCriteria();
		}
	}

	/**
	 * ANDs two sets of criteria together:
	 * 
	 * @param  persistent criteria
	 */
	public void addAndCriteria(Criteria pc)
	{
		// by combining a second criteria by 'AND' the existing criteria needs to be enclosed
		// in parenthesis
		if (!criteria.isEmpty())
		{
			this.setEmbraced(true);
			pc.setEmbraced(true);
			pc.setType(AND);
			criteria.addElement(pc);
		}
		else
		{
			setEmbraced(false);
			setType(NONE);    // root object
			criteria = pc.getCriteria();
		}
	}

/**
 * Insert the method's description here.
 * Creation date: (08.05.01 15:07:41)
 * @return boolean
 */
public boolean isEmpty() {
	return criteria.isEmpty();
}
}