package ojb.broker.server;

import java.net.*;
import java.io.*;
import java.util.*;

public class PersistenceBrokerServer {

  private static final String defaultFileName = "repository.xml";
  private static final int defaultPort = 2001; // credits to HAL

  private File repositoryFile;
  private ServerSocket server;
  private int numThreads = 10;

  private Vector processors = new Vector();
  private BrokerPool brokers;

  BrokerPool getBrokerPool()
  {
     return brokers;
  }

    
  public PersistenceBrokerServer(File repositoryFile, int port) throws IOException
  {
    this.repositoryFile = repositoryFile.getCanonicalFile();
    this.server = new ServerSocket(port);
    this.brokers = new BrokerPool(repositoryFile);
  }


  public PersistenceBrokerServer(File repositoryFile) throws IOException {
    this(repositoryFile, defaultPort);
  }

  private RequestProcessor addNewProcessor()
  {
      RequestProcessor t = new RequestProcessor(this, this.brokers);
      processors.addElement(t);
      return t;
  }

  void releaseProcessor(RequestProcessor p)
  {
    synchronized (processors)
    {
        processors.addElement(p);
        processors.notifyAll();
    }

  }


  public void run() {
  
    for (int i = 0; i < numThreads; i++) {
      new Thread(addNewProcessor(),"OJB Worker " + processors.size()).start();
      System.out.print(".");
    }
    System.out.println("\nOJB PersistenceBrokerServer accepting connections on port "
     + server.getLocalPort());
    System.out.println("Serving Mapping Repository: " +  repositoryFile.getAbsolutePath());
    while (true) {
      try {
        Socket request = server.accept();
        synchronized (processors)
        {
            while (processors.isEmpty())
            {
                try
                {
                    processors.wait();
                }
                catch(InterruptedException e)
                {
                }
            }
            RequestProcessor rp = (RequestProcessor) processors.elementAt(0);
            processors.removeElementAt(0);
            rp.setSocket(request);


        /*if (processors.isEmpty())
        {
            RequestProcessor rp = addNewProcessor();
            rp.setSocket(request);
            new Thread(rp,"OJB Worker " + processors.size()).start();
        }
        else
        {
            RequestProcessor rp = (RequestProcessor) processors.elementAt(0);
            processors.removeElementAt(0);
            rp.setSocket(request);
        }
       */


        }
      }
      catch (IOException e) {
        System.err.println(e.getMessage());
      }   
    }
    
  }
  
  public static void main(String[] args) {

    // get the repository dir
    File repositoryFile;
    try {
      repositoryFile = new File(args[0]);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      System.out.println("Usage: java PersistenceBrokerServer RepositoryFile port");
      return;
    }
    
    // set the port to listen on
    int port;
    try {
      port = Integer.parseInt(args[1]);
      if (port < 0 || port > 65535) port = defaultPort;
    }  
    catch (Exception e) {
      port = defaultPort;
    }  
    
    try {            
      PersistenceBrokerServer pbserver = new PersistenceBrokerServer(repositoryFile, port);
      pbserver.run(); //start();
    }
    catch (IOException e) {
      System.out.println("OJB PersistenceBrokerServer could not start because of an "
       + e.getClass());
      System.out.println(e);
    }
  
  }

}
