package ojb.broker.server;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.Serializable;
import ojb.broker.server.*;
import ojb.broker.PersistenceBrokerException;


/*
 * Serializable Stub of an PkEnumeration that can be send to a client.
 * This stub knows how to handle nextElement() and hasMoreElements() calls
 */
public class PkEnumerationStub implements Enumeration, Serializable
{

    /**
	 * reference to the used PersistenceBroker
	 */
	private PersistenceBrokerClient broker = null;

	private int iteratorId;

	public PkEnumerationStub(PersistenceBrokerClient pbc, int iid)
	{
	    this.broker = pbc;
	    this.iteratorId = iid;
    }

    /**
	 * Release all internally used Database resources of the iterator.
	 * Clients must call this methods explicitely if the iterator is not
	 * exhausted by the client application. If the Iterator is exhauseted
	 * this method will be called implicitely.
	 */
	public void releaseDbResources()
	{
	    broker.releaseDbResources(iteratorId);
	}

    public boolean hasMoreElements()
    {
        try
        {
            return broker.hasNext(iteratorId);
        }
        catch(PersistenceBrokerException e)
        {
            return false;
        }
    }

    public Object nextElement()
    {
        try
        {
            return broker.next(iteratorId);
        }
        catch(PersistenceBrokerException e)
        {
            throw new NoSuchElementException(e.getMessage());
        }
    }
}