package ojb.broker.server;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.io.Serializable;
import ojb.broker.server.*;
import ojb.broker.PersistenceBrokerException;


/*
 * Serializable Stub of an Iterator that can be send to a client.
 * This stub knows how to handle next() and hasNext() calls
 */
public class RsIteratorStub implements Iterator, Serializable
{

    /**
	 * reference to the used PersistenceBroker
	 */
	private PersistenceBrokerClient broker = null;

	private int iteratorId;

	public RsIteratorStub(PersistenceBrokerClient pbc, int iid)
	{
	    this.broker = pbc;
	    this.iteratorId = iid;
    }


    public boolean hasNext()
    {
        try {
            return broker.hasNext(iteratorId);
        }
        catch(PersistenceBrokerException e) {
            return false;
        }
    }

    public Object next() throws NoSuchElementException
    {
        try {
            return broker.next(iteratorId);
        }
        catch(PersistenceBrokerException e) {
            throw new NoSuchElementException(e.getMessage());
        }
    }

    public void remove()
    {

    }


    /**
	 * Release all internally used Database resources of the iterator.
	 * Clients must call this methods explicitely if the iterator is not
	 * exhausted by the client application. If the Iterator is exhauseted
	 * this method will be called implicitely.
	 */
	public void releaseDbResources()
	{
	    broker.releaseDbResources(iteratorId);
	}


}