/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg;
import java.util.Properties;
import ojb.broker.query.*;
import ojb.broker.*;
import org.odmg.ObjectNameNotUniqueException;
import org.odmg.ObjectNameNotFoundException;
import java.io.*;

/**
 * 
 * ODMG NamedRoots implementation.
 * 
 * this implementation stores the (name, Identity) pairs in
 * 
 * a database table.
 * 
 * therefore the NamedRootsMap underlies the same transaction management
 * 
 * as all other persistent objects
 * 
 * @author: Thomas Mahler
 * 
 */
public class NamedRootsMap
{

	/**
	 * singleton instance
	 */
	private static NamedRootsMap _instance;

	/**
	 * 
	 * constructor is private, use getInstance()
	 * 
	 */
	private NamedRootsMap() {}

	/**
	 * lookup key in map. If found return a deserialized Identity object, else null
	 */
	public Identity get(String key)
	{
		NamedRootsEntry example = new NamedRootsEntry(key, null);
		Query query = new QueryByExample(example);
		try
		{
			PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();
			NamedRootsEntry entry = (NamedRootsEntry) broker.getObjectByQuery(query);
			return Identity.fromByteArray(entry.getOid());
		}
		catch (Throwable t)
		{
			return null;
		}
	}

	/**
	 * factory method returns singleton instance
	 */
	public static NamedRootsMap getInstance()
	{
		if (_instance == null)
		{
			_instance = new NamedRootsMap();
		}
		return _instance;
	}

	/**
	 * put a serialized verion of oid into the map
	 */
	public void put(String key, Identity oid) throws ObjectNameNotUniqueException
	{
		NamedRootsEntry entry = new NamedRootsEntry(key, oid.serialize());
		try
		{
			PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
			TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
			NamedRootsEntry stored = 
				(NamedRootsEntry) broker.getObjectByQuery(new QueryByExample(entry));
			if (stored != null)
			{
				throw new ObjectNameNotUniqueException("the key " + key + " is not unique");
			}
			else
			{
				// don't store us, we'll store you :-)
				// i.e. no need to store entry here, just ask for beeing stored on next commit
				tx.register(entry);
			}
		}
		catch (PersistenceBrokerException ex) {}
	}

	/**
	 * remove an entry from the NamedRoots map
	 */
	public void unbind(String key) throws ObjectNameNotFoundException
	{
		NamedRootsEntry example = new NamedRootsEntry(key, null);
		try
		{
			PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
			TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
			NamedRootsEntry stored = 
				(NamedRootsEntry) broker.getObjectByQuery(new QueryByExample(example));
			if (stored == null)
			{
				throw new ObjectNameNotFoundException("don't know " + key);
			}
			else
			{
				// mark stored entry for deletion
				// deletion will occur on next commit
				tx.markDelete(stored);
			}
		}
		catch (PersistenceBrokerException ex) {}
	}

}