/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg;
import org.odmg.*;
import ojb.broker.Identity;
import ojb.odmg.collections.*;
import ojb.broker.metadata.*;

/**
 * Facade to the persistence ObjectServer system.
 * Implements the factory interface for a particular ODMG implementation.
 * @author Thomas Mahler
 * @version ODMG 3.0
 */
public final class OJB extends Object implements org.odmg.Implementation
{

	/**
	 * the singleton instance
	 */
	private static OJB _instance;

	/**
	 * Create a <code>Transaction</code> object and associate it with the current thread.
	 *
	 * @return The newly created <code>Transaction</code> instance.
	 *
	 * @see org.odmg.Transaction
	 *
	 */
	public Transaction newTransaction()
	{
		Transaction tx;
		tx = tx_table.get(Thread.currentThread());
		if (tx == null)
		{
			tx = new TransactionImpl(tx_table, currentDatabase);
			tx_table.put(Thread.currentThread(), tx);
		}
		return tx;

	}

	/**
	 * Get the current <code>Transaction</code> for the thread.
	 *
	 * @return The current <code>Transaction</code> object or null if there is none.
	 *
	 * @see org.odmg.Transaction
	 *
	 */
	public Transaction currentTransaction()
	{
		return tx_table.get(Thread.currentThread());
	}

	/**
	 * Create a new <code>Database</code> object.
	 *
	 * @return The new <code>Database</code> object.
	 *
	 * @see org.odmg.Database
	 *
	 */
	public Database newDatabase()
	{
		currentDatabase = new DatabaseImpl();
		return currentDatabase;
	}

	/**
	 * Create a new <code>OQLQuery</code> object.
	 *
	 * @return The new <code>OQLQuery</code> object.
	 *
	 * @see org.odmg.OQLQuery
	 *
	 */
	public OQLQuery newOQLQuery()
	{
		return new ojb.odmg.oql.OQLQueryImpl();
	}

	/**
	 * Create a new <code>DList</code> object.
	 *
	 * @return The new <code>DList</code> object.
	 *
	 * @see org.odmg.DList
	 *
	 */
	public DList newDList()
	{
		return new DListImpl();
	}

	/**
	 * Create a new <code>DBag</code> object.
	 * @return The new <code>DBag</code> object.
	 * @see org.odmg.DBag
	 */
	public DBag newDBag()
	{
		return new DBagImpl();
	}

	/**
	 * Create a new <code>DSet</code> object.
	 * @return The new <code>DSet</code> object.
	 * @see org.odmg.DSet
	 */
	public DSet newDSet()
	{
		return new DSetImpl();
	}

	/**
	 * Create a new <code>DArray</code> object.
	 * @return The new <code>DArray</code> object.
	 * @see org.odmg.DArray
	 */
	public DArray newDArray()
	{
		return new DListImpl();
	}

	/**
	 * Create a new <code>DMap</code> object.
	 * @returnThe new <code>DMap</code> object.
	 * @see org.odmg.DMap
	 */
	public DMap newDMap()
	{
		return new DMapImpl();
	}

	/**
	 * Get a <code>String</code> representation of the object's identifier.
	 * OJB returns the serialized Identity of the object.
	 * @param obj The object whose identifier is being accessed.
	 * @return The object's identifier in the form of a String
	 */
	public String getObjectId(Object obj)
	{
		Identity oid = new Identity(obj);
		return new String(oid.serialize());
	}

	/**
	 * Get the <code>Database</code> that contains the object <code>obj</code>.
	 * @param obj The object.
	 * @return The <code>Database</code> that contains the object.
	 * If obj.getClass() is defined in the repository return current db
	 * else throw exception, as we can't tell where unknown classes reside
	 */
	public Database getDatabase(Object obj)
	{
		// if no object specified just return the current Database
		if (obj == null)
		{
		  return currentDatabase;
		}
		else
		{
		  try
		  {
			  // if there is a ClassDescriptor in the repository...
			  ClassDescriptor cld =
				  DescriptorRepository.getInstance().getDescriptorFor(obj.getClass());
			  // the objects class is defined in the current db
			  return currentDatabase;
		  }
		  catch (ojb.broker.ClassNotPersistenceCapableException ex)    // if not, we can't help...
		  {
			  throw new ODMGRuntimeException(ex.getMessage());
		  }
		}
	}

	/**
	 * Method declaration
	 *
	 *
	 * @return
	 *
	 * @see
	 */
	public static OJB getInstance()
	{
		if (_instance == null)
		{
			_instance = new OJB();
		}
		return _instance;
	}

	/**
	 * the currently active Database
	 */
	private DatabaseImpl currentDatabase = null;

	/**
	 * this flag en/disables tracing
	 */
	static final boolean debug = false;



	/**
	 * Method declaration
	 *
	 *
	 * @param curDB
	 *
	 * @see
	 */
	void setCurrentDatabase(DatabaseImpl curDB)
	{
		currentDatabase = curDB;
	}

	/**
	 * 
	 * the internal table containing all transactions and their respective threads
	 * 
	 */
	private TransactionTable tx_table = new TransactionTable();
}