/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;
import ojb.odmg.states.*;
import java.util.Enumeration;
import org.odmg.*;
import ojb.broker.Identity;
import ojb.broker.PersistenceBroker;

/**
 * Class declaration
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public class ObjectStateTable extends Object
{

	/**
	 * Creates new ObjectStateTable
	 */
	public ObjectStateTable() {}

	/**
	 * perfom checkpoint for all tx-state in the table
	 */
	public void checkpoint() throws TransactionAbortedException
	{
		try
		{
			// all neccessary db operations are executed within a PersistenceBroker transaction:
			PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();
			broker.beginTransaction();
			Enumeration enum = table.elements();
			while (enum.hasMoreElements())
			{
				ObjectTransactionWrapper mod = (ObjectTransactionWrapper) enum.nextElement();
				if (OJB.debug)
				{
					System.out.println("checkpoint: " + new Identity(mod.getObject()) + ", " 
									   + mod.getModificationState());
				}

				// if the Object has been modified has been modified by transaction, mark object as dirty
				if (mod.hasChanged())
				{
					mod.setModificationState(mod.getModificationState().markDirty());
				}
				mod.getModificationState().checkpoint(mod, broker);
			}
			broker.commitTransaction();
		}
		catch (Throwable t)
		{
			throw new TransactionAbortedException(t.getMessage());
		}
	}

	/**
	 * perform commit on all tx-states
	 */
	public void commit() throws TransactionAbortedException
	{
		try
		{
			// all neccessary db operations are executed within a PersistenceBroker transaction:
			PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
			broker.beginTransaction();
			Enumeration enum = table.elements();
			while (enum.hasMoreElements())
			{
				ObjectTransactionWrapper mod = (ObjectTransactionWrapper) enum.nextElement();
				if (OJB.debug)
				{
					System.out.println("commit: " + mod);
				}

				// if the Object has been modified has been modified by transaction, mark object as dirty
				if (mod.hasChanged())
				{
					mod.setModificationState(mod.getModificationState().markDirty());
				}
				mod.getModificationState().commit(mod, broker);
			}
			broker.commitTransaction();
		}
		catch (Throwable t)
		{
			throw new TransactionAbortedException(t.getMessage());
		}
	}

/**
 * perform rollback on all tx-states
 */
public void rollback() {
	Enumeration enum = table.elements();
	while (enum.hasMoreElements()) {
		ObjectTransactionWrapper mod = (ObjectTransactionWrapper) enum.nextElement();
		if (OJB.debug) {
			System.out.println("rollback: " + mod);
		}

		// if the Object has been modified has been modified by transaction, mark object as dirty
		if (mod.hasChanged()) 
		{
			mod.setModificationState(mod.getModificationState().markDirty());
		}

		mod.getModificationState().rollback(mod, null);
	}
}

	/**
	 * the internal table mapping Objects to their ObjectTransactionWrappers
	 */
	private Hashtable table = new Hashtable();

	/**
	 * remove an objects entry from the Hashtable
	 */
	public void remove(Object key)
	{
		table.remove(new Integer(System.identityHashCode(key)));
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (11.02.2001 12:45:08)
	 * 
	 * @return java.lang.String
	 * 
	 */
	public Enumeration elements()
	{
		return table.elements();
	}

	/**
	 * retrieve an objects ObjectModification state from the hashtable
	 */
	public ObjectTransactionWrapper get(Object key)
	{
		Integer keyInteger = new Integer(System.identityHashCode(key));
		ObjectTransactionWrapper result = (ObjectTransactionWrapper) table.get(keyInteger);
		if (result == null)
		{
			result = new ObjectTransactionWrapper(key);
			table.put(keyInteger, result);
			if (OJB.debug)
			{
				System.out.println("register: " + result);
			}
		}
		return result;
	}

	/**
	 * store an objects transactional state into the Hashtable
	 */
	public void put(Object key, ObjectTransactionWrapper modification)
	{
		Integer keyInt = new Integer(System.identityHashCode(key));
		if (OJB.debug)
		{
			System.out.println("register: " + modification);
		}
		table.put(keyInt, modification);
	}

	/**
	 * 
	 * Insert the method's description here.
	 * 
	 * Creation date: (11.02.2001 12:45:08)
	 * 
	 * @return java.lang.String
	 * 
	 */
	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		buf.append("### ObjectStateTable dump:\n");
		java.util.Enumeration enum = table.elements();
		while (enum.hasMoreElements())
		{
			ObjectTransactionWrapper mod = (ObjectTransactionWrapper) enum.nextElement();
			buf.append(mod.toString());
			buf.append("\n");
		}
		return buf.toString();
	}

	/**
	 * retrieve an objects ObjectModification state from the hashtable
	 */
	public boolean contains(Object key)
	{
		Integer keyInteger = new Integer(System.identityHashCode(key));
		ObjectTransactionWrapper result = (ObjectTransactionWrapper) table.get(keyInteger);
		return (result != null) ? true : false;
	}
}