/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg;

/**
 * 
 * Title:        ObJectRelationalBridge
 * 
 * Description:  Relational - Object Bridge
 * 
 * Copyright:    Copyright (c) David Dixon-Peugh
 * 
 * Company:
 * 
 * @author David Dixon-Peugh
 * 
 * @version 1.0
 * 
 */
import java.util.*;
import java.lang.reflect.*;
import ojb.broker.metadata.*;
import ojb.odmg.states.ModificationState;
import ojb.broker.Identity;

/**
 * Class declaration
 * 
 * 
 * @author
 * @version %I%, %G%
 */
public class ObjectTransactionWrapper implements ojb.broker.ObjectModification
{

	/**
	 * 
	 * myObj holds the object we are wrapping.
	 * 
	 */
	private transient Object myObj;

	/**
	 * 
	 * beforeImage holds a mapping between field
	 * 
	 * names and values at the start of the transaction.
	 * 
	 * 
	 * 
	 * afterImage holds the mapping at the
	 * 
	 * end of the transaction.
	 * 
	 */
	private transient Map beforeImage;
	private transient Map afterImage;

	/**
	 * 
	 * Create a new wrapper, but we can't manage
	 * 
	 * anything yet.
	 * 
	 */
	private ObjectTransactionWrapper() {}

	/**
	 * 
	 * Create a wrapper by providing an Object.
	 * 
	 */
	public ObjectTransactionWrapper(Object obj)
	{
		manage(obj);
		setInitialModificationState();
	}

	/**
	 * Manage an object.
	 */
	public void manage(Object obj)
	{
		myObj = obj;
		try
		{
			beforeImage = getMap();
		}
		catch (Exception ex)
		{
			beforeImage = null;
			throw new org.odmg.ClassNotPersistenceCapableException(ex.getMessage());
		}
	}

	/**
	 * 
	 * Simple maintenance methods.
	 * 
	 */
	public Object getObject()
	{
		return myObj;
	}

	/**
	 * We need to implement the Two-Phase Commit
	 * protocol.
	 * 
	 * beginCommit is where we say if we can or cannot
	 * commit the transaction.  At the begining however,
	 * we need to attain the after image so we can isolate
	 * everything.
	 * 
	 * We should issue the call against the database
	 * at this point.  If we get a SQL Exception, we
	 * should throw the org.odmg.TransactionAbortedException.
	 * 
	 * We should also check to see if the object is
	 * TransactionAware.  If so, we should give it a chance
	 * to kill the transaction before we toss it to the
	 * database.
	 */
	public void beforeCommit()
	{
		if (myObj instanceof TransactionAware)
		{
			TransactionAware ta = (TransactionAware) myObj;
			ta.beforeCommit();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void afterCommit()
	{
		if (myObj instanceof TransactionAware)
		{
			TransactionAware ta = (TransactionAware) myObj;
			ta.afterCommit();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void beforeAbort()
	{
		if (myObj instanceof TransactionAware)
		{
			TransactionAware ta = (TransactionAware) myObj;
			ta.beforeAbort();
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @see
	 */
	public void afterAbort()
	{
		if (myObj instanceof TransactionAware)
		{
			TransactionAware ta = (TransactionAware) myObj;
			ta.afterAbort();
		}
	}

	/**
	 * The objects modification state, e.g. Old and Clean
	 */
	private ModificationState modificationState = null;

	/**
	 * tracks whether update is neccesary or not
	 */
	private boolean needsInsert = false;

	/**
	 * tracks whether update is neccesary or not
	 */
	private boolean needsUpdate = false;

	/**
	 * 
	 * abort method comment.
	 * 
	 */
	public void abort() {}

	/**
	 * 
	 * commit method comment.
	 * 
	 */
	public void commit() {}

	/**
	 * 
	 * getMap() will return the image of the Object.
	 * 
	 */
	private Map getMap() 
			throws IllegalAccessException, ojb.broker.ClassNotPersistenceCapableException
	{
		Map fieldValues = new HashMap();
		ClassDescriptor mif = 
			DescriptorRepository.getInstance().getDescriptorFor(getObject().getClass());
		FieldDescriptor[] fieldDescs = mif.getFieldDescriptions();
		for (int i = 0; i < fieldDescs.length; i++)
		{
			FieldDescriptor fd = fieldDescs[i];
			Field f = fd.getPersistentField();
			f.setAccessible(true);
			fieldValues.put(fd.getColumnName(), f.get(myObj));
		}
		return fieldValues;
	}

	/**
	 * 
	 * returns the Modification-state.
	 * 
	 * @return ojb.server.states.ModificationState
	 * 
	 */
	public ModificationState getModificationState()
	{
		return modificationState;
	}

	/**
	 * returns true if the underlying Object needs an INSERT statement. Returns false else.
	 */
	public boolean needsInsert()
	{
		return this.getModificationState().needsInsert();
	}

	/**
	 * returns true if the underlying Object needs an UPDATE statement.
	 * 
	 * Else Returns false.
	 * 
	 */
	public boolean needsUpdate()
	{
		return needsUpdate;
	}

	/**
	 * setFields sets the fields on the object.  This is used when the
	 * transaction is aborted.
	 */
	private void setFields(Map theMap) 
			throws IllegalAccessException, ojb.broker.ClassNotPersistenceCapableException
	{
		ClassDescriptor mif = DescriptorRepository.getInstance().getDescriptorFor(myObj.getClass());
		FieldDescriptor[] fieldDescs = mif.getFieldDescriptions();
		for (int i = 0; i < fieldDescs.length; i++)
		{
			FieldDescriptor fd = fieldDescs[i];
			Field f = fd.getPersistentField();
			f.setAccessible(true);
			f.set(myObj, theMap.get(fd.getColumnName()));
		}
	}

	/**
	 * 
	 * sets the initial MoificationState of the wrapped object myObj. The initial state will be StateNewDirty if myObj
	 * 
	 * is not persisten already. The state will be set to StateOldClean if the object is already persistent.
	 * 
	 */
	private void setInitialModificationState()
	{
		// determine appropriate modification state
		ModificationState initialState;
		Object result;
		try
		{
			// try to lookup the object.
			result = 
				((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker().getObjectByIdentity(new Identity(myObj));
		}
		catch (ojb.broker.PersistenceBrokerException ex)
		{
			throw new org.odmg.ODMGRuntimeException(ex.getMessage());
		}
		if (result == null)
		{
			// if object is not already persistent it must be marked as new
			// it must be marked as dirty because it must be stored even if it will not modified during tx
			initialState = ojb.odmg.states.StateNewDirty.getInstance();
		}
		else
		{
			// if object is already persistent it will be marked as old.
			// it is marked as clean as it has not been modified during tx already
			initialState = ojb.odmg.states.StateOldClean.getInstance();
		}
		// remember it:
		modificationState = initialState;
	}

	/**
	 * set the Modification state to a new value. Used during state transitions.
	 * @param newModificationState ojb.server.states.ModificationState
	 */
	public void setModificationState(ModificationState newModificationState)
	{
		if (newModificationState != modificationState)
		{
			if (OJB.debug)
			{
				System.out.println("transition: " + new Identity(this.myObj) + " (" 
								   + modificationState + " --> " + newModificationState + ")");
			}
			modificationState = newModificationState;
		}
	}

	/**
	 * returns a String representation.
	 * @return java.lang.String
	 */
	public String toString()
	{
		return new ojb.broker.Identity(myObj) + "(" + modificationState + ")";
	}

/**
 * checks whether object and internal clone differ and returns true if so, returns false else.
 * @return boolean
 */
public boolean hasChanged() 
{
	Map currentImage = null;
	try
	{
		currentImage = getMap();
	}
	catch (Exception e){}
	
	if (beforeImage.equals(currentImage)) return false;
	else return true;
}
}