/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/**
 * 
 * Title:        ObJectRelationalBridge<p>
 * 
 * Description:  Relational - Object Bridge<p>
 * 
 * Copyright:    Copyright (c)2000 Thomas Mahler, David Dixon-Peugh<p>
 * 
 * Company:      <p>
 * 
 * @author David Dixon-Peugh
 * 
 * @version 1.0
 * 
 */
package ojb.odmg;
import org.odmg.TransactionAbortedException;

/**
 * 
 * TransactionAware is an interface that can be implemented
 * 
 * to provide hooks into the Transaction interface provided
 * 
 * by ObJectRelationalBridge.
 * 
 * 
 * 
 * Only objects which have a write lock acquired on them can
 * 
 * participate in a transaction.
 * 
 * 
 * 
 * To kill a transaction, implement beforeCommit() and throw
 * 
 * a TransactionAbortedException.  This will force the entire
 * 
 * transaction to rollback.
 * 
 * 
 * 
 * To rebuild an object after a rollback use the afterAbort()
 * 
 * call.  This is a good place to populate transient or other
 * 
 * variables.
 * 
 * 
 * 
 * beforeAbort and afterCommit are there for informational
 * 
 * purposes.
 * 
 * 
 * 
 * Here are some common ways you can expect this interface
 * 
 * to be called:
 * 
 * 
 * 
 * Sucessful commit:
 * 
 * beforeCommit()
 * 
 * afterCommit()
 * 
 * 
 * 
 * Transaction Failure (1):
 * 
 * beforeCommit()
 * 
 * beforeAbort()
 * 
 * afterAbort()
 * 
 * 
 * 
 * Transaction Failure (2):
 * 
 * beforeAbort()
 * 
 * afterAbort()
 * 
 * 
 * 
 * Commits and Aborts aren't directly provided to TransactionAware classes.
 * 
 * The idea is that Transactions are difficult to handle, and most of it
 * 
 * will be handled by ObjectSnapshot.  However, you use TransactionAware
 * 
 * to do one of two things, kill a transaction from happening, and clean
 * 
 * up after a rollback.
 * 
 * 
 * 
 */
public interface TransactionAware
{

	/**
	 * 
	 * beforeCommit will give an object a chance to kill a
	 * 
	 * transaction before it is committed.
	 * 
	 * 
	 * 
	 * To kill a transaction, throw a new TransactionAbortedException.
	 * 
	 */
	public void beforeCommit() throws org.odmg.TransactionAbortedException;

	/**
	 * 
	 * afterCommit is called only after a successful commit has taken
	 * 
	 * place.
	 * 
	 */
	public void afterCommit();

	/**
	 * 
	 * beforeAbort is called before a transaction is aborted.
	 * 
	 */
	public void beforeAbort();

	/**
	 * 
	 * afterAbort will be called after a transaction has been aborted.
	 * 
	 * The values of fields which get persisted will have changed to
	 * 
	 * what they were at the begining of the transaction.  This method
	 * 
	 * should be overridden to reset any transient or non-persistent
	 * 
	 * fields.
	 * 
	 */
	public void afterAbort();
}