/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/
/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.collections;
import java.util.*;
import ojb.broker.*;
import ojb.odmg.*;
import ojb.odmg.OJB;
import ojb.broker.ManageableCollection;
import ojb.broker.metadata.ClassDescriptor;
import org.odmg.*;
import ojb.odmg.oql.*;
import ojb.broker.query.*;
import ojb.broker.metadata.*;

/**
 * Insert the type's description here.
 * Creation date: (10.02.2001 20:50:26)
 * @author: Administrator
 */
public class DListImpl extends java.util.AbstractList implements DList, DArray,
		ManageableCollection, java.io.Serializable
{
	private int id;
	private Vector elements;
	private int size;

	/**
	 * DListImpl constructor comment.
	 */
	public DListImpl()
	{
		super();
		elements = new Vector();
		id = generateNewId();
		this.size = 0;
	}

	/**
	 * DListImpl constructor comment.
	 */
	public DListImpl(int theId, int theSize)
	{
		super();
		id = theId;
		this.size = theSize;
		TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
		tx.register(this);
	}

	/**
	 * Inserts the specified element at the specified position in this list
	 * (optional operation).  Shifts the element currently at that position
	 * (if any) and any subsequent elements to the right (adds one to their
	 * indices).
	 *
	 * @param index index at which the specified element is to be inserted.
	 * @param element element to be inserted.
	 *
	 * @throws UnsupportedOperationException if the <tt>add</tt> method is not
	 * supported by this list.
	 * @throws    ClassCastException if the class of the specified element
	 * prevents it from being added to this list.
	 * @throws    IllegalArgumentException if some aspect of the specified
	 * element prevents it from being added to this list.
	 * @throws    IndexOutOfBoundsException if the index is out of range
	 * (index &lt; 0 || index &gt; size()).
	 */
	public void add(int index, Object element)
	{
		TransactionImpl tx =  getCurrentTransaction();
		tx.register(this);
		this.size++;
		DListEntry entry = new DListEntry(this, element);
		tx.register(entry);
		tx.register(element);
		elements.add(index, entry);
		// changing the position markers of entries:
		int offset = 0;
		try
		{
			offset = ((DListEntry) elements.get(index - 1)).getPosition();
		}
		catch (Exception ignored) {}
		for (int i = offset; i < elements.size(); i++)
		{
			entry = (DListEntry) elements.get(i);
			entry.setPosition(i);
		}
	}

    /**
     * Removes the element at the specified position in this list (optional
     * operation).  Shifts any subsequent elements to the left (subtracts one
     * from their indices).  Returns the element that was removed from the
     * list.<p>
     *
     * This implementation always throws an
     * <tt>UnsupportedOperationException</tt>.
     *
     * @param index the index of the element to remove.
     * @return the element previously at the specified position.
     *
     * @throws UnsupportedOperationException if the <tt>remove</tt> method is
     *		  not supported by this list.
     * @throws IndexOutOfBoundsException if the specified index is out of
     * 		  range (<tt>index &lt; 0 || index &gt;= size()</tt>).
     */
    public Object remove(int index)
    {
        DListEntry entry = (DListEntry) elements.get(index);
        TransactionImpl tx =  getCurrentTransaction();
		tx.markDelete(entry);

		return entry.getRealSubject();
    }

    private TransactionImpl getCurrentTransaction()
    {
        return (TransactionImpl) OJB.getInstance().currentTransaction();
    }

	/**
	 * Creates a new <code>DList</code> object that contains the contents of this
	 * <code>DList</code> object concatenated
	 * with the contents of the <code>otherList</code> object.
	 * @param	otherList	The list whose elements are placed at the end of the list
	 * returned by this method.
	 * @return	A new <code>DList</code> that is the concatenation of this list and
	 * the list referenced by <code>otherList</code>.
	 */
	public org.odmg.DList concat(org.odmg.DList otherList)
	{
		DListImpl result = new DListImpl();
		result.addAll(this);
		result.addAll(otherList);
		return result;
	}

	/**
	 * Determines whether there is an element of the collection that evaluates to true
	 * for the predicate.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	True if there is an element of the collection that evaluates to true
	 * for the predicate, otherwise false.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public boolean existsElement(String predicate) throws org.odmg.QueryInvalidException
	{
	    DList results = (DList) this.query(predicate);
	    if (results == null || results.size() == 0 ) return false;
	    else return true;

	}

	/**
	 * return a unique id for PK
	 */
	private int generateNewId()
	{
        PersistenceBroker broker = PersistenceBrokerFactory.createPersistenceBroker();
        try
        {
            return broker.getUniqueId(this.getClass(), "id");
        }
        catch(PersistenceBrokerException e)
        {
            return -1;
        }
    }

	/**
	 * Returns the element at the specified position in this list.
	 *
	 * @param index index of element to return.
	 * @return the element at the specified position in this list.
	 *
	 * @throws IndexOutOfBoundsException if the index is out of range (index
	 * &lt; 0 || index &gt;= size()).
	 */
	public Object get(int index)
	{
		DListEntry entry = (DListEntry) elements.get(index);
		return entry.getRealSubject();
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (10.02.2001 20:53:01)
	 * @return java.util.Vector
	 */
	java.util.Vector getElements()
	{
		return elements;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (10.02.2001 20:53:01)
	 * @return int
	 */
	int getId()
	{
		return id;
	}

	/**
	 * Returns an iterator over the elements in this collection.  There are no
	 * guarantees concerning the order in which the elements are returned
	 * (unless this collection is an instance of some class that provides a
	 * guarantee).
	 *
	 * @returns an <tt>Iterator</tt> over the elements in this collection
	 */
	public java.util.Iterator iterator()
	{
		return new DListIterator(this);
	}

	/**
	 * Returns a list iterator of the elements in this list (in proper
	 * sequence).
	 *
	 * @return a list iterator of the elements in this list (in proper
	 * sequence).
	 */
	public java.util.ListIterator listIterator()
	{
		return new DListIterator(this);
	}

	/**
	 * Returns a list iterator of the elements in this list (in proper
	 * sequence), starting at the specified position in this list.  The
	 * specified index indicates the first element that would be returned by
	 * an initial call to the <tt>next</tt> method.  An initial call to
	 * the <tt>previous</tt> method would return the element with the
	 * specified index minus one.
	 *
	 * @param index index of first element to be returned from the
	 * list iterator (by a call to the <tt>next</tt> method).
	 * @return a list iterator of the elements in this list (in proper
	 * sequence), starting at the specified position in this list.
	 * @throws IndexOutOfBoundsException if the index is out of range (index
	 * &lt; 0 || index &gt; size()).
	 */
	public java.util.ListIterator listIterator(int index)
	{
		return new DListIterator(this, index);
	}

	private Criteria getPkCriteriaForAllElements()
	{
        try
        {
            Criteria crit = null;
            for (int i = 0; i < elements.size(); i++)
            {
                DListEntry entry = (DListEntry) elements.get(i);
                Object obj = entry.getRealSubject();
                ClassDescriptor cld = DescriptorRepository.getInstance().getDescriptorFor(obj.getClass());

                FieldDescriptor[] pkFields = cld.getPkFields();
                Object[] pkValues = cld.getKeyValues(obj);

                Criteria criteria = new Criteria();
                for (int j = 0; j < pkFields.length; j++)
                {
                    FieldDescriptor fld = pkFields[j];
                    criteria.addEqualTo(fld.getPersistentField().getName(), pkValues[j]);
                }

                try
                {
                    if (crit == null) crit = criteria;
                    else crit.addOrCriteria(criteria);
                }
                catch(CriteriaException e)
                {
                }
            }
            return crit;
        }
        catch(ojb.broker.ClassNotPersistenceCapableException e)
        {
            return null;
        }
    }

    private Class getElementsExtentClass()
    {
        // we ll have to compute the most general extent class here !!!
        DListEntry entry = (DListEntry) elements.get(0);
        Class elementsClass = entry.getRealSubject().getClass();
        Class extentClass = DescriptorRepository.getInstance().getExtentClass(elementsClass);
        return extentClass;
    }

	/**
	 * Evaluate the boolean query predicate for each element of the collection and
	 * return a new collection that contains each element that evaluated to true.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	A new collection containing the elements that evaluated true for the predicate.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public org.odmg.DCollection query(String predicate) throws org.odmg.QueryInvalidException
	{
	    // 1.build complete OQL statement
        String oql = "select all from java.lang.Object where " + predicate;
        OQLQuery predicateQuery = OJB.getInstance().newOQLQuery();
        predicateQuery.create(oql);
        Query pQ = ((OQLQueryImpl)predicateQuery).getQuery();
        Criteria pCrit = pQ.getCriteria();

        Criteria allElementsCriteria = this.getPkCriteriaForAllElements();
        // join selection of elements with predicate criteria:
        allElementsCriteria.addAndCriteria(pCrit);

        Class clazz = this.getElementsExtentClass();

        Query q = new QueryByCriteria(clazz, allElementsCriteria);
        System.out.println(q.toString());


		// 2. perfom query
		// retrieve a broker instance
		PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
		// ask the broker to perfom the query
		Transaction tx = OJB.getInstance().currentTransaction();
		boolean needsCommit = false;
		if (tx == null)
		{
			tx = OJB.getInstance().newTransaction();
			tx.begin();
			needsCommit = true;
		}
		DList result = null;
        try
        {
            result = (DListImpl) broker.getCollectionByQuery(DListImpl.class, q);
        }
        catch(PersistenceBrokerException e)
        {
        }

        if (needsCommit)
		{
			tx.commit();
		}

		// 3. return resulting collection
        return result;

	}

	/**
	 * Access all of the elements of the collection that evaluate to true for the
	 * provided query predicate.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	An iterator used to iterate over the elements that evaluated true for the predicate.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public java.util.Iterator select(String predicate) throws org.odmg.QueryInvalidException
	{
	    return this.query(predicate).iterator();
	}

	/**
	 * Selects the single element of the collection for which the provided OQL query
	 * predicate is true.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return The element that evaluates to true for the predicate. If no element
	 * evaluates to true, null is returned.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public Object selectElement(String predicate) throws org.odmg.QueryInvalidException
	{
	    return ((DList) this.query(predicate)).get(0);
	}

	/**
	 * Returns the number of elements in this collection.  If this collection
	 * contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
	 * <tt>Integer.MAX_VALUE</tt>.
	 *
	 * @return the number of elements in this collection
	 */
	public int size()
	{
		return elements.size();
	}

	/**
	 * add a single Object to the Collection. This method is used during reading Collection elements
	 * from the database. Thus it is is save to cast anObject to the underlying element type of the
	 * collection.
	 */
	public void ojbAdd(java.lang.Object anObject)
	{
        this.size++;
        DListEntry entry = new DListEntry(this, anObject);
        entry.setPosition(elements.size());
        elements.add(entry);
	}

	/**
	 * adds a Collection to this collection. Used in reading Extents from the Database.
	 * Thus it is save to cast otherCollection to this.getClass().
	 */
	public void ojbAddAll(ojb.broker.ManageableCollection otherCollection)
	{
		this.addAll((DListImpl) otherCollection);
	}

	/**
	 * returns an Iterator over all elements in the collection. Used during store and delete Operations.
	 * If the implementor does not return an iterator over ALL elements, OJB cannot store and delete all elements properly.
	 */
	public java.util.Iterator ojbIterator()
	{
		return this.iterator();
	}

	/**
	 * Resize the array to have <code>newSize</code> elements.
	 * @param	newSize	The new size of the array.
	 */
	public void resize(int newSize) {}

}