/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/
/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.collections;
import java.util.Vector;
import java.util.Iterator;
import java.util.ListIterator;
import ojb.odmg.OJB;
import ojb.odmg.TransactionImpl;

/**
 * Insert the type's description here.
 * Creation date: (09.02.2001 15:49:39)
 * @author: Administrator
 */
class DListIterator implements ListIterator
{
	protected ListIterator iter;
	private DListImpl dlist;
	private DListEntry currentEntry = null;

	/**
	 * DListIterator constructor comment.
	 */
	public DListIterator(DListImpl list)
	{
		this.dlist = list;
		this.iter = list.getElements().listIterator();
	}

	/**
	 * DListIterator constructor comment.
	 */
	public DListIterator(DListImpl list, int index)
	{
		this.dlist = list;
		this.iter = list.getElements().listIterator(index);
	}

	/**
	 * Inserts the specified element into the list (optional operation).  The
	 * element is inserted immediately before the next element that would be
	 * returned by <tt>next</tt>, if any, and after the next element that
	 * would be returned by <tt>previous</tt>, if any.  (If the list contains
	 * no elements, the new element becomes the sole element on the list.)
	 * The new element is inserted before the implicit cursor: a subsequent
	 * call to <tt>next</tt> would be unaffected, and a subsequent call to
	 * <tt>previous</tt> would return the new element.  (This call increases
	 * by one the value that would be returned by a call to <tt>nextIndex</tt>
	 * or <tt>previousIndex</tt>.)
	 * 
	 * @exception UnsupportedOperationException if the <tt>add</tt> method is
	 * not supported by this list iterator.
	 * 
	 * @exception ClassCastException if the class of the specified element
	 * prevents it from being added to this Set.
	 * 
	 * @exception IllegalArgumentException if some aspect of this element
	 * prevents it from being added to this Collection.
	 */
	public void add(java.lang.Object o)
	{
		DListEntry entry = new DListEntry(this.dlist, o);
		entry.setPosition(this.nextIndex() - 1);
		iter.add(entry);

        TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
        tx.register(entry);
	}

	/**
	 * Returns <tt>true</tt> if the iteration has more elements. (In other
	 * words, returns <tt>true</tt> if <tt>next</tt> would return an element
	 * rather than throwing an exception.)
	 * 
	 * @return <tt>true</tt> if the iterator has more elements.
	 */
	public boolean hasNext()
	{
		return iter.hasNext();
	}

	/**
	 * Returns <tt>true</tt> if this list iterator has more elements when
	 * traversing the list in the reverse direction.  (In other words, returns
	 * <tt>true</tt> if <tt>previous</tt> would return an element rather than
	 * throwing an exception.)
	 * 
	 * @return <tt>true</tt> if the list iterator has more elements when
	 * traversing the list in the reverse direction.
	 */
	public boolean hasPrevious()
	{
		return iter.hasPrevious();
	}

	/**
	 * Returns the next element in the interation.
	 * 
	 * @returns the next element in the interation.
	 * @exception NoSuchElementException iteration has no more elements.
	 */
	public Object next()
	{
	    currentEntry =  ((DListEntry) iter.next());
		return currentEntry.getRealSubject();
	}

	/**
	 * Returns the index of the element that would be returned by a subsequent
	 * call to <tt>next</tt>. (Returns list size if the list iterator is at the
	 * end of the list.)
	 * 
	 * @return the index of the element that would be returned by a subsequent
	 * call to <tt>next</tt>, or list size if list iterator is at end
	 * of list.
	 */
	public int nextIndex()
	{
		return iter.nextIndex();
	}

	/**
	 * Returns the previous element in the list.  This method may be called
	 * repeatedly to iterate through the list backwards, or intermixed with
	 * calls to <tt>next</tt> to go back and forth.  (Note that alternating
	 * calls to <tt>next</tt> and <tt>previous</tt> will return the same
	 * element repeatedly.)
	 * 
	 * @return the previous element in the list.
	 * 
	 * @exception NoSuchElementException if the iteration has no previous
	 * element.
	 */
	public java.lang.Object previous()
	{
	    currentEntry =  ((DListEntry) iter.previous());
		return currentEntry.getRealSubject();
	}

	/**
	 * Returns the index of the element that would be returned by a subsequent
	 * call to <tt>previous</tt>. (Returns -1 if the list iterator is at the
	 * beginning of the list.)
	 * 
	 * @return the index of the element that would be returned by a subsequent
	 * call to <tt>previous</tt>, or -1 if list iterator is at
	 * beginning of list.
	 */
	public int previousIndex()
	{
		return iter.previousIndex();
	}

	/**
	 * 
	 * Removes from the underlying collection the last element returned by the
	 * iterator (optional operation).  This method can be called only once per
	 * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
	 * the underlying collection is modified while the iteration is in
	 * progress in any way other than by calling this method.
	 * 
	 * @exception UnsupportedOperationException if the <tt>remove</tt>
	 * operation is not supported by this Iterator.
	 * 
	 * @exception IllegalStateException if the <tt>next</tt> method has not
	 * yet been called, or the <tt>remove</tt> method has already
	 * been called after the last call to the <tt>next</tt>
	 * method.
	 */
	public void remove()
	{
		iter.remove();
		TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
        tx.markDelete(currentEntry);
        currentEntry = null;
	}

	/**
	 * Replaces the last element returned by <tt>next</tt> or
	 * <tt>previous</tt> with the specified element (optional operation).
	 * This call can be made only if neither <tt>ListIterator.remove</tt> nor
	 * <tt>ListIterator.add</tt> have been called after the last call to
	 * <tt>next</tt> or <tt>previous</tt>.
	 * 
	 * @exception UnsupportedOperationException if the <tt>set</tt> operation
	 * is not supported by this list iterator.
	 * @exception ClassCastException if the class of the specified element
	 * prevents it from being added to this list.
	 * @exception IllegalArgumentException if some aspect of the specified
	 * element prevents it from being added to this list.
	 * @exception IllegalStateException if neither <tt>next</tt> nor
	 * <tt>previous</tt> have been called, or <tt>remove</tt> or
	 * <tt>add</tt> have been called after the last call to
	 * <tt>next</tt> or <tt>previous</tt>.
	 */
	public void set(java.lang.Object o)
	{
		currentEntry.setRealSubject(o);
	}

}