/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/
/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.collections;
import org.odmg.*;
import java.util.Iterator;
import ojb.broker.*;
import ojb.odmg.*;
import java.util.Vector;
import ojb.odmg.oql.*;
import ojb.broker.query.*;
import ojb.broker.metadata.*;


/**
 * Insert the type's description here.
 * Creation date: (26.02.2001 13:05:03)
 * @author: Administrator
 */
public class DSetImpl extends java.util.AbstractSet implements org.odmg.DSet
{
	private int id;
	private Vector elements;
	private int size;

	/**
	 * DSetImpl constructor comment.
	 */
	public DSetImpl()
	{
		super();
		elements = new Vector();
		id = generateNewId();
		this.size = 0;
	}

	/**
	 * DListImpl constructor comment.
	 */
	public DSetImpl(int theId, int theSize)
	{
		id = theId;
		this.size = theSize;
		TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
		tx.register(this);
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param o
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean add(Object o)
	{
		if (!this.contains(o))
		{
			TransactionImpl tx = (TransactionImpl) OJB.getInstance().currentTransaction();
			tx.register(this);
			this.size++;
			DSetEntry entry = new DSetEntry(this, o);
			elements.add(entry);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Create a new <code>DSet</code> object that contains the elements of this
	 * collection minus the elements in <code>otherSet</code>.
	 * @param	otherSet	A set containing elements that should not be in the result set.
	 * @return	A newly created <code>DSet</code> instance that contains the elements
	 * of this set minus those elements in <code>otherSet</code>.
	 */
	public org.odmg.DSet difference(org.odmg.DSet otherSet)
	{
		DSetImpl result = new DSetImpl();
		Iterator iter = this.iterator();
		while (iter.hasNext())
		{
			Object candidate = iter.next();
			if (!otherSet.contains(candidate))
			{
				result.add(candidate);
			}
		}
		return result;
	}

	/**
	 * Determines whether there is an element of the collection that evaluates to true
	 * for the predicate.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	True if there is an element of the collection that evaluates to true
	 * for the predicate, otherwise false.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public boolean existsElement(String predicate) throws org.odmg.QueryInvalidException
	{
        DList results = (DList) this.query(predicate);
        if (results == null || results.size() == 0 ) return false;
        else return true;
	}

	/**
	 * return a unique id for PK
	 */
	private int generateNewId()
	{
        PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();
        try
        {
            return broker.getUniqueId(DListImpl.class, "id");
        }
        catch(PersistenceBrokerException e)
        {
            return -1;
        }
    }

	/**
	 * Insert the method's description here.
	 * Creation date: (26.02.2001 13:29:42)
	 * @return java.util.Vector
	 */
	java.util.Vector getElements()
	{
		return elements;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (26.02.2001 13:50:57)
	 * @return int
	 */
	int getId()
	{
		return id;
	}

	/**
	 * Create a new <code>DSet</code> object that is the set intersection of this
	 * <code>DSet</code> object and the set referenced by <code>otherSet</code>.
	 * @param	otherSet	The other set to be used in the intersection operation.
	 * @return	A newly created <code>DSet</code> instance that contains the
	 * intersection of the two sets.
	 */
	public org.odmg.DSet intersection(org.odmg.DSet otherSet)
	{
		DSet union = this.union(otherSet);
		DSetImpl result = new DSetImpl();
		Iterator iter = union.iterator();
		while (iter.hasNext())
		{
			Object candidate = iter.next();
			if (this.contains(candidate) && otherSet.contains(candidate))
			{
				result.add(candidate);
			}
		}
		return result;
	}

	/**
	 * Returns an iterator over the elements in this collection.  There are no
	 * guarantees concerning the order in which the elements are returned
	 * (unless this collection is an instance of some class that provides a
	 * guarantee).
	 * 
	 * @returns an <tt>Iterator</tt> over the elements in this collection
	 */
	public java.util.Iterator iterator()
	{
		return new DSetIterator(this);
	}

	/**
	 * Determine whether this set is a proper subset of the set referenced by
	 * <code>otherSet</code>.
	 * @param	otherSet	Another set.
	 * @return True if this set is a proper subset of the set referenced by
	 * <code>otherSet</code>, otherwise false.
	 */
	public boolean properSubsetOf(org.odmg.DSet otherSet)
	{
		return (this.size() > 0 && this.size() < otherSet.size() && this.subsetOf(otherSet));
	}

	/**
	 * Determine whether this set is a proper superset of the set referenced by
	 * <code>otherSet</code>.
	 * @param	otherSet	Another set.
	 * @return True if this set is a proper superset of the set referenced by
	 * <code>otherSet</code>, otherwise false.
	 */
	public boolean properSupersetOf(org.odmg.DSet otherSet)
	{
		return (otherSet.size() > 0 && otherSet.size() < this.size() && this.supersetOf(otherSet));
	}

	/**
	 * Evaluate the boolean query predicate for each element of the collection and
	 * return a new collection that contains each element that evaluated to true.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	A new collection containing the elements that evaluated true for the predicate.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public org.odmg.DCollection query(String predicate) throws org.odmg.QueryInvalidException
	{
        // 1.build complete OQL statement
        String oql = "select all from java.lang.Object where " + predicate;
        OQLQuery predicateQuery = OJB.getInstance().newOQLQuery();
        predicateQuery.create(oql);
        Query pQ = ((OQLQueryImpl)predicateQuery).getQuery();
        Criteria pCrit = pQ.getCriteria();

        Criteria allElementsCriteria = this.getPkCriteriaForAllElements();
        // join selection of elements with predicate criteria:
        pCrit.addAndCriteria(allElementsCriteria);

        Class clazz = this.getElementsExtentClass();

        Query q = new QueryByCriteria(clazz, pCrit);
        System.out.println(q.toString());


		// 2. perfom query
		// retrieve a broker instance
		PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
		// ask the broker to perfom the query
		Transaction tx = OJB.getInstance().currentTransaction();
		boolean needsCommit = false;
		if (tx == null)
		{
			tx = OJB.getInstance().newTransaction();
			tx.begin();
			needsCommit = true;
		}
		DList result = null;
        try
        {
            result = (DListImpl) broker.getCollectionByQuery(DListImpl.class, q);
        }
        catch(PersistenceBrokerException e)
        {
        }

        if (needsCommit)
		{
			tx.commit();
		}

		// 3. return resulting collection
        return result;

	}

    private Criteria getPkCriteriaForAllElements()
	{
        try
        {
            Criteria crit = null;
            for (int i = 0; i < elements.size(); i++)
            {
                DListEntry entry = (DListEntry) elements.get(i);
                Object obj = entry.getRealSubject();
                ClassDescriptor cld = DescriptorRepository.getInstance().getDescriptorFor(obj.getClass());

                FieldDescriptor[] pkFields = cld.getPkFields();
                Object[] pkValues = cld.getKeyValues(obj);

                Criteria criteria = new Criteria();
                for (int j = 0; j < pkFields.length; j++)
                {
                    FieldDescriptor fld = pkFields[j];
                    criteria.addEqualTo(fld.getPersistentField().getName(), pkValues[j]);
                }

                try
                {
                    if (crit == null) crit = criteria;
                    else crit.addOrCriteria(criteria);
                }
                catch(CriteriaException e)
                {
                }
            }
            return crit;
        }
        catch(ojb.broker.ClassNotPersistenceCapableException e)
        {
            return null;
        }
    }

    private Class getElementsExtentClass()
    {
        // we ll have to compute the most general extent class here !!!
        DListEntry entry = (DListEntry) elements.get(0);
        Class elementsClass = entry.getRealSubject().getClass();
        Class extentClass = DescriptorRepository.getInstance().getExtentClass(elementsClass);
        return extentClass;
    }


	/**
	 * Access all of the elements of the collection that evaluate to true for the
	 * provided query predicate.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return	An iterator used to iterate over the elements that evaluated true for the predicate.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public java.util.Iterator select(String predicate) throws org.odmg.QueryInvalidException
	{
        return this.query(predicate).iterator();
	}

	/**
	 * Selects the single element of the collection for which the provided OQL query
	 * predicate is true.
	 * @param	predicate	An OQL boolean query predicate.
	 * @return The element that evaluates to true for the predicate. If no element
	 * evaluates to true, null is returned.
	 * @exception	QueryInvalidException	The query predicate is invalid.
	 */
	public Object selectElement(String predicate) throws org.odmg.QueryInvalidException
	{
        return ((DList) this.query(predicate)).get(0);
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (26.02.2001 13:29:42)
	 * @param newElements java.util.Vector
	 */
	void setElements(java.util.Vector newElements)
	{
		elements = newElements;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (26.02.2001 13:50:57)
	 * @param newId int
	 */
	void setId(int newId)
	{
		id = newId;
	}

	/**
	 * Returns the number of elements in this collection.  If this collection
	 * contains more than <tt>Integer.MAX_VALUE</tt> elements, returns
	 * <tt>Integer.MAX_VALUE</tt>.
	 * 
	 * @return the number of elements in this collection
	 */
	public int size()
	{
		return elements.size();
	}

	/**
	 * Determine whether this set is a subset of the set referenced by <code>otherSet</code>.
	 * @param	otherSet	Another set.
	 * @return True if this set is a subset of the set referenced by <code>otherSet</code>,
	 * otherwise false.
	 */
	public boolean subsetOf(org.odmg.DSet otherSet)
	{
		return otherSet.containsAll(this);
	}

	/**
	 * Determine whether this set is a superset of the set referenced by <code>otherSet</code>.
	 * @param	otherSet	Another set.
	 * @return True if this set is a superset of the set referenced by <code>otherSet</code>,
	 * otherwise false.
	 */
	public boolean supersetOf(org.odmg.DSet otherSet)
	{
		return this.containsAll(otherSet);
	}

	/**
	 * Create a new <code>DSet</code> object that is the set union of this
	 * <code>DSet</code> object and the set referenced by <code>otherSet</code>.
	 * @param	otherSet	The other set to be used in the union operation.
	 * @return	A newly created <code>DSet</code> instance that contains the union of the two sets.
	 */
	public org.odmg.DSet union(org.odmg.DSet otherSet)
	{
		DSetImpl result = new DSetImpl();
		result.addAll(this);
		result.addAll(otherSet);
		return result;
	}


}