// $ANTLR 2.7.1: "oql-ojb.g" -> "OQLParser.java"$


/*
 * ObJectBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/**
  * This OQL grammar has been derived from a OQL sample grammar from the ODMG WebSite.
  * The original grammar is copyright protected by MicroData Base Systems:
  * Copyright (c) 1999 Micro Data Base Systems, Inc. All rights reserved. 
  *
  * The original grammar has been modified to fit into the ObJectBridge (OJB) 
  * Persistence Managment System.
  * 
  * modifications done by Ch. Rath & Th. Mahler
  *
  * This grammar can be used to build an OQL Parser with the ANTLR Parser 
  * construction set.
  * The grammar defines a Parser that translates valid OQL Strings
  * into ojb.broker.query.Query Objects. These query objects can be used
  * to perform database queries by means of the OJB PersistenceBroker.
  * @see ojb.server.oql.OQLQueryImpl for implementation details.
  */

package ojb.odmg.oql;
 
import ojb.broker.query.*; 
 

import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;
import antlr.collections.AST;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

public class OQLParser extends antlr.LLkParser
       implements OQLLexerTokenTypes
 {

	Class m_ClassToSearchFrom = null;

protected OQLParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public OQLParser(TokenBuffer tokenBuf) {
  this(tokenBuf,2);
}

protected OQLParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public OQLParser(TokenStream lexer) {
  this(lexer,2);
}

public OQLParser(ParserSharedInputState state) {
  super(state,2);
  tokenNames = _tokenNames;
}

	public final Query  buildQuery() throws RecognitionException, TokenStreamException {
		Query result = null;
		
		
		try {      // for error handling
			result=query();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
		return result;
	}
	
	public final Query  query() throws RecognitionException, TokenStreamException {
		Query result = null;
		
		
		try {      // for error handling
			Criteria criteria = null;
			criteria=selectExpr();
			
					if ((m_ClassToSearchFrom != null) && (criteria != null)) {
						result = QueryFactory.newQuery(m_ClassToSearchFrom, criteria); 
					}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
		return result;
	}
	
	public final Criteria  selectExpr() throws RecognitionException, TokenStreamException {
		Criteria rc = null;
		
		
		try {      // for error handling
			match(LITERAL_select);
			projectionAttributes();
			rc = new Criteria();
			fromClause();
			{
			if ((LA(1)==LITERAL_where)) {
				whereClause(rc);
			}
			else if ((LA(1)==EOF||LA(1)==LITERAL_order)) {
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
			{
			if ((LA(1)==LITERAL_order)) {
				orderClause(rc);
			}
			else if ((LA(1)==EOF)) {
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
		return rc;
	}
	
	public final void projectionAttributes() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			if ((LA(1)==Identifier)) {
				projection();
				{
				_loop79:
				do {
					if ((LA(1)==TOK_COMMA)) {
						match(TOK_COMMA);
						projection();
					}
					else {
						break _loop79;
					}
					
				} while (true);
				}
			}
			else if ((LA(1)==TOK_STAR)) {
				match(TOK_STAR);
			}
			else {
				throw new NoViableAltException(LT(1), getFilename());
			}
			
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_1);
		}
	}
	
	public final void fromClause() throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			match(LITERAL_from);
			id = LT(1);
			match(Identifier);
			
					m_ClassToSearchFrom = null;
				try
				{
						m_ClassToSearchFrom = Class.forName(id.getText());
					}
					catch (Exception ex) 
					{}
				
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_2);
		}
	}
	
	public final void whereClause(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(LITERAL_where);
			orExpr(pCriteria);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_3);
		}
	}
	
	public final void orderClause(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(LITERAL_order);
			match(LITERAL_by);
			sortCriterion(pCriteria);
			{
			_loop83:
			do {
				if ((LA(1)==TOK_COMMA)) {
					match(TOK_COMMA);
					sortCriterion(pCriteria);
				}
				else {
					break _loop83;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_0);
		}
	}
	
	public final void orExpr(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			andExpr(pCriteria);
			{
			_loop88:
			do {
				if ((LA(1)==LITERAL_or)) {
					match(LITERAL_or);
					Criteria orCriteria = new Criteria();
					andExpr(orCriteria);
					
						 	 try 
							 {
							 	pCriteria.addOrCriteria(orCriteria);
							 } 
							 catch (Exception e)
							 {} 
							
				}
				else {
					break _loop88;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_3);
		}
	}
	
	public final void projection() throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			match(Identifier);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_4);
		}
	}
	
	public final void sortCriterion(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			id = LT(1);
			match(Identifier);
			{
			switch ( LA(1)) {
			case LITERAL_asc:
			{
				match(LITERAL_asc);
				break;
			}
			case LITERAL_desc:
			{
				match(LITERAL_desc);
				break;
			}
			case EOF:
			case TOK_COMMA:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			pCriteria.orderBy(id.getText());
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_5);
		}
	}
	
	public final void andExpr(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			quantifierExpr(pCriteria);
			{
			_loop91:
			do {
				if ((LA(1)==LITERAL_and)) {
					match(LITERAL_and);
					Criteria andCriteria = new Criteria();
					quantifierExpr(andCriteria);
					pCriteria.addAndCriteria(andCriteria);
				}
				else {
					break _loop91;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_6);
		}
	}
	
	public final void quantifierExpr(
		Criteria pCriteria
	) throws RecognitionException, TokenStreamException {
		
		Token  id = null;
		
		try {      // for error handling
			
				int index = 1;
				String[] args;
				int op = 0; 
				Object obj;
				String ident;
				
			id = LT(1);
			match(Identifier);
			
					ident = id.getText();	
				
			{
			_loop96:
			do {
				if ((_tokenSet_7.member(LA(1)))) {
					op = 0;
					{
					switch ( LA(1)) {
					case TOK_EQ:
					{
						match(TOK_EQ);
						op = 1;
						break;
					}
					case TOK_NE:
					{
						match(TOK_NE);
						op = 2;
						break;
					}
					case TOK_LT:
					{
						match(TOK_LT);
						op = 3;
						break;
					}
					case TOK_GT:
					{
						match(TOK_GT);
						op = 4;
						break;
					}
					case TOK_LE:
					{
						match(TOK_LE);
						op = 5;
						break;
					}
					case TOK_GE:
					{
						match(TOK_GE);
						op = 6;
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					{
					obj=literal();
					
									switch (op) {
										case 1:
											pCriteria.addEqualTo(ident, obj);
										break;
					
										case 2:
											pCriteria.addNotEqualTo(ident, obj);
										break;
					
										case 3:
											pCriteria.addLessThan(ident, obj);
										break;
					
										case 4:
											pCriteria.addGreaterThan(ident, obj);
										break;
					
										case 5:
											pCriteria.addLessOrEqualThan(ident, obj);
										break;
					
										case 6:
											pCriteria.addGreaterOrEqualThan(ident, obj);
										break;
					
										default:
										break;
									}
								
					}
				}
				else if ((LA(1)==LITERAL_like)) {
					match(LITERAL_like);
					obj=literal();
					
								pCriteria.addLike(ident, obj);
							
				}
				else {
					break _loop96;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_8);
		}
	}
	
	public final Object  literal() throws RecognitionException, TokenStreamException {
		Object obj = null;
		
		Token  tokInt = null;
		Token  tokDec = null;
		Token  tokDec2 = null;
		Token  tokStr = null;
		Token  tokDate = null;
		Token  tokTime = null;
		Token  tokTS = null;
		
		try {      // for error handling
			switch ( LA(1)) {
			case LITERAL_nil:
			{
				match(LITERAL_nil);
				obj = null;
				break;
			}
			case LITERAL_true:
			case LITERAL_false:
			{
				{
				if ((LA(1)==LITERAL_true)) {
					match(LITERAL_true);
					obj = new Boolean(true);
				}
				else if ((LA(1)==LITERAL_false)) {
					match(LITERAL_false);
					obj = new Boolean(false);
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				break;
			}
			case TOK_PARAMETER:
			{
				match(TOK_PARAMETER);
				
						        obj = new Object();
						
				break;
			}
			case INT_LITERAL:
			{
				tokInt = LT(1);
				match(INT_LITERAL);
				
								// was passiert wenn Wert zu gross ist?
								obj = new Integer(tokInt.getText());
							
				break;
			}
			case TOK_APPROXIMATE_NUMERIC_LITERAL:
			case TOK_EXACT_NUMERIC_LITERAL:
			{
				{
				if ((LA(1)==TOK_APPROXIMATE_NUMERIC_LITERAL)) {
					tokDec = LT(1);
					match(TOK_APPROXIMATE_NUMERIC_LITERAL);
					obj = new Double(tokDec.getText());
				}
				else if ((LA(1)==TOK_EXACT_NUMERIC_LITERAL)) {
					tokDec2 = LT(1);
					match(TOK_EXACT_NUMERIC_LITERAL);
					obj = new Double(tokDec2.getText());
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				break;
			}
			case StringLiteral:
			{
				tokStr = LT(1);
				match(StringLiteral);
				obj = new String(tokStr.getText());
				break;
			}
			case LITERAL_date:
			{
				match(LITERAL_date);
				tokDate = LT(1);
				match(StringLiteral);
				
								// Format yyyy-mm-dd
								obj = java.sql.Date.valueOf(tokDate.getText());
							
				break;
			}
			case LITERAL_time:
			{
				match(LITERAL_time);
				tokTime = LT(1);
				match(StringLiteral);
				
								// Format hh:mm:ss
								obj = java.sql.Time.valueOf(tokTime.getText());
							
				break;
			}
			case LITERAL_timestamp:
			{
				match(LITERAL_timestamp);
				tokTS = LT(1);
				match(StringLiteral);
				
								// Format yyyy-mm-dd hh:mm:ss.fffffffff
								obj = java.sql.Timestamp.valueOf(tokTS.getText());
							
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			consume();
			consumeUntil(_tokenSet_9);
		}
		return obj;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"right parenthesis",
		"left parenthesis",
		"comma",
		"semicolon",
		"TOK_DOTDOT",
		"TOK_COLON",
		"dot",
		"dot",
		"operator",
		"comparison operator",
		"operator",
		"operator",
		"operator",
		"operator",
		"comparison operator",
		"comparison operator",
		"comparison operator",
		"comparison operator",
		"comparison operator",
		"left bracket",
		"right bracket",
		"TOK_PARAMETER",
		"NameFirstCharacter",
		"NameCharacter",
		"Identifier",
		"INT_LITERAL",
		"TOK_UNSIGNED_INTEGER",
		"TOK_APPROXIMATE_NUMERIC_LITERAL",
		"numeric value",
		"character string",
		"character string",
		"WhiteSpace",
		"NewLine",
		"CommentLine",
		"MultiLineComment",
		"\"select\"",
		"\"from\"",
		"\"where\"",
		"\"order\"",
		"\"by\"",
		"\"asc\"",
		"\"desc\"",
		"\"or\"",
		"\"and\"",
		"\"like\"",
		"\"nil\"",
		"\"true\"",
		"\"false\"",
		"\"date\"",
		"\"time\"",
		"\"timestamp\""
	};
	
	private static final long _tokenSet_0_data_[] = { 2L, 0L };
	public static final BitSet _tokenSet_0 = new BitSet(_tokenSet_0_data_);
	private static final long _tokenSet_1_data_[] = { 1099511627776L, 0L };
	public static final BitSet _tokenSet_1 = new BitSet(_tokenSet_1_data_);
	private static final long _tokenSet_2_data_[] = { 6597069766658L, 0L };
	public static final BitSet _tokenSet_2 = new BitSet(_tokenSet_2_data_);
	private static final long _tokenSet_3_data_[] = { 4398046511106L, 0L };
	public static final BitSet _tokenSet_3 = new BitSet(_tokenSet_3_data_);
	private static final long _tokenSet_4_data_[] = { 1099511627840L, 0L };
	public static final BitSet _tokenSet_4 = new BitSet(_tokenSet_4_data_);
	private static final long _tokenSet_5_data_[] = { 66L, 0L };
	public static final BitSet _tokenSet_5 = new BitSet(_tokenSet_5_data_);
	private static final long _tokenSet_6_data_[] = { 74766790688770L, 0L };
	public static final BitSet _tokenSet_6 = new BitSet(_tokenSet_6_data_);
	private static final long _tokenSet_7_data_[] = { 8134656L, 0L };
	public static final BitSet _tokenSet_7 = new BitSet(_tokenSet_7_data_);
	private static final long _tokenSet_8_data_[] = { 215504279044098L, 0L };
	public static final BitSet _tokenSet_8 = new BitSet(_tokenSet_8_data_);
	private static final long _tokenSet_9_data_[] = { 496979263889410L, 0L };
	public static final BitSet _tokenSet_9 = new BitSet(_tokenSet_9_data_);
	
	}
