/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.oql;

import ojb.odmg.collections.DListImpl;
import org.odmg.*;
import java.util.Collection;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import ojb.odmg.*;
import ojb.broker.*;
import ojb.broker.metadata.ClassDescriptor;
import java.io.StringReader;
import org.odmg.OQLQuery;
import ojb.broker.query.*;
import antlr.RecognitionException;
import antlr.TokenStreamException;
/**
 * Insert the type's description here.
 * Creation date: (08.05.01 13:12:58)
 * @author: Administrator
 */
public class OQLQueryImpl implements org.odmg.OQLQuery {

    /**
     * holds the query string of representing this query
     */
	private String queryString = null;
	/**
	* holds the compiled query object
	*/

    public Query getQuery()
    {
        return query;
    }

    private Query query = null;

	private Iterator bindIterator = null;
/**
 * OQLQueryImpl constructor comment.
 */
public OQLQueryImpl() {
	super();
}
/**
* Bind a parameter to the query.
* A parameter is denoted in the query string passed to <code>create</code> by <i>$i</i>,
* where <i>i</i> is the rank of the parameter, beginning with 1.
* The parameters are set consecutively by calling this method <code>bind</code>.
* The <i>ith</i> variable is set by the <i>ith</i> call to the <code>bind</code> method.
* If any of the <i>$i</i> are not set by a call to <code>bind</code> at the point
* <code>execute</code> is called, <code>QueryParameterCountInvalidException</code> is thrown.
* The parameters must be objects, and the result is an <code>Object</code>.
* Objects must be used instead of primitive types (<code>Integer</code> instead
* of <code>int</code>) for passing the parameters.
* <p>
* If the parameter is of the wrong type,
* <code>QueryParameterTypeInvalidException</code> is thrown.
* After executing a query, the parameter list is reset.
* @parameter	A value to be substituted for a query parameter.
* @exception QueryParameterCountInvalidException The number of calls to
* <code>bind</code> has exceeded the number of parameters in the query.
* @exception QueryParameterTypeInvalidException The type of the parameter does
* not correspond with the type of the parameter in the query.
*/
public void bind(Object parameter) throws org.odmg.QueryParameterCountInvalidException, org.odmg.QueryParameterTypeInvalidException
{
    try
    {
        SelectionCriteria crit = (SelectionCriteria) this.bindIterator.next();
        crit.bind(parameter);
    }
    catch(Exception e)
    {
        throw new org.odmg.QueryParameterCountInvalidException(e.getMessage());
    }
}

private Vector flatten(Criteria crit, Vector acc)
{
	Enumeration e = crit.getElements();
	while (e.hasMoreElements())
	{
		Object o = e.nextElement();
		if (o instanceof Criteria)
		{
			Criteria pc = (Criteria) o;
			flatten(pc, acc);
		}
		else
		{
			SelectionCriteria c = (SelectionCriteria) o;
			acc.add(c);
        }
    }
    return acc;
}




/**
* Create an OQL query from the string parameter.
* In order to execute a query, an <code>OQLQuery</code> object must be created
* by calling <code>Implementation.newOQLQuery</code>, then calling the
* <code>create</code> method with the query string.
* The <code>create</code> method might throw <code>QueryInvalidException</code>
* if the query could not be compiled properly. Some implementations may not want
* to compile the query before <code>execute</code> is called. In this case
* <code>QueryInvalidException</code> is thrown when <code>execute</code> is called.
* @param	query	An OQL query.
* @exception	QueryInvalidException	The query syntax is invalid.
*/
public void create(String query) throws org.odmg.QueryInvalidException 
{

    try
    {
        queryString = query;
        // Use the OQL parser to transform a query string to a valid ojb.broker.query object
        StringReader reader = new StringReader(queryString);
        OQLLexer lexer = new OQLLexer(reader);
        OQLParser parser = new OQLParser(lexer);
        this.query = parser.buildQuery();

        this.bindIterator = this.flatten(this.query.getCriteria(), new Vector()).iterator();
    }
    catch(RecognitionException e)
    {
        throw new QueryInvalidException(e.getMessage());
    }
    catch(TokenStreamException e)
    {
        throw new QueryInvalidException(e.getMessage());
    }


}
/**
* Execute the query.
* After executing a query, the parameter list is reset.
* Some implementations may throw additional exceptions that are also derived
* from <code>ODMGException</code>.
* @return	The object that represents the result of the query.
* The returned data, whatever its OQL type, is encapsulated into an object.
* For instance, when OQL returns an integer, the result is put into an
* <code>Integer</code> object. When OQL returns a collection (literal or object),
* the result is always a Java collection object of the same kind
* (for instance, a <code>DList</code>).
* @exception	QueryException	An exception has occurred while executing the query.
*/
public Object execute() throws org.odmg.QueryException {
	try 
	{

		// retrieve a broker instance
		PersistenceBroker broker = ((DatabaseImpl) OJB.getInstance().getDatabase(null)).getBroker();
		// ask the broker to perfom the query
		Transaction tx = OJB.getInstance().currentTransaction();
		boolean needsCommit = false;
		if (tx == null)
		{
			tx = OJB.getInstance().newTransaction();
			tx.begin();
			needsCommit = true;
		}
		DList result = (DListImpl) broker.getCollectionByQuery(DListImpl.class, query);
		
		if (needsCommit)
		{
			tx.commit();
		}

		return result;

	} catch (Exception e) {
		System.err.println("parser exception: " + e);
		// so we can get stack trace               
		e.printStackTrace();
	   throw new org.odmg.QueryException();
	}
	
}
}
