/* ====================================================================
 * OQL Sample Grammar for Object Data Management Group (ODMG) 
 *
 * Copyright (c) 1999 Micro Data Base Systems, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by Micro Data Base Systems, Inc.
 *    (http://www.mdbs.com) for the use of the Object Data Management Group
 *    (http://www.odmg.org/)."
 *
 * 4. The names "mdbs" and "Micro Data Base Systems" must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission. For written permission, please contact
 *    info@mdbs.com.
 *
 * 5. Products derived from this software may not be called "mdbs"
 *    nor may "mdbs" appear in their names without prior written
 *    permission of Micro Data Base Systems, Inc. 
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by Micro Data Base Systems, Inc.
 *    (http://www.mdbs.com) for the use of the Object Data Management Group
 *    (http://www.odmg.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY MICRO DATA BASE SYSTEMS, INC. ``AS IS'' AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL MICRO DATA BASE SYSTEMS, INC. OR
 * ITS ASSOCIATES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


/* 
**  oql-ojb.g 
** 
** Built with Antlr 2.7.1 
**   java antlr.Tool oql-ojb.g 
*/ 
header{

/*
 * ObJectBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

/**
  * This OQL grammar has been derived from a OQL sample grammar from the ODMG WebSite.
  * The original grammar is copyright protected by MicroData Base Systems:
  * Copyright (c) 1999 Micro Data Base Systems, Inc. All rights reserved. 
  *
  * The original grammar has been modified to fit into the ObJectBridge (OJB) 
  * Persistence Managment System.
  * 
  * modifications done by Ch. Rath & Th. Mahler
  *
  * This grammar can be used to build an OQL Parser with the ANTLR Parser 
  * construction set.
  * The grammar defines a Parser that translates valid OQL Strings
  * into ojb.broker.query.Query Objects. These query objects can be used
  * to perform database queries by means of the OJB PersistenceBroker.
  * @see ojb.server.oql.OQLQueryImpl for implementation details.
  */

package ojb.odmg.oql;
 
import ojb.broker.query.*; 
 
} 


//-----------------------------
// Lexer
//-----------------------------

class OQLLexer extends Lexer ;
options {
    k=2;
    charVocabulary = '\3'..'\377';
    testLiterals=false;     // don't automatically test for literals
    caseSensitive=true;
    caseSensitiveLiterals=true;
}

/* punctuation */
TOK_RPAREN
    options {
        paraphrase = "right parenthesis";
    }                       :       ')'             ;
TOK_LPAREN
    options {
        paraphrase = "left parenthesis";
    }                       :       '('             ;
TOK_COMMA
    options {
        paraphrase = "comma";
    }                       :       ','             ;
TOK_SEMIC
    options {
        paraphrase = "semicolon";
    }                       :       ';'             ;

TOK_DOTDOT  :   ".." ;
TOK_COLON   :   ':' ;
// protected - removed to fix bug with '.' 10/4/99
TOK_DOT
    options {
        paraphrase = "dot";
    }                       :       '.'             ;
TOK_INDIRECT
    options {
        paraphrase = "dot";
    }                       :       '-' '>'         { $setType(TOK_DOT); } ;
TOK_CONCAT
    options {
        paraphrase = "operator";
    }                       :       '|' '|'         ;
TOK_EQ
    options {
        paraphrase = "comparison operator";
    }                       :       '='             ;
TOK_PLUS
    options {
        paraphrase = "operator";
    }                       :       '+'             ;
TOK_MINUS
    options {
        paraphrase = "operator";
    }                       :       '-'             ;
TOK_SLASH
    options {
        paraphrase = "operator";
    }                       :       '/'             ;
TOK_STAR
    options {
        paraphrase = "operator";
    }                       :       '*'             ;
TOK_LE
    options {
        paraphrase = "comparison operator";
    }                       :       '<' '='         ;
TOK_GE
    options {
        paraphrase = "comparison operator";
    }                       :       '>' '='         ;
TOK_NE
    options {
        paraphrase = "comparison operator";
    }                       :       '<' '>'         ;
TOK_LT
    options {
        paraphrase = "comparison operator";
    }                       :       '<'             ;
TOK_GT
    options {
        paraphrase = "comparison operator";
    }                       :       '>'             ;
TOK_LBRACK
    options {
        paraphrase = "left bracket";
    }                       :       '['             ;
TOK_RBRACK
    options {
        paraphrase = "right bracket";
    }                       :       ']'             ;


TOK_PARAMETER
    : 	    '$' ('0'..'9')+ ;

/*
 * Names
 */
protected
NameFirstCharacter:
        ( 'A'..'Z' | 'a'..'z' | '_' | '.' )
    ;

protected
NameCharacter:
        ( 'A'..'Z' |'a'..'z' | '_' | '.' | '0'..'9' )
    ;

Identifier
        options {testLiterals=true;}
            :

        NameFirstCharacter
        ( NameCharacter )*
    ;

INT_LITERAL :
	('+' | '-')? ('0'..'9')+
	;

/* Numbers */
protected
TOK_UNSIGNED_INTEGER :
        '0'..'9'
    ;


// a couple protected methods to assist in matching floating point numbers
protected
TOK_APPROXIMATE_NUMERIC_LITERAL
        :       'e' ('+'|'-')? ('0'..'9')+
	;


// a numeric literal
TOK_EXACT_NUMERIC_LITERAL
    options {
        paraphrase = "numeric value";
    }                       :

            '.'
            (
                TOK_UNSIGNED_INTEGER
            )+
            { _ttype = TOK_EXACT_NUMERIC_LITERAL; }

            (
                TOK_APPROXIMATE_NUMERIC_LITERAL
                { _ttype = TOK_APPROXIMATE_NUMERIC_LITERAL; }
            )?

        |   (
                TOK_UNSIGNED_INTEGER
            )+
            { _ttype = TOK_UNSIGNED_INTEGER; }

            // only check to see if it's a float if looks like decimal so far
            (
                '.'
                (
                    TOK_UNSIGNED_INTEGER
                )*
                { _ttype = TOK_EXACT_NUMERIC_LITERAL; }
                (TOK_APPROXIMATE_NUMERIC_LITERAL { _ttype = TOK_APPROXIMATE_NUMERIC_LITERAL; } )?

            |    TOK_APPROXIMATE_NUMERIC_LITERAL { _ttype = TOK_APPROXIMATE_NUMERIC_LITERAL; }
            )
	;


/*
** Define a lexical rule to handle strings.
*/
CharLiteral
    options {
        paraphrase = "character string";
    }                       
  :
        '\''!
        (
            '\'' '\''           { $setText("'"); }
        |   '\n'                { newline(); }
        |   ~( '\'' | '\n' )
        )*
        '\''!
    ;

StringLiteral
    options {
        paraphrase = "character string";
    }                       
  :

        '"'!
        (
            '\\' '"'            { $setText("\""); }
        |   '\n'                { newline(); }
        |   ~( '\"' | '\n' )
        )*
        '"'!
    ;

/*
** Define white space so that we can throw out blanks
*/
WhiteSpace :
        (
            ' '
        |   '\t'
        |   '\r'
        )   { $setType(Token.SKIP); }
    ;

NewLine :
        '\n'    { newline(); $setType(Token.SKIP); }
    ;

/*
** Define a lexical rule to handle line comments.
*/
CommentLine :
        '/'! '/'!
        (
            ~'\n'!
        )*
        '\n'!   { newline(); $setType(Token.SKIP); }
    ;

/*
** Define a lexical rule to handle block comments.
*/
MultiLineComment :
        "/*"
        (
            { LA(2)!='/' }? '*'
        |   '\n' { newline(); }
        |   ~('*'|'\n')
        )*
        "*/"
        { $setType(Token.SKIP); }
    ;


//-------------------------------
// Parser
//-------------------------------

class OQLParser extends Parser ;
options {
    k = 2;
    codeGenMakeSwitchThreshold = 3;
    codeGenBitsetTestThreshold = 4;
}

// Membervariablen
{
	Class m_ClassToSearchFrom = null;
}


buildQuery 
returns [Query result = null]
  :
	result = query
//	EOF
  ;

query 
returns [Query result = null]
 :	
	{ Criteria criteria = null; }

	criteria = selectExpr

	{ 
		if ((m_ClassToSearchFrom != null) && (criteria != null)) {
			result = QueryFactory.newQuery(m_ClassToSearchFrom, criteria); 
		}
	}
  ;

selectExpr
returns [Criteria rc = null]
  :
	"select"
	projectionAttributes

	{ rc = new Criteria(); }
	fromClause
	( whereClause[rc] )?
	( orderClause[rc] )?
  ;

fromClause 
  :
	"from" id:Identifier
	{ 
		m_ClassToSearchFrom = null;
      	try
      	{
			m_ClassToSearchFrom = Class.forName(id.getText());
		}
		catch (Exception ex) 
		{}
	}
  ;

whereClause 
[Criteria pCriteria]
  :
	"where" orExpr[pCriteria]
  ;

projectionAttributes 
  :
	(
		projection
		(
			TOK_COMMA
			projection
		)*
      | TOK_STAR
	)
  ;

projection 
  :
	Identifier
  ;

orderClause 
[Criteria pCriteria]
  :
	"order" "by"
	sortCriterion[pCriteria]
	(
		TOK_COMMA sortCriterion[pCriteria]
	)*
  ;

sortCriterion 
[Criteria pCriteria]
  :
	id:Identifier
	(
		"asc"
	  | "desc"
	)?
	{ pCriteria.orderBy(id.getText()); }
  ;

orExpr 
[Criteria pCriteria]
  :
	andExpr[pCriteria]
	(
		"or"
		{ Criteria orCriteria = new Criteria(); }
		andExpr[orCriteria]
		{ 
	 	 try 
		 {
		 	pCriteria.addOrCriteria(orCriteria);
		 } 
		 catch (Exception e)
		 {} 
		}
	)*
  ;

andExpr 
[Criteria pCriteria]
  :
	quantifierExpr[pCriteria]
	(
		"and"
		{ Criteria andCriteria = new Criteria(); }

		quantifierExpr[andCriteria]
		{ pCriteria.addAndCriteria(andCriteria); }
	)*
  ;

quantifierExpr 
[Criteria pCriteria]
  :
	{
	int index = 1;
	String[] args;
	int op = 0; 
	Object obj;
	String ident;
	}

	id:Identifier
    { 
		ident = id.getText();	
	}

    (
		{ op = 0; }
	    (
    	    TOK_EQ
			{ op = 1; }
          | TOK_NE
			{ op = 2; }
          | TOK_LT
			{ op = 3; }
          | TOK_GT
			{ op = 4; }
          | TOK_LE
			{ op = 5; }
          | TOK_GE
			{ op = 6; }
        )

        (
			// Simple comparison of expressions
            obj = literal
			{
				switch (op) {
					case 1:
						pCriteria.addEqualTo(ident, obj);
					break;

					case 2:
						pCriteria.addNotEqualTo(ident, obj);
					break;

					case 3:
						pCriteria.addLessThan(ident, obj);
					break;

					case 4:
						pCriteria.addGreaterThan(ident, obj);
					break;

					case 5:
						pCriteria.addLessOrEqualThan(ident, obj);
					break;

					case 6:
						pCriteria.addGreaterOrEqualThan(ident, obj);
					break;

					default:
					break;
				}
			}
		)

        // Like comparison
      | "like" obj = literal
		{
			pCriteria.addLike(ident, obj);
		}
	)*
    ;


literal 
returns [Object obj = null]
:
			// Object
            "nil"
			{ obj = null; }

			// Boolean
        |   (  "true" 
			   { obj = new Boolean(true); }
			 | "false"
			   { obj = new Boolean(false); }
			)
		    // Parameter, just fill with placeholder object
		    // did not manage to get it wotk with (TOK_DOLLAR TOK_UNSIGNED_INTEGER)
		|   TOK_PARAMETER
		    {
		        obj = new Object();
		    }

			// Integer
        |   tokInt:INT_LITERAL    //TOK_UNSIGNED_INTEGER
			{ 
				// was passiert wenn Wert zu gross ist?
				obj = new Integer(tokInt.getText());
			}

			// Double
        |   (
               tokDec:TOK_APPROXIMATE_NUMERIC_LITERAL
			   { obj = new Double(tokDec.getText()); }

             | tokDec2:TOK_EXACT_NUMERIC_LITERAL
			   { obj = new Double(tokDec2.getText()); }
            )

			// String
        |   tokStr:StringLiteral
			{ obj = new String(tokStr.getText()); }

// Explizit mit Schluesselwort oder ohne, dann immer als String.
// Haengt vom Framework ab, wie kann es mit Date, Time und Timestamp umgehen kann.

			// Date
        |   "date" tokDate:StringLiteral
			{ 
				// Format yyyy-mm-dd
				obj = java.sql.Date.valueOf(tokDate.getText());
			}

			// Time
        |   "time" tokTime:StringLiteral
			{ 
				// Format hh:mm:ss
				obj = java.sql.Time.valueOf(tokTime.getText());
			}

			// Timestamp
        |   "timestamp" tokTS:StringLiteral
			{ 
				// Format yyyy-mm-dd hh:mm:ss.fffffffff
				obj = java.sql.Timestamp.valueOf(tokTS.getText());
			}
    ;
