/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/
/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.states;
import ojb.broker.accesslayer.ObjectCache;
import ojb.odmg.ObjectTransactionWrapper;
import ojb.broker.Identity;
import ojb.broker.PersistenceBroker;

/**
 * Describes an objects transactional state regarding commiting and rollbacking
 */
public abstract class ModificationState extends Object implements java.io.Serializable
{

    protected transient ObjectCache cache = ObjectCache.getInstance();

	/**
	 * return resulting state after marking clean
	 */
	public abstract ModificationState markClean();

	/**
	 * return resulting state after marking delete
	 */
	public abstract ModificationState markDelete();

	/**
	 * return resulting state after marking dirty
	 */
	public abstract ModificationState markDirty();

	/**
	 * return resulting state after marking new
	 */
	public abstract ModificationState markNew();

	/**
	 * return resulting state after marking old
	 */
	public abstract ModificationState markOld();

	/**
	 * return a String representation
	 * @return java.lang.String
	 */
	public String toString()
	{
		return this.getClass().getName();
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param objToCache
	 * 
	 * @see
	 */
	protected void moveToCache(Object objToCache)
	{
		try
		{
			cache.cache(objToCache);
		}
		catch (ojb.broker.ClassNotPersistenceCapableException ex)
		{
			throw new org.odmg.ClassNotPersistenceCapableException(ex.getMessage());
		}
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @return
	 * 
	 * @see
	 */
	public boolean needsInsert()
	{
		return false;
	}

	/**
	 * remove Object from the ObjectCache
	 */
	protected void removeFromCache(Object objToBeRemoved)
	{
		cache.remove(new Identity(objToBeRemoved));
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param mod
	 * 
	 * @throws ojb.broker.PersistenceBrokerException
	 * 
	 * @see
	 */
	public abstract void checkpoint(ObjectTransactionWrapper mod, PersistenceBroker broker)
			throws ojb.broker.PersistenceBrokerException;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param mod
	 * 
	 * @throws ojb.broker.PersistenceBrokerException
	 * 
	 * @see
	 */
	public abstract void commit(ObjectTransactionWrapper mod, PersistenceBroker broker)
			throws ojb.broker.PersistenceBrokerException;

	/**
	 * Method declaration
	 * 
	 * 
	 * @param mod
	 * 
	 * @see
	 */
	public abstract void rollback(ObjectTransactionWrapper mod, PersistenceBroker broker);
}