/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/
/*
 * ObJectRelationalBridge - Bridging Java Objects and Relational Databases
 * http://objectbridge.sourceforge.net
 * Copyright (C) 2000, 2001 Thomas Mahler, et al.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */
package ojb.odmg.states;
import ojb.broker.*;
import ojb.odmg.ObjectTransactionWrapper;

/**
 * this state represents new objects which have not been altered during tx.
 */
public class StateNewClean extends ModificationState
{

	/**
	 * return resulting state after marking clean
	 */
	public ModificationState markClean()
	{
		return this;
	}

	/**
	 * return resulting state after marking delete
	 */
	public ModificationState markDelete()
	{
		return StateNewDelete.getInstance();
	}

	/**
	 * return resulting state after marking dirty
	 */
	public ModificationState markDirty()
	{
		return StateNewDirty.getInstance();
	}

	/**
	 * return resulting state after marking new
	 */
	public ModificationState markNew()
	{
		return this;
	}

	/**
	 * return resulting state after marking old
	 */
	public ModificationState markOld()
	{
		return StateOldClean.getInstance();
	}

	private static StateNewClean _instance = new StateNewClean();

	/**
	 * private constructor: we use singleton instances
	 */
	private StateNewClean() {}

	/**
	 * perform a checkpoint, i.e. perform updates on underlying db but keep locks on objects
	 */
	public static StateNewClean getInstance()
	{
		return _instance;
	}

	/**
	 * object is new, thus we need an INSERT to store it
	 */
	public boolean needsInsert()
	{
		return true;
	}

	/**
	 * rollback the transaction
	 */
	public void checkpoint(ObjectTransactionWrapper mod, PersistenceBroker broker)
			throws ojb.broker.PersistenceBrokerException
	{
		broker.store(mod.getObject(), mod);
	}

	/**
	 * commit the associated transaction
	 */

	/**
	 * commit the associated transaction
	 */
	public void commit(ObjectTransactionWrapper mod, PersistenceBroker broker) throws ojb.broker.PersistenceBrokerException
	{
		broker.store(mod.getObject(), mod);
		this.moveToCache(mod.getObject());
	}

	/**
	 * Method declaration
	 * 
	 * 
	 * @param mod
	 * 
	 * @see
	 */
	public void rollback(ObjectTransactionWrapper mod, PersistenceBroker broker)
	{
		this.removeFromCache(mod.getObject());
	}

}