package test.ojb.broker;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.io.IOException;

import ojb.broker.metadata.*;
import ojb.broker.accesslayer.*;
import ojb.broker.*;
import ojb.broker.query.*;
import junit.framework.*;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/** Demo Application that shows basic concepts for Applications using the PersistenceBroker
  * as a mediator for persistence
  */
public class BrokerExamples extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	PersistenceBroker broker;
	private static Class CLASS = BrokerExamples.class;
	private int testId = 88888;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public BrokerExamples(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article createArticle(int id)
{
	Article a = new Article();
	a.setArticleId(id);
	a.setArticleName("New Funny Article " + id);
	a.setIsSelloutArticle(true);
	a.setMinimumStock(100);
	a.setOrderedUnits(17);
	a.setPrice(0.45);
	a.setProductGroupId(1);
	a.setStock(234);
	a.setSupplierId(4);
	a.setUnit("bottle");
	ProductGroup tmpPG = new ProductGroup();
	tmpPG.setId(1);
	Identity pgID = new Identity(tmpPG);
	ProductGroupProxy pgProxy = new ProductGroupProxy(pgID, broker);
	a.setProductGroup(pgProxy);
	return a;
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
    try
    {
        broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml");
    }
    catch(IOException e)
    {
    }
    catch(ParserConfigurationException e)
    {
    }
    catch(SAXException e)
    {
    }


    // enable tracing of distinct system layers
	//PersistenceBrokerClient.setDebug(true);
	//JdbcAccess.setDebug(true);
	//ClassDescriptor.setDebug(true);
	//RsIterator.setDebug(true);
	//CollectionDescriptor.setDebug(true);
	//ObjectReferenceDescriptor.setDebug(true);
	//SqlGenerator.setDebug(true);
	//ObjectCache.setDebug(true);
	//RepositoryXmlHandler.setDebug(true);
	//DescriptorRepository.setDebug(true);



}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{

	ObjectCache.getInstance().clear();

}

/**
 * Insert the method's description here.
 * Creation date: (23.12.2000 20:05:49)
 */
public void testCollectionRetrieval()
{
	// Use PersistenceBroker to handle persistence of Business objects

	// loop through categories 1 to 9 and print them
	// A ProductGroup holds a Vector of all Article Objects in the specific category
	// the repository.xml specifies that the Vector of CArtikels has to be materialized immediately.
	// Thus printing the category will also print all Articles in the category
	try
	{
	for (int i = 1; i < 9; i++)
	{
		ProductGroup example = new ProductGroup();
		example.setId(i);

		ProductGroup group = (ProductGroup) broker.getObjectByQuery(QueryFactory.newQuery(example));
		assertEquals("should be equal",i,group.getId());
		
		System.out.println(group + "\n\n");

	}
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}

}

/**
 * Insert the method's description here.
 * Creation date: (23.12.2000 20:05:49)
 */
public void testModifications()
{
	//create a new Article and play with it
	Article article = new Article();
	article.setPrice(2);
	article.setStock(0);
	article.setArticleId(testId);

	try {
	for (int i = 1; i < 50; i++)
	{
		article.addToStock(10);
		broker.store(article);
		broker.delete(article);
		broker.store(article);

	}
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}

}

/**
  * here we see what a good job the ObjectCache does:
  * if the Cache has not been flushed by the GC, All previously loaded Objects are still hot.
  * I.e. they are reloaded from the Cache and not materialized from the RDBMS
  */
public void testObjectCache()
{
	try
	{
		Article tmpArticle = createArticle(testId);
		Identity tmpOID = new Identity(tmpArticle);
		broker.store(tmpArticle);
		ObjectCache.getInstance().clear();
		Article cached = (Article) ObjectCache.getInstance().lookup(tmpOID);
		assertNull("After flushing Object should be not in cache", cached);
		Article article = (Article) broker.getObjectByQuery(new QueryByExample(tmpOID));
		cached = (Article) ObjectCache.getInstance().lookup(tmpOID);
		assertNotNull("now object should be found in cache", cached);
		assertEquals("should be same reference", cached, article);
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article createSimpleArticle(int id)
{
	Article a = new Article();
	a.setArticleId(id);
	a.setArticleName("New Funny Article " + id);
	a.setIsSelloutArticle(false);
	a.setMinimumStock(100);
	a.setOrderedUnits(17);
	a.setPrice(0.45);
	a.setProductGroupId(1);
	a.setStock(234);
	a.setSupplierId(4);
	a.setUnit("bottle");
	return a;
}

/**
  * here we see what a good job the ObjectCache does:
  * if the Cache has not been flushed by the GC, All previously loaded Objects are still hot.
  * I.e. they are reloaded from the Cache and not materialized from the RDBMS
  */
public void testShallowAndDeepRetrieval()
{
	try
	{
		// ensure there is an item to find
		Article tmpArticle = createArticle(testId);
		Identity tmpOID = new Identity(tmpArticle);
		broker.store(tmpArticle);
		ObjectCache.getInstance().clear();

		// switch to shallow retrieval
		ClassDescriptor cld = DescriptorRepository.getInstance().getDescriptorFor(Article.class);
		ObjectReferenceDescriptor ord = (ObjectReferenceDescriptor)cld.getObjectReferenceDescriptors().get(0);
		ord.setCascadeRetrieve(false);
		
		
		Article article = (Article) broker.getObjectByQuery(new QueryByExample(tmpOID));
		assertNull("now reference should be null", article.getProductGroup());

		// now switch to deep retrieval
		ord.setCascadeRetrieve(true);
		article = (Article) broker.getObjectByQuery(new QueryByExample(tmpOID));
		//assertNotNull("now reference should NOT be null", article.getProductGroup());
		
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
}