package test.ojb.broker;

import junit.framework.*;
import ojb.broker.*;
import ojb.broker.metadata.*;
import ojb.broker.accesslayer.*;
import ojb.broker.query.*;
import java.util.*;
import java.io.IOException;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/**
 * Insert the type's description here.
 * Creation date: (06.12.2000 21:47:56)
 * @author: Administrator
 */
public class PersistenceBrokerTest extends TestCase {
	PersistenceBroker broker;

	int testId = 12345;

	private static Class CLASS = PersistenceBrokerTest.class;

/**
 * BrokerTests constructor comment.
 * @param name java.lang.String
 */
public PersistenceBrokerTest(String name) {
	super(name);
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article createArticle(int id) 
{
	
	Article a = new Article();
	a.setArticleId(id);
	a.setArticleName("New Funny Article " + id);
	a.setIsSelloutArticle(true);
	a.setMinimumStock(100);
	a.setOrderedUnits(17);
	a.setPrice(0.45);
	a.setProductGroupId(1);
	a.setStock(234);
	a.setSupplierId(4);
	a.setUnit("bottle");
	return a;
	
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected void deleteArticle(int id) throws PersistenceBrokerException
{
	Article a = new Article();
	a.setArticleId(id);
	broker.delete(a);
	
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected void deleteArticle(Article articleToDelete) throws PersistenceBrokerException
{
	broker.delete(articleToDelete);

}
/**
 * Insert the method's description here.
 * Creation date: (23.12.2000 18:30:38)
 * @param args java.lang.String[]
 */
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article readArticleByExample(int id) throws PersistenceBrokerException
{
	
	Article example = new Article();
	example.setArticleId(id);
	
	
	return (Article) broker.getObjectByQuery(QueryFactory.newQuery(example));
	
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article readArticleByIdentity(int id) throws PersistenceBrokerException
{
	
	Article a = new Article();
	a.setArticleId(id);
	Identity oid = new Identity(a);
	
	return (Article) broker.getObjectByIdentity(oid);
	
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
    try
    {
        broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml");
    }
    catch(IOException e)
    {
    }
    catch(ParserConfigurationException e)
    {
    }
    catch(SAXException e)
    {
    }
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected void storeArticle(Article anArticle) throws PersistenceBrokerException
{

	broker.store(anArticle);
	
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{
	try
	{
		Article a = createArticle(testId);
		deleteArticle(a);
		ObjectCache.getInstance().clear();
	}
	catch (Throwable t)
	{
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testDelete()
{
	try
	{
		Article a = createArticle(testId);
		storeArticle(a);
		Article b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		deleteArticle(b);
		b = readArticleByIdentity(testId);
		assertNull("should be null after deletion", b);
		b = readArticleByExample(testId);
		assertNull("should be null after deletion", b);
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testEscaping()
{
	try
	{
		Article a = createArticle(testId);
		deleteArticle(a);
		Article b = readArticleByIdentity(testId);
		assertNull("should be null after deletion", b);
		a.setArticleName("Single quote 'article ");
		storeArticle(a);
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		Collection col;
		Iterator iter;
		String name = a.getArticleName();
		Criteria criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		Query query = QueryFactory.newQuery(Article.class, criteria);
		col = broker.getCollectionByQuery(query);
		iter = col.iterator();
		assert("should have one element", iter.hasNext());
		assertEquals("should be equal", name, ((InterfaceArticle) iter.next()).getArticleName());
		//
		a.setArticleName("2 Single quotes 'article' ");
		storeArticle(a);
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		name = a.getArticleName();
		criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		query = QueryFactory.newQuery(Article.class, criteria);
		col = broker.getCollectionByQuery(query);
		iter = col.iterator();
		assert("should have one element", iter.hasNext());
		assertEquals("should be equal", name, ((InterfaceArticle) iter.next()).getArticleName());
		//
		a.setArticleName("double quote \"article ");
		storeArticle(a);
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		name = a.getArticleName();
		criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		query = QueryFactory.newQuery(Article.class, criteria);
		col = broker.getCollectionByQuery(query);
		iter = col.iterator();
		assert("should have one element", iter.hasNext());
		assertEquals("should be equal", name, ((InterfaceArticle) iter.next()).getArticleName());
		//
		a.setArticleName("2 double quotes \"article\" ");
		storeArticle(a);
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		name = a.getArticleName();
		criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		query = QueryFactory.newQuery(Article.class, criteria);
		col = broker.getCollectionByQuery(query);
		iter = col.iterator();
		assert("should have one element", iter.hasNext());
		assertEquals("should be equal", name, ((InterfaceArticle) iter.next()).getArticleName());
		//
		a.setArticleName("a comma thing ,article, ");
		storeArticle(a);
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		name = a.getArticleName();
		criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		query = QueryFactory.newQuery(Article.class, criteria);
		col = broker.getCollectionByQuery(query);
		iter = col.iterator();
		assert("should have one element", iter.hasNext());
		assertEquals("should be equal", name, ((InterfaceArticle) iter.next()).getArticleName());
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetByExampleAndGetByIdentity()
{
	try
	{
		Article a = createArticle(testId);
		storeArticle(a);
		Article b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		Article c = readArticleByExample(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), c.getArticleName());
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetCollectionByQuery()
{
	try
	{
		// 1. ensure there are 0 items matching the query
		String name = "CollectionTestArticle";
		Criteria criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		Query query = QueryFactory.newQuery(Article.class, criteria);
		Collection col = broker.getCollectionByQuery(query);
		Iterator iter = col.iterator();
		while (iter.hasNext())
		{
			broker.delete(iter.next());
		}
		col = broker.getCollectionByQuery(query);
		assertEquals("size of collection should be zero", 0, col.size());
		//2. insert 3 matching items
		int id1 = 4001;
		int id2 = 4002;
		int id3 = 4003;
		Article a1 = createArticle(id1);
		a1.setArticleName(name);
		broker.store(a1);
		Article a2 = createArticle(id2);
		a2.setArticleName(name);
		broker.store(a2);
		Article a3 = createArticle(id3);
		a3.setArticleName(name);
		broker.store(a3);
		// 3. check if all items are found
		col = broker.getCollectionByQuery(query);
		assertEquals("size of collection should be three", 3, col.size());
		iter = col.iterator();
		while (iter.hasNext())
		{
			assertEquals("should be same value", name, ((InterfaceArticle) iter.next()).getArticleName());
		}
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetIteratorByQuery()
{
	try
	{
		// 1. ensure there are 0 items matching the query
		// build query
		String name = "CollectionTestArticle";
		Criteria criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		Query query = QueryFactory.newQuery(Article.class, criteria);
		// delete all items found
		Iterator iter = broker.getIteratorByQuery(query);
		while (iter.hasNext())
		{
			broker.delete(iter.next());
		}
		iter = broker.getIteratorByQuery(query);
		assert("size of collection should be zero", !iter.hasNext());
		//2. insert 3 matching items
		int id1 = 4001;
		int id2 = 4002;
		int id3 = 4003;
		Article a1 = createArticle(id1);
		a1.setArticleName(name);
		broker.store(a1);
		Article a2 = createArticle(id2);
		a2.setArticleName(name);
		broker.store(a2);
		Article a3 = createArticle(id3);
		a3.setArticleName(name);
		broker.store(a3);
		// 3. check if all items are found
		iter = broker.getIteratorByQuery(query);
		int count = 0;
		while (iter.hasNext())
		{
			count++;
			assertEquals("should be same value", name, ((InterfaceArticle) iter.next()).getArticleName());
		}
		assertEquals("Iterator should produce 3 items", 3, count);
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetMultipleIteratorsByQuery()
{
	try
	{
		// 1. ensure there are 0 items matching the query
		String name = "CollectionTestArticle";
		Criteria criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		Query query = QueryFactory.newQuery(Article.class, criteria);
		Iterator iter = broker.getIteratorByQuery(query);
		while (iter.hasNext())
		{
			broker.delete(iter.next());
		}
		iter = broker.getIteratorByQuery(query);
		assert("size of collection should be zero", !iter.hasNext());
		//2. insert 3 matching items
		int id1 = 4001;
		int id2 = 4002;
		int id3 = 4003;
		Article a1 = createArticle(id1);
		a1.setArticleName(name);
		broker.store(a1);
		Article a2 = createArticle(id2);
		a2.setArticleName(name);
		broker.store(a2);
		Article a3 = createArticle(id3);
		a3.setArticleName(name);
		broker.store(a3);
		// 3. return multiple iterators
		Iterator i1 = broker.getIteratorByQuery(query);
		Iterator i2 = broker.getIteratorByQuery(query);
		Iterator i3 = broker.getIteratorByQuery(query);
		// 4. Test the iterators
		for (int i = 0; i < 3; i++)
		{
			assert("should have more elements", i3.hasNext());
			assert("should have more elements", i1.hasNext());
			assert("should have more elements", i2.hasNext());
			assertEquals("should be same value", name, ((InterfaceArticle) i2.next()).getArticleName());
			assertEquals("should be same value", name, ((InterfaceArticle) i1.next()).getArticleName());
			assertEquals("should be same value", name, ((InterfaceArticle) i3.next()).getArticleName());
		}
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetObjectByQuery() 
{
	try {
	// ensure article is persistent
	Article a = createArticle(testId);
	storeArticle(a);

	// build query-by-example and execute
	Query query = QueryFactory.newQuery(a);
	
	Article b = (Article) broker.getObjectByQuery(query);
	assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());

	Article c = readArticleByExample(testId);
	assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), c.getArticleName());


	// now Test a criteria query
	Criteria crit = new Criteria();
	crit.addEqualTo("articleId", new Integer(a.getArticleId()));
	Query q = QueryFactory.newQuery(Article.class, crit);
	InterfaceArticle d = (InterfaceArticle) broker.getObjectByQuery(q);
	assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), d.getArticleName());

	
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testGetPKEnumerationByConstraints()
{
	try
	{
		// 1. ensure there are 0 items matching the query
		String name = "CollectionTestArticle";
		Criteria criteria = new Criteria();
		criteria.addEqualTo("articleName", name);
		Query query = QueryFactory.newQuery(Article.class, criteria);
		Enumeration enum = broker.getPKEnumerationByQuery(ArticlePrimaryKey.class, query);
		while (enum.hasMoreElements())
		{
			ArticlePrimaryKey pk = (ArticlePrimaryKey) enum.nextElement();
			deleteArticle(pk.id);
		}
		enum = broker.getPKEnumerationByQuery(ArticlePrimaryKey.class, query);
		assert("size of collection should be zero", !enum.hasMoreElements());
		//2. insert 3 matching items
		int id1 = 4001;
		int id2 = 4002;
		int id3 = 4003;
		Article a1 = createArticle(id1);
		a1.setArticleName(name);
		broker.store(a1);
		Article a2 = createArticle(id2);
		a2.setArticleName(name);
		broker.store(a2);
		Article a3 = createArticle(id3);
		a3.setArticleName(name);
		broker.store(a3);
		// 3. check if all items are found
		enum = broker.getPKEnumerationByQuery(ArticlePrimaryKey.class, query);
		int count = 0;
		while (enum.hasMoreElements())
		{
			count++;
			Article tmp = readArticleByIdentity(((ArticlePrimaryKey) enum.nextElement()).id);
			assertEquals("should be same value", name, tmp.getArticleName());
		}
		assertEquals("Iterator should produce 3 items", 3, count);
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testInsert()
{
	try
	{
		Article a = createArticle(testId);
		deleteArticle(a);
		Article b = readArticleByIdentity(testId);
		assertNull("should be null after deletion", b);
		storeArticle(a);
		b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		// check if object is not only stored in cache but also in db
		b = null;
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals(
			"after inserting and flushing the cache an object should still be equal to its re-read pendant",
			a.getArticleName(),
			b.getArticleName());
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testUpdate()
{
	try
	{
		deleteArticle(testId);
		Article a = createArticle(testId);
		storeArticle(a);
		Article b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		String newname = "TESTUPDATE";
		b.setArticleName(newname);
		storeArticle(b);
		b = null;
		//ObjectCache.getInstance().remove(a);
		b = readArticleByIdentity(testId);
		assertEquals("should be equal after update", newname, b.getArticleName());
		// ensure that object is really stored in DB and not only in ObjectCache 
		b = null;
		ObjectCache.getInstance().clear();
		b = readArticleByIdentity(testId);
		assertEquals("should be equal after update and db lookup", newname, b.getArticleName());
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:51:22)
 */
public void testUpdateWithModification()
{
	try
	{
		SimpleObjectModificationImpl modification = new SimpleObjectModificationImpl();
		assert("should not be marked for update yet", !modification.needsUpdate());
		assert("should not be marked for insert", !modification.needsInsert());
		Article a = createArticle(testId);
		storeArticle(a);
		Article b = readArticleByIdentity(testId);
		assertEquals("after inserting an object it should be equal to its re-read pendant", a.getArticleName(), b.getArticleName());
		String newname = "TESTUPDATE";
		b.setArticleName(newname);
		modification.markModified();
		assert("should be marked for update", modification.needsUpdate());
		assert("should not be marked for insert", !modification.needsInsert());
		broker.store(b, modification);
		b = null;
		b = readArticleByIdentity(testId);
		assertEquals("should be equal after update", newname, b.getArticleName());
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
}