package test.ojb.broker;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.io.IOException;

import ojb.broker.metadata.*;
import ojb.broker.accesslayer.*;
import ojb.broker.*;
import ojb.broker.query.*;
import ojb.odmg.DatabaseImpl;
import ojb.odmg.OJB;
import junit.framework.*;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/** This TestClass tests OJB facilities to work with polymorphism.
  */
public class PolymorphicExtents extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	PersistenceBroker broker;
	private static Class CLASS = PolymorphicExtents.class;
	private int testId = 88888;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public PolymorphicExtents(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article createArticle(int id)
{
	Article a = new Article();
	a.setArticleId(id);
	a.setArticleName("New Funny Article " + id);
	a.setIsSelloutArticle(true);
	a.setMinimumStock(100);
	a.setOrderedUnits(17);
	a.setPrice(0.45);
	a.setProductGroupId(1);
	a.setStock(234);
	a.setSupplierId(4);
	a.setUnit("bottle");
	ProductGroup tmpPG = new ProductGroup();
	tmpPG.setId(1);
	Identity pgID = new Identity(tmpPG);
	ProductGroupProxy pgProxy = new ProductGroupProxy(pgID, broker);
	a.setProductGroup(pgProxy);
	return a;
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
    try
    {
        broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml");
    }
    catch(IOException e)
    {
    }
    catch(ParserConfigurationException e)
    {
    }
    catch(SAXException e)
    {
    }


    // enable tracing of distinct system layers
	//PersistenceBrokerClient.setDebug(true);
	//JdbcAccess.setDebug(true);
	//ClassDescriptor.setDebug(true);
	//RsIterator.setDebug(true);
	//CollectionDescriptor.setDebug(true);
	//ObjectReferenceDescriptor.setDebug(true);
	//SqlGenerator.setDebug(true);
	//ObjectCache.setDebug(true);
	//RepositoryXmlHandler.setDebug(true);
	//DescriptorRepository.setDebug(true);



}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{

	ObjectCache.getInstance().clear();

}

/** Test query support for polymorphic extents*/
public void testCollectionByQuery()
{
	try
	{
		Criteria crit = new Criteria();
		crit.addEqualTo("articleName", "Hamlet");
		Query q = QueryFactory.newQuery(InterfaceArticle.class, crit);

		Collection result = broker.getCollectionByQuery(q);

		System.out.println(result);

		assertNotNull("should return at least one item", result);
		assert("should return at least one item",result.size() > 0);
		
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/**
 * try to retrieve a polymorphic collection attribute
 * (ProductGroup.allArticlesInGroup contains items
 * of type Test.ojb.broker.Article which forms an extent)
 *  ProductGroup 5 contain items from table Artikel, BOOKS and CDS
 */
public void testCollectionRetrieval()
{
	try
	{
		ProductGroup example = new ProductGroup();
		example.setId(5);

		ProductGroup group = (ProductGroup) broker.getObjectByQuery(QueryFactory.newQuery(example));
		
		System.out.println(group + "\n\n");

	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}

}

/** Test EXTENT lookup: a collection with ALL objects in the Article extent*/
public void testExtentByQuery()
{
	try
	{
		// no criteria signals to omit a WHERE clause
		Query q = QueryFactory.newQuery(InterfaceArticle.class, null);

		Collection result = broker.getCollectionByQuery(q);

		
		System.out.println("OJB proudly presents: The InterfaceArticle Extent\n" + result);

		assertNotNull("should return at least one item", result);
		assert("should return at least one item",result.size() > 0);
		
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/** Test to lookup items from extent classes*/
public void testRetrieveObjectByIdentity()
{
	try
	{
		Article example = new Article();
		example.setArticleId(200); // id not present in table ARTICLES but int table CDS
		Identity oid = new Identity(example);

		InterfaceArticle result = (InterfaceArticle) broker.getObjectByIdentity(oid);

		assertNotNull("should find a CD-article",result);
		assert("should be of type CdArticle", (result instanceof CdArticle));

		example = new Article();
		example.setArticleId(100); // id not present in table ARTICLES but int table CDS
		oid = new Identity(example);

		result = (InterfaceArticle) broker.getObjectByIdentity(oid);

		assertNotNull("should find a Book-article",result);
		assert("should be of type BookArticle", (result instanceof BookArticle));

		
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/** try to load polymorphic references 
  * (OrderPosition.article is of type InterfaceArticle)
  */
public void testRetrieveReferences()
{
	try
	{
		for (int i = 1; i < 4; i++)
		{
			OrderPosition tmp = new OrderPosition();
			tmp.setId(i);
			Identity oid = new Identity(tmp);
			OrderPosition pos = (OrderPosition) broker.getObjectByIdentity(oid);
			System.out.println(pos);
		}
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/** testing the SequenceManagement with extents.
  */
public void testSequenceManager()
{
	try
	{
		// ask for a new unique ID in the InterfaceArticle extent
		// the UID must be unique accross tables ARTIKEL, BOOKS, CDS !
        int uid = broker.getUniqueId(InterfaceArticle.class, "articleId");

		// now check if is really unique !
		Article tmp = new Article();
		tmp.setArticleId(uid);
		Query q = new QueryByExample(tmp);
		InterfaceArticle result = (InterfaceArticle) broker.getObjectByQuery(q);

		assertNull("" + uid + " should be unique", result);

		System.out.println("next free UID for InterfaceArticle : " + uid);

		// the next id from the sequence should be uid + 1, even if we did not store any object using uid.
		int nextUid = broker.getUniqueId(InterfaceArticle.class, "articleId");
		assertEquals("nextUid - uid = 1 !!!", 1, nextUid - uid);
			
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
}