package test.ojb.broker;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.io.IOException;

import ojb.broker.metadata.*;
import ojb.broker.accesslayer.*;
import ojb.broker.*;
import junit.framework.*;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/** Demo Application that shows basic concepts for Applications using the PersistenceBroker
  * as a mediator for persistence
  */
public class ProxyExamples extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	PersistenceBroker broker;
	private static Class CLASS = ProxyExamples.class;
	private int testId = 88888;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public ProxyExamples(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 22:15:33)
 * @param id int
 */
protected Article createArticle(int id) 
{
	
	Article a = new Article();
	a.setArticleId(id);
	a.setArticleName("New Funny Article " + id);
	a.setIsSelloutArticle(true);
	a.setMinimumStock(100);
	a.setOrderedUnits(17);
	a.setPrice(0.45);
	a.setProductGroupId(1);
	a.setStock(234);
	a.setSupplierId(4);
	a.setUnit("bottle");
	return a;
	
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
    try
    {
        broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml");
    }
    catch(IOException e)
    {
    }
    catch(ParserConfigurationException e)
    {
    }
    catch(SAXException e)
    {
    }


    // enable tracing of distinct system layers
	//PersistenceBrokerClient.setDebug(true);
	//JdbcAccess.setDebug(true);
	//ClassDescriptor.setDebug(true);
	//RsIterator.setDebug(true);
	//CollectionDescriptor.setDebug(true);
	//ObjectReferenceDescriptor.setDebug(true);
	//SqlGenerator.setDebug(true);
	//ObjectCache.setDebug(true);
	//RepositoryXmlHandler.setDebug(true);
	//DescriptorRepository.setDebug(true);



}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{

	ObjectCache.getInstance().clear();

}

/**
  * This example shows how the PersistenceBroker can be used with a highly configurable proxy concept.
  * The main idea is, not to return materialized objects but rather lazy proxies, that defer materialization
  * until it is definitely neccesary (e.g. reading an Objects attribute).
  *
  * To achieve such a behaviour, you can define proxies for reference- and collection attributes.
  * Use the <referenced.proxy> tag and the <items.proxy> tag for this. As an example see the Repository.xml file
  * in this examples directory.
  *
  * It is not always the best option to use lazy materialization. The usage of proxies can be completely configured
  * in the xml repository. That is, if you decide not to use proxies, you don't have to change program-code,
  * but only out-comment the corresponding entry in the repository.
  * Example: Replace
  * <referenced.proxy>ojb.examples.proxies.ProductGroupProxy</referenced.proxy>
  * by <!--referenced.proxy>ojb.examples.proxies.ProductGroupProxy</referenced.proxy-->
  * and <items.proxy>ojb.examples.proxies.ArticleProxy</items.proxy>
  * by <!--items.proxy>ojb.examples.proxies.ArticleProxy</items.proxy-->
  * in this examples repository.xml to turn off laziness. If this could be always as simple ;-)
  */
public void testMain1()
{
	try
	{
		Vector myArticles = new Vector();
		// In the following code we will generate 10 Proxy-objects.
		// There is no need for DB Access, 
		// uUntil their reals subjects attributes are referenced,
		for (int i = 1; i < 10; i++)
		{
			Object[] pkvals = new Object[1];
			pkvals[0] = new Integer(i);
			Identity id = new Identity(Article.class, pkvals);
			InterfaceArticle A = new ArticleProxy(id, PersistenceBrokerFactory.createPersistenceBroker("repository.xml"));
			myArticles.add(A);
			System.out.println(A);
		}
		// Until now no DB access !
		// In the following code we call methods that reference the real subjects attributes.
		// To access an articles name as in getArticleName(), the proxy object has to materialze the real subjects from db.
		// but note: the references to an Articles productgroup are not materialized immediately, 
		// but contain proxy objects, representing ProductGroups.
		for (int i = 0; i < 9; i++)
		{
			InterfaceArticle a = (InterfaceArticle) myArticles.get(i);
			System.out.println("Article[" + a.getArticleId() + "] : " + a.getArticleName());
		}
		// In the following code we will access the real ProductGroup objects.
		// thus the Proxies have to materialize them.
		for (int i = 0; i < 9; i++)
		{
			InterfaceArticle a = (InterfaceArticle) myArticles.get(i);
			System.out.println("Article[" + a.getArticleId() + "] is in group " + a.getProductGroup().getName());
		}
		// in the following code we will touch fields of the ProductGroup references.
		// Now proxies in the AllArticlesInGroup collection need to be materialized
		System.out.println("now playing with product group no. 2");
		Object[] pkvals = new Object[1];
		pkvals[0] = new Integer(2);
		Identity id = new Identity(ProductGroup.class, pkvals);
		InterfaceProductGroup group2 = null;
		try
		{
			group2 = (InterfaceProductGroup) VirtualProxy.createProxy(ProductGroupProxy.class, id);
		}
		catch (Exception ignored)
		{
		}
		System.out.println(group2.toString());
		for (int i = 0; i < group2.getAllArticles().size(); i++)
		{
			InterfaceArticle a = (InterfaceArticle) group2.getAllArticles().get(i);
			System.out.println(a.getArticleName());
			broker.store(a);
		}
		broker.store(group2);
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}
}