package test.ojb.broker;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.io.IOException;

import ojb.broker.metadata.*;
import ojb.broker.accesslayer.*;
import ojb.broker.*;
import ojb.broker.query.*;
import ojb.odmg.DatabaseImpl;
import ojb.odmg.OJB;
import junit.framework.*;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;

/** Demo Application that shows basic concepts for Applications using the PersistenceBroker
  * as a mediator for persistence
  */
public class TreeTest extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	PersistenceBroker broker;
	private static Class CLASS = TreeTest.class;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public TreeTest(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (13.02.2001 18:50:29)
 * @return Test.ojb.broker.Tree
 * @param parent Test.ojb.broker.Tree
 */
public TreeGroup createTreeGroup()
{
	TreeGroup result = new TreeGroup();
    try
    {
        result.setId(broker.getUniqueId(Tree.class, "id"));
    }
    catch(PersistenceBrokerException e)
    {
    }
    result.setData("" + result.getId());
	result.setChilds(new Vector());
	result.setMembers(new Vector());
	return result;
}

/**
 * Insert the method's description here.
 * Creation date: (13.02.2001 18:50:29)
 * @return Test.ojb.broker.Tree
 * @param parent Test.ojb.broker.Tree
 */
public Tree createTreeNodeWithParent(Tree parent) {

	Tree result = new Tree();
    try
    {
        result.setId(broker.getUniqueId(Tree.class, "id"));
    }
    catch(PersistenceBrokerException e)
    {
    }
    result.setData(parent.getId() + "-" + result.getId());
	result.setParentId(parent.getId());
	result.setChilds(new Vector());
	
	return result;
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
    try
    {
        broker = PersistenceBrokerFactory.createPersistenceBroker("repository.xml");
    }
    catch(IOException e)
    {
    }
    catch(ParserConfigurationException e)
    {
    }
    catch(SAXException e)
    {
    }


    // enable tracing of distinct system layers
	//PersistenceBrokerClient.setDebug(true);
	//JdbcAccess.setDebug(true);
	//ClassDescriptor.setDebug(true);
	//RsIterator.setDebug(true);
	//CollectionDescriptor.setDebug(true);
	//ObjectReferenceDescriptor.setDebug(true);
	//SqlGenerator.setDebug(true);
	//ObjectCache.setDebug(true);
	//RepositoryXmlHandler.setDebug(true);
	//DescriptorRepository.setDebug(true);



}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{

	ObjectCache.getInstance().clear();

}

/**
  */
public void testCreate()
{
	try
	{

		Tree root = new Tree();
        try
        {
            root.setId(broker.getUniqueId(Tree.class, "id"));
        }
        catch(PersistenceBrokerException e)
        {
        }
        root.setData("a brand new root: " + root.getId());

		root.addChild(createTreeNodeWithParent(root));
		root.addChild(createTreeNodeWithParent(root));
		root.addChild(createTreeNodeWithParent(root));
		root.addChild(createTreeNodeWithParent(root));
		root.addChild(createTreeNodeWithParent(root));


		Tree child = root.getChild(0); 
		child.addChild(createTreeNodeWithParent(child));
		child.addChild(createTreeNodeWithParent(child));
		child.addChild(createTreeNodeWithParent(child));

		child = child.getChild(1);
		child.addChild(createTreeNodeWithParent(child));
		child.addChild(createTreeNodeWithParent(child));

		
		System.out.println("original tree:");
		System.out.println(root);


		broker.store(root);

		Identity oid = new Identity(root);

		Tree retrieved = (Tree) broker.getObjectByIdentity(oid);

		System.out.println("tree after reading from db:");
		System.out.println(retrieved);

		assertEquals("tree should have same size after retrival",root.size(), retrieved.size());
		
		
	}
	catch (Throwable t)
	{
		fail(t.getMessage());
	}
}

/**
  */
public void testTreeGroup()
{
	try
	{
		TreeGroup root = createTreeGroup();
		root.setData("The Tree Group root: " + root.getId());
		TreeGroup green = createTreeGroup();
		green.setData("the GREEN group" + green.getId());
		TreeGroup red = createTreeGroup();
		red.setData("the RED group" + red.getId());
		
		TreeGroup child;
		for (int i = 0; i < 3; i++)
		{
			child = createTreeGroup();
			root.addChild(child);
			green.addMember(child);
			child = createTreeGroup();
			root.addChild(child);
			red.addMember(child);
		}

		for (int i = 0; i < 6; i++)
		{
			child = root.getChild(i);
			child.addChild(createTreeGroup());	
		}
		
 
		System.out.println("original TreeGroup:");
		System.out.println(root);
		System.out.println("GREEN TreeGroup:");
		System.out.println(green);
		System.out.println("RED TreeGroup:");
		System.out.println(red);

		
		broker.store(root);
		broker.store(green);
		broker.store(red);
	   
		Identity oid = new Identity(root);
		TreeGroup root_r = (TreeGroup) broker.getObjectByIdentity(oid);
		System.out.println("tree after reading from db:");
		System.out.println(root_r);
		assertEquals("tree should have same size after retrival", root.size(), root_r.size());

		oid = new Identity(green);
		TreeGroup green_r = (TreeGroup) broker.getObjectByIdentity(oid);
		System.out.println("tree after reading from db:");
		System.out.println(green_r);
		assertEquals("tree should have same size after retrival", green.size(), green_r.size());


		oid = new Identity(red);
		TreeGroup red_r = (TreeGroup) broker.getObjectByIdentity(oid);
		System.out.println("tree after reading from db:");
		System.out.println(red_r);
		assertEquals("tree should have same size after retrival", red.size(), red_r.size());
 
	}
	catch (Throwable t)
	{
		System.out.println(t.getMessage());
		fail(t.getMessage());
	}
}
}