package test.ojb.odmg;

import ojb.broker.PersistenceBroker;
import java.util.Vector;
import ojb.broker.PersistenceBrokerFactory;
import ojb.broker.Identity;
import ojb.odmg.*;

/** Simple Article class is not derived from any base class nor does it implement any Interface, 
  * but still it can be made persistent by the PersistenceBroker.
  * Has a lot of private members to be mapped to rdbms columns, but only few business methods
  */
public class Article implements ojb.odmg.TransactionAware {
/** return a string representaion of an article*/
public String toString() 
{
	String group = "" + productGroupId;
	if (productGroup != null) group = productGroup.getName();
	
	return "----\n" +
	 "Article No:   " + articleId + "\n" +
	 "Description:  " + articleName + "\n" +
	 "ProductGroup: " + group + "\n" +
	 "stock:        " + stock + "\n" +
	 "price:        " + price + "\n" +
	 "stock-value   " + getStockValue() + "\n";
	 
  }                              

	/** maps to db-column "Artikel-Nr";INT;PrimaryKey*/
	private int articleId;
	/** maps to db-column Artikelname;CHAR*/
	private String articleName;
	/** maps to db-column Auslaufartikel;SMALL INT*/
	private int isSelloutArticle;
	/** maps to db-column Mindestbestand;INT*/
	private int minimumStock;
	/** maps to db-column BestellteEinheiten;INT*/
	private int orderedUnits;
	/** maps to db-column Einzelpreis;DECIMAL*/
	private double price;
	/** reference to the articles category*/
	private ProductGroup productGroup;
	/** maps to db-column Kategorie-Nr;INT*/
	private int productGroupId;
	/** maps to db-column Lagerbestand;INT*/
	private int stock;
	/** maps to db-column Lieferanten-Nr;INT*/
	private int supplierId;
	/** maps to db-column Liefereinheit;CHAR*/
	private String unit;
/** increase the amount of articles in stock by diff
	* mark the object as modified only if value changes (i.e. diff != 0 )
	*/
public void addToStock(int diff)
{
	stock += diff;
}  

/**
 * return an articles unique id.
 * @return int the articles unique id
 */  
public int getArticleId()
{
	return articleId;
}
/**
 * return an articles name.
 * @return java.lang.String
 */ 
public java.lang.String getArticleName()
{
	return articleName;
}
/** return an articles ProductGroup*/
public ProductGroup getProductGroup()
{
	return productGroup;
}
/**
 * return stock of Article.
 * @return int
 */ 
public int getStock()
{
	return stock;
}
/** compute the total value of an articles stock*/
public double getStockValue()
{
	return price * stock;
}
/**
 * set an articles unique id.
 * @param newArticleId int
 */ 
public void setArticleId(int newArticleId) {
	articleId = newArticleId;
}
/**
 * set an articles name.
 * @param newArticleName java.lang.String
 */ 
public void setArticleName(java.lang.String newArticleName)
{
	articleName = newArticleName;
}
/**
 * set stock of Article. Mark object as modified only if value changed.
 * @param newStock int
 */ 
public void setStock(int newStock)
{
	stock = newStock;
}

/**
  * afterAbort will be called after a transaction has been aborted.
  * The values of fields which get persisted will have changed to
  * what they were at the begining of the transaction.  This method
  * should be overridden to reset any transient or non-persistent
  * fields.
  */
public void afterAbort()
{
	System.out.println("afterAbort: " + new Identity(this));
}

/**
  * afterCommit is called only after a successful commit has taken
  * place.
  */
public void afterCommit()
{
	System.out.println("afterCommit: " + new Identity(this));
}

/**
  * beforeAbort is called before a transaction is aborted.
  */
public void beforeAbort()
{
	System.out.println("beforeAbort: " + new Identity(this));
}

/**
  * beforeCommit will give an object a chance to kill a
  * transaction before it is committed.
  *
  * To kill a transaction, throw a new TransactionAbortedException.
  */
public void beforeCommit() throws org.odmg.TransactionAbortedException {
	System.out.println("beforeCommit: " + new Identity(this));
}


public boolean equals(Object obj)
{
    if (obj instanceof Article)
    {
        return (this.articleId == ((Article) obj).articleId);
    }
    else return false;
}

}