package test.ojb.odmg;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;

import ojb.broker.accesslayer.*;
import ojb.broker.*;
import ojb.odmg.collections.DListImpl;
import ojb.odmg.*;
import org.odmg.*;
import junit.framework.*;

/** Demo Application that shows basic concepts for Applications using the OJB ODMG
  * implementation as an transactional object server.
  */
public class DListTest extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	private static Class CLASS = DListTest.class;
	private String databaseName;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public DListTest(String name)
{
	super(name);
}

protected Article createArticle() 
{
	
	Article a = new Article();
    PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();
    try
    {
        a.setArticleId(broker.getUniqueId(Article.class, "articleId"));
    }
    catch(PersistenceBrokerException e)
    {
    }
    a.setArticleName("New Funny Article " + a.getArticleId());
	
	a.setStock(234);
	
	return a;
	
}

protected void printDList(DList list)
{
	Iterator iter = list.iterator();
	while (iter.hasNext())
	{
		Article a = (Article) iter.next();
		System.out.print(a.getArticleId() + ", ");
	}
	System.out.println();
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
	databaseName = "repository.xml";
	//ObjectCache.setCacheUsed(false);

	// enable tracing of distinct system layers
	//PersistenceBrokerClient.setDebug(true);
	//JdbcAccess.setDebug(true);
	//ClassDescriptor.setDebug(true);
	//RsIterator.setDebug(true);
	//CollectionDescriptor.setDebug(true);
	//ObjectReferenceDescriptor.setDebug(true);
	//SqlGenerator.setDebug(true);
	//ObjectCache.setDebug(true);
	//RepositoryXmlHandler.setDebug(true);
	//DescriptorRepository.setDebug(true);


}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{
	try
	{
		OJB.getInstance().currentTransaction().abort();
		OJB.getInstance().getDatabase(null).close();
		databaseName = null;

		
	}
	catch (Throwable t)
	{
	}
}

public void testAdding()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
	Transaction tx = odmg.newTransaction();
	tx.begin();
	DList list = odmg.newDList();
    PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();
    String name = null;
    try
    {
        name = "the_dlist_" + (broker.getUniqueId(ojb.odmg.collections.DListImpl.class, "id")-1);
    }
    catch(PersistenceBrokerException e)
    {
    }

    try
	{
		db.bind(list, name);
		tx.commit();
	}
	catch (Exception ignored)
	{
	}
	tx = odmg.newTransaction();
	tx.begin();
	for (int i = 0; i < 5; i++)
	{
	
		Article a = createArticle();
		list.add(a);
	}
	tx.commit();
	System.out.println("sequence of items in list:");
	Iterator iter = list.iterator();
	while (iter.hasNext())
	{
		Article a = (Article) iter.next();
		System.out.print(a.getArticleId() + ", ");
	}
	System.out.println();
	tx = odmg.newTransaction();
	tx.begin();
	try
	{
		DList lookedUp = (DList) db.lookup(name);
		System.out.println("sequence of items in lookedup list:");
		iter = lookedUp.iterator();
		while (iter.hasNext())
		{
			Article a = (Article) iter.next();
			System.out.print(a.getArticleId() + ", ");
		}
		System.out.println();
	}
	catch (Throwable ignored)
	{
		System.out.println(ignored.getMessage());
		ignored.printStackTrace();
		fail();
	}
	tx.commit();
}

public void testAddingWithIndex()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
	Transaction tx = odmg.newTransaction();
	tx.begin();
	DList list = odmg.newDList();
    PersistenceBroker broker = ((DatabaseImpl)OJB.getInstance().getDatabase(null)).getBroker();

    String name = null;
    try
    {
        name = "the_dlist_" + (broker.getUniqueId(ojb.odmg.collections.DListImpl.class, "id")-1);
    }
    catch(PersistenceBrokerException e)
    {
    }

    try
	{
		db.bind(list, name);
		tx.commit();
	}
	catch (Exception ignored)
	{
	}
	tx = odmg.newTransaction();
	tx.begin();
	for (int i = 0; i < 5; i++)
	{
		
		Article a = createArticle();
		list.add(a);
	}

	printDList(list);
	
	Article a = createArticle();
	list.add(2, a);
	printDList(list);
	
	a = createArticle();
	list.add(0, a);
	printDList(list);
	
	a = createArticle();
	list.add(7, a);
	printDList(list);

	
	
	tx.commit();
	System.out.println("sequence of items in list:");
	Iterator iter = list.iterator();
	while (iter.hasNext())
	{
		a = (Article) iter.next();
		System.out.print(a.getArticleId() + ", ");
	}
	System.out.println();
	tx = odmg.newTransaction();
	tx.begin();
	try
	{
		DList lookedUp = (DList) db.lookup(name);
		System.out.println("sequence of items in lookedup list:");
		iter = lookedUp.iterator();
		while (iter.hasNext())
		{
			a = (Article) iter.next();
			System.out.print(a.getArticleId() + ", ");
		}
		System.out.println();
	}
	catch (Throwable ignored)
	{
		System.out.println(ignored.getMessage());
		ignored.printStackTrace();
		fail();
	}
	tx.commit();
}

public void testDBag() throws Exception
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	db.open(databaseName, Database.OPEN_READ_WRITE);
	Transaction tx = odmg.newTransaction();
	tx.begin();
	DBag bag1 = odmg.newDBag();
	DBag bag2 = odmg.newDBag();
	Article a, b, c, d, e;
	a = createArticle();
	b = createArticle();
	c = createArticle();
	d = createArticle();
	e = createArticle();
	bag1.add(a);
	bag1.add(b);
	bag1.add(c);
	bag2.add(b);
	bag2.add(c);
	bag2.add(d);
	bag2.add(e);
	DBag bag3 = bag1.difference(bag2);
	assertEquals("should contain only 1 element", 1, bag3.size());

	bag3 = bag1.intersection(bag2);
	assertEquals("should contain two elements", 2, bag3.size());
	
	tx.commit();
}

public void testDSet() throws Exception
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	db.open(databaseName, Database.OPEN_READ_WRITE);
	Transaction tx = odmg.newTransaction();
	tx.begin();
	
	DSet set1 = odmg.newDSet();
	DSet set2 = odmg.newDSet();
	Article a, b, c, d, e;
	a = createArticle();
	b = createArticle();
	c = createArticle();
	d = createArticle();
	e = createArticle();
	set1.add(a);
	set1.add(b);
	set1.add(c);
	
	set2.add(b);
	set2.add(c);
	set2.add(d);
	set2.add(e);
	
	db.bind(set1, "set1");
	db.bind(set2, "set2");
	tx.commit();

	// low lookup both sets
	tx = odmg.newTransaction();
	tx.begin();
	DSet set1a = (DSet) db.lookup("set1");
	DSet set2a = (DSet) db.lookup("set2");

	// check looked up sets
	assert(set1a.containsAll(set1));
	assert(set2a.containsAll(set2));

	// now Test set operations:
	DSet set3 = set1.difference(set2);
	assertEquals(1, set3.size());

	set3 = set1.intersection(set2);
	assertEquals(2, set3.size());

	set3 = set1.union(set2);
	assertEquals(5, set3.size());

	assert(set1.properSubsetOf(set3));
	assert(set2.properSubsetOf(set3));

	assert(set3.properSupersetOf(set1));
	assert(set3.properSupersetOf(set2));

	assert(!set1.properSubsetOf(set2));
	
		
	
	tx.commit();
}
}