package test.ojb.odmg;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;


import ojb.odmg.*;
import org.odmg.*;
import junit.framework.*;
import ojb.broker.Identity;

/** Demo Application that shows basic concepts for Applications using the OJB ODMG
  * implementation as an transactional object server.
  */
public class NamedRoots extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	private static Class CLASS = NamedRoots.class;
	private String databaseName;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public NamedRoots(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
private void clearNRM()
{
	try
	{
		/*
		ojb.broker.metadata.ClassDescriptor cld = ojb.broker.metadata.DescriptorRepository.getInstance().getDescriptorFor(NamedRootsEntry.class);

		java.sql.Statement stmt = ojb.broker.accesslayer.StatementManager.getGenericStatement(cld);
		String nrmTable = cld.getTableName();
		String scheme = cld.getScheme();
		if (scheme != null)
		{	nrmTable = scheme + "." + nrmTable;
		}
		
		stmt.executeUpdate("DELETE FROM " + nrmTable);
		 */
		
	}
	catch (Throwable t)
	{
	}
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
	databaseName = "repository.xml";


}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{
	try
	{
		OJB.getInstance().currentTransaction().abort();
		OJB.getInstance().getDatabase(null).close();
		databaseName = null;

		clearNRM();
		
	}
	catch (Throwable t)
	{
	}
}

public void testBind()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();

	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}

	// clean up roots map
	clearNRM();
	
	Transaction tx = odmg.newTransaction();
	tx.begin();

	Article example = new Article();
	example.setArticleId(7);
	example.setArticleName("named roots article");

	String name = "wild thing";

	try
	{
		db.bind(example, name);
		tx.commit();
	}
	catch (Throwable ex)
	{
		tx.abort();
		fail("should not have happened: " + ex.getMessage());
	}

	tx = odmg.newTransaction();
	tx.begin();
	try
	{
		Article value = (Article) db.lookup(name);
		tx.commit();
	}
	catch (ObjectNameNotFoundException ex)
	{
		tx.abort();
		fail("name " + name + " should not be unknown");
	}

	try
	{
		tx.begin();
		db.bind(example, name);
		tx.commit();
	}
	catch (ObjectNameNotUniqueException ex)
	{
		tx.abort();
		// close database
		try
		{
			db.close();
		}
		catch (ODMGException e)
		{
			fail("ODMGException: " + e.getMessage());
		}

		return;
	}


	// the following code will only be reached on failure
	
	// close database
	try
	{
		db.close();
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}

	fail("should throw an ObjectNameNotUniqueException");

}

public void testLookup()
{

	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();

	
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}

	// clear named roots.
	clearNRM();

	Transaction tx = odmg.newTransaction();

	//bind object to name
	tx.begin();

	Article example = new Article();
	example.setArticleId(7);
	example.setArticleName("named roots article");
	Identity oid = new Identity(example);
	String name = "wild thing 12";

	try
	{
		db.bind(example, name);
		tx.commit();
	}
	catch (ObjectNameNotUniqueException ex)
	{
		tx.abort();
		fail(ex.getMessage());
	}

	// Test look up
	Article lookedUp1 = null, lookedUp2 = null;
	tx = odmg.newTransaction();
	tx.begin();
	
	try
	{
		// lookup by name binding
		lookedUp1 = (Article) db.lookup(name);
	}
	catch (ObjectNameNotFoundException ex)
	{
		fail("lookup by name: " + name + " should not be unknown");
	}

	tx.commit();

	// looking up object by OID should return same Object as by name
	assertEquals("lookups should return identical object", example, lookedUp1);

	// close database
	try
	{
		db.close();
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
}

public void testUnBind()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();

	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}

	// clear named roots.
	clearNRM();

	Transaction tx = odmg.newTransaction();
	// 1. perform binding
	tx.begin();

	String name = "wild thing 777";
	Article example = new Article();
	example.setArticleId(7);
	example.setArticleName("named roots article");

	try
	{
		db.bind(example, name);
		tx.commit();
	}
	catch (ObjectNameNotUniqueException ex)
	{
		tx.abort();
		fail(ex.getMessage());
	}

	// 2. perform unbind
	tx = odmg.newTransaction();
	tx.begin();
	try
	{
		db.unbind(name);
		tx.commit();
	}
	catch (ObjectNameNotFoundException ex)
	{
		tx.abort();
		fail("name " + name + "should be known");
	}

	// 3. check if name is really unknown now
	tx = odmg.newTransaction();
	tx.begin();
	try
	{
		Article value = (Article) db.lookup(name);
	}
	catch (ObjectNameNotFoundException ex)
	{
		// OK !
		tx.commit();
		// close database
		try
		{
			db.close();
		}
		catch (ODMGException e)
		{
			fail("ODMGException: " + e.getMessage());
		}

		return;
	}

	// NOT OK!
	tx.abort();
	fail("name " + name + " should not be known after unbind");

}
}