package test.ojb.odmg;




import java.util.Vector;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;


import ojb.odmg.states.*;
import ojb.odmg.*;
import org.odmg.*;
import junit.framework.*;
import ojb.broker.accesslayer.ObjectCache;
import ojb.broker.Identity;

/** Demo Application that shows basic concepts for Applications using the OJB ODMG
  * implementation as an transactional object server.
  */
public class OdmgExamples extends TestCase {
public static void main(String[] args)
{
	String[] arr = {CLASS.getName()};
	junit.textui.TestRunner.main(arr);
}

	private static Class CLASS = OdmgExamples.class;
	private String databaseName;

/**
 * Insert the method's description here.
 * Creation date: (24.12.2000 00:33:40)
 */
public OdmgExamples(String name)
{
	super(name);
}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:58:53)
 */
public void setUp()
{
	databaseName = "repository.xml";


}

/**
 * Insert the method's description here.
 * Creation date: (06.12.2000 21:59:14)
 */
public void tearDown()
{
	databaseName = null;
}

/**Test state transition of modification states*/
public void testModificationStates()
{
	ModificationState oldState = StateNewClean.getInstance();
	ModificationState newState = oldState.markDirty();
	assertEquals(StateNewDirty.getInstance(), newState);

	oldState = newState;
	newState = oldState.markDirty();
	assertEquals(oldState, newState);

	
	
}

public void testOdmgSession()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
	Transaction tx = odmg.newTransaction();

	//perform transaction
	try
	{
		tx.begin();

		Article example = new Article();
		example.setArticleId(777);
		db.makePersistent(example);



		// modify Object
		example.setStock(333);
		example.addToStock(47);
		example.addToStock(7);
		example.addToStock(4);


		System.out.println("now commit all changes...");
		tx.commit();
	}
	catch (Exception ex)
	{
		tx.abort();
	}

	// close database	
	try
	{
		db.close();
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
}

public void testOQLQuery()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
	Transaction tx = odmg.newTransaction();

	//perform transaction
	try
	{
		tx.begin();


		OQLQuery query = odmg.newOQLQuery();
		query.create("select anArticle from " + Article.class.getName() + " where articleId = 7");
        List results = (List) query.execute();

        Article a = (Article) results.get(0);


        // cross check with PersistenceBroker lookup
		// 1. get OID
		Article example = new Article();
		example.setArticleId(7);
		Identity oid = new Identity(example);
		// 2. lookup object by OID
		ObjectCache.getInstance().clear();
		Article b = (Article) ((DatabaseImpl) db).getBroker().getObjectByIdentity(oid);

	    assertEquals("should be same object", a, b);

		System.out.println("now commit all changes...");
		tx.commit();
	}
	catch (Exception ex)
	{
		tx.abort();
		fail("ODMGException: " + ex.getMessage());
	}

	// close database
	try
	{
		db.close();
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
}

public void testOQLQueryBind()
{
	// get facade instance
	Implementation odmg = OJB.getInstance();
	Database db = odmg.newDatabase();
	//open database
	try
	{
		db.open(databaseName, Database.OPEN_READ_WRITE);
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
	Transaction tx = odmg.newTransaction();

	//perform transaction
	try
	{
		tx.begin();


		OQLQuery query = odmg.newOQLQuery();
		query.create("select anArticle from " + Article.class.getName() + " where articleId = $678");
        query.bind(new Integer(7));

        List results = (List) query.execute();

        Article a = (Article) results.get(0);

        //crosscheck with PersistenceBroker lookup
		// 1. get OID
		Article example = new Article();
		example.setArticleId(7);
		Identity oid = new Identity(example);

		ObjectCache.getInstance().clear();
		// 2. lookup object by OID
		Article b = (Article) ((DatabaseImpl) db).getBroker().getObjectByIdentity(oid);

	    assertEquals("should be same object", a, b);

		System.out.println("now commit all changes...");
		tx.commit();
	}
	catch (Exception ex)
	{
		tx.abort();
		fail("ODMGException: " + ex.getMessage());
	}

	// close database
	try
	{
		db.close();
	}
	catch (ODMGException ex)
	{
		fail("ODMGException: " + ex.getMessage());
	}
}

public void testOQLQueryOnCollections()
{
    // get facade instance
    Implementation odmg = OJB.getInstance();
    Database db = odmg.newDatabase();
    //open database
    try
    {
       db.open(databaseName, Database.OPEN_READ_WRITE);
    }
    catch (ODMGException ex)
    {
        fail("ODMGException: " + ex.getMessage());
    }
    Transaction tx = odmg.newTransaction();

    //perform transaction
    try
    {
        tx.begin();
        OQLQuery query = odmg.newOQLQuery();
        query.create("select aLotOfArticles from " + Article.class.getName() + " where productGroupId = 4");

        DCollection results = (DCollection) query.execute();
        results = results.query("price > 35");

        // now perform control query
        query = odmg.newOQLQuery();
        query.create("select aLotOfArticles from " + Article.class.getName() + " where productGroupId = 4 and price  > 35");

        DCollection check = (DCollection) query.execute();

        assertEquals(results,check);


       tx.commit();
    }
    catch (Exception ex)
    {
       tx.abort();
       ex.printStackTrace();
       fail("ODMGException: " + ex.getMessage());
    }

    // close database
    try
    {
       db.close();
    }
    catch (ODMGException ex)
    {
        fail("ODMGException: " + ex.getMessage());
    }
}


/**try to open non-existing db*/
public void testWrongDbName()
{
	// get facade instance
	OJB objectserver = OJB.getInstance();
	Database db = objectserver.newDatabase();
	
	//try open database with non existing repository file:
	String wrongDatabaseName = "ThereIsNoSuchFile"; 
	try
	{
		db.open(wrongDatabaseName, Database.OPEN_READ_WRITE);
		fail("should not be able to open database "+ wrongDatabaseName); 
	}
	catch (ODMGException ex)
	{
		return;
	}
}
}
