package test.ojb.server;

/**
 * Title:        ObJectBridge
 * Description:  Object Relational Mapping tool
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author Thomas Mahler & David Dixon-Peugh
 * @version 1.0
 */
import junit.framework.*;
import ojb.server.*;
import ojb.server.lockstrategy.*;

import org.odmg.Transaction;
public class ReaderWriterLockTest extends TestCase {
  TransactionTable tbl;
  Transaction t1;
  Transaction t2;
  Transaction t3;

  public ReaderWriterLockTest( String name ) {
    super( name );
  }

  public void setUp() {
    tbl = new TransactionTable();
    t1 = new TransactionImpl( tbl );
    t2 = new TransactionImpl( tbl );
    t3 = new TransactionImpl( tbl );
  }

  public void testReadLock() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
  }

  public void testMultipleReadLock() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( rwl.read( t2 ));
  }

  public void testSingleWriteLock() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.write( t1 ));
  }

  public void testMultipleWriteLock() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.write( t1 ));
    assert( !rwl.write( t2 ));
  }

  public void testWriteWithExistingReader() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( !rwl.write( t2 ));
  }

  public void testUpgradeLock() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( rwl.upgrade( t1 ));
  }

  public void testUpgradeOnNonReader() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( !rwl.upgrade( t2 ));
  }

  public void testUpgradeWithMultipleReaders() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( rwl.read( t2 ));
    assert( !rwl.upgrade( t2 ));
  }

  public void testReaderRelease() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( !rwl.release( t1 ));
  }

  public void testReaderReleaseThenWrite() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( rwl.release( t1 ));
    assert( rwl.write( t2 ));
  }

  public void testReaderReleaseThenUpgrade() {
    ReaderWriterLock rwl = RWLockFactory.getLock( ReaderWriterLock.RW_SERIALIZABLE );
    assert( rwl.read( t1 ));
    assert( rwl.release( t1 ));
    assert( !rwl.upgrade( t1 ));
  }
}