package test.ojb.server;

/**
 * Title:        ObJectBridge
 * Description:  Object Relational Mapping tool
 * Copyright:    Copyright (c) 2000
 * Company:
 * @author Thomas Mahler & David Dixon-Peugh
 * @version 1.0
 */

import java.util.*;

import junit.framework.Test;
import junit.framework.TestResult;

public class StateMachineTest implements Test{
  private StateMachine impl;
  private StateMachineModel model;
  private int testCount = 0;
  private int numPass = 0;

  private boolean [][]visited;
  private int []modelVisits;
  private int []implVisits;

  public StateMachineTest() {
  }

  public StateMachineTest( String URI, String className ) {
    try {
      Class implClass = Class.forName(className);
      setModel( URI );
      setImplementation( (StateMachine) implClass.newInstance() );
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public StateMachineTest( String URI, StateMachine newImpl ) {
    setModel( URI );
    setImplementation( newImpl );
  }


  public void setImplementation( StateMachine newImpl ) {
    impl = newImpl;
  }

  public void setModel( StateMachineModel newModel ) {
    model = newModel;
  }

  public void setModel( String URI ) {
    model = new StateMachineModel();
    model.load(URI);
  }

  public void run(TestResult result) {
    Set stateSpace  = model.getStateSpace();
    Set actionSpace = model.getActionSpace();

    Iterator states = stateSpace.iterator();
    while (states.hasNext()) {
      Iterator trans = actionSpace.iterator();
      ModelState s = (ModelState) states.next();

      while (trans.hasNext()) {
        result.startTest(this);
        String t = (String) trans.next();

        // Restart the Implementation, and move it into the
        // testing state.
        impl.start();

        Iterator path = model.getPath( s ).iterator();
        while (path.hasNext()) {
          ModelTransition t1 = (ModelTransition) path.next();
          impl.transition( t1.getName() );
        }

        // impl should be in state S now.  Now, we should
        // take the Transition, and see if the results match.
        testCount ++;

        if (!impl.getState().equals(s.getID())) {
          result.addError(this, new TestStateUnreachable( "Unreachable: " + s.getID() ));
        } else {
          ModelTransition mt = s.getTransition(t);

          if ( mt != null ) {
            if (   impl.transition( t )
                && impl.getState().equals( mt.getTarget()) ) {
              numPass++;
            } else {
              result.addError(this, new TransitionDoesntMatch( "Transition doesn't match: " +
                                                               " FROM: " + s.getID() +
                                                               " BY: " + t +
                                                               " TO: " + mt.getTarget() +
                                                               " RESULT: " + impl.getState() ));
            }
          } else {
            if ( !impl.transition( t ) ) {
              numPass++;
            } else {
              result.addError(this, new TransitionDoesntMatch( "Transition doesn't match: " +
                                                               " FROM: " + s.getID() +
                                                               " BY: " + t +
                                                               " TO: UNDEFINED" +
                                                               " RESULT: " + impl.getState() ));
            }
          }
        }
        result.endTest(this);
      }
    }
  }

  public int countTestCases() {
    return model.getStateSpace().size() * model.getActionSpace().size();
  }

  public static void main(String args[]) {
    try {
      StateMachineTest smt = new StateMachineTest( args[0], args[1] );

      TestResult tr = new TestResult();
      tr.startTest( smt );

      smt.run(tr);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}